@@ -38,41 +38,6 @@
 const std::string Imager::DEVICE_NAME = "Imager Agent";
 std::shared_ptr<Imager> imager(new Imager());
 
-
-
-// Driver entry points ----------------------------------------------------------------------------
-
-void ISGetProperties(const char *dev)
-{
-    imager->ISGetProperties(dev);
-}
-
-void ISNewSwitch(const char *dev, const char *name, ISState *states, char *names[], int n)
-{
-    imager->ISNewSwitch(dev, name, states, names, n);
-}
-
-void ISNewText(const char *dev, const char *name, char *texts[], char *names[], int n)
-{
-    imager->ISNewText(dev, name, texts, names, n);
-}
-
-void ISNewNumber(const char *dev, const char *name, double values[], char *names[], int n)
-{
-    imager->ISNewNumber(dev, name, values, names, n);
-}
-
-void ISNewBLOB(const char *dev, const char *name, int sizes[], int blobsizes[], char *blobs[], char *formats[],
-               char *names[], int n)
-{
-    imager->ISNewBLOB(dev, name, sizes, blobsizes, blobs, formats, names, n);
-}
-
-void ISSnoopDevice(XMLEle *root)
-{
-    imager->ISSnoopDevice(root);
-}
-
 // Imager ----------------------------------------------------------------------------
 
 Imager::Imager()
@@ -162,7 +127,8 @@ void Imager::initiateNextCapture()
                 IDSetNumber(&ProgressNP, "CCD is not connected");
                 return;
             }
-            CCDImageBinN[0].value = CCDImageBinN[1].value = currentGroup()->binning();
+            CCDImageBinN[0].value = currentGroup()->binning();
+            CCDImageBinN[1].value = currentGroup()->binning();
             sendNewNumber(&CCDImageBinNP);
             CCDImageExposureN[0].value = currentGroup()->exposure();
             sendNewNumber(&CCDImageExposureNP);
@@ -294,9 +260,9 @@ bool Imager::initProperties()
     IUFillBLOB(&FitsB[0], "IMAGE", "Image", "");
     IUFillBLOBVector(&FitsBP, FitsB, 1, getDefaultName(), "IMAGE", "Image Data", DOWNLOAD_TAB, IP_RO, 60, IPS_IDLE);
 
-    defineNumber(&GroupCountNP);
-    defineText(&ControlledDeviceTP);
-    defineText(&ImageNameTP);
+    defineProperty(&GroupCountNP);
+    defineProperty(&ControlledDeviceTP);
+    defineProperty(&ImageNameTP);
 
     for (int i = 0; i < GroupCountN[0].value; i++)
     {
@@ -334,19 +300,19 @@ bool Imager::updateProperties()
 {
     if (isConnected())
     {
-        defineLight(&StatusLP);
+        defineProperty(&StatusLP);
         ProgressN[0].value = group = 0;
         ProgressN[1].value = image = 0;
         ProgressNP.s               = IPS_IDLE;
-        defineNumber(&ProgressNP);
+        defineProperty(&ProgressNP);
         BatchSP.s = IPS_IDLE;
-        defineSwitch(&BatchSP);
+        defineProperty(&BatchSP);
         DownloadN[0].value = 0;
         DownloadN[1].value = 0;
         DownloadNP.s       = IPS_IDLE;
-        defineNumber(&DownloadNP);
+        defineProperty(&DownloadNP);
         FitsBP.s = IPS_IDLE;
-        defineBLOB(&FitsBP);
+        defineProperty(&FitsBP);
     }
     else
     {
