@@ -37,37 +37,6 @@ static pthread_mutex_t condMutex = PTHREAD_MUTEX_INITIALIZER;
 
 static std::unique_ptr<CCDSim> ccdsim(new CCDSim());
 
-void ISGetProperties(const char * dev)
-{
-    ccdsim->ISGetProperties(dev);
-}
-
-void ISNewSwitch(const char * dev, const char * name, ISState * states, char * names[], int n)
-{
-    ccdsim->ISNewSwitch(dev, name, states, names, n);
-}
-
-void ISNewText(const char * dev, const char * name, char * texts[], char * names[], int n)
-{
-    ccdsim->ISNewText(dev, name, texts, names, n);
-}
-
-void ISNewNumber(const char * dev, const char * name, double values[], char * names[], int n)
-{
-    ccdsim->ISNewNumber(dev, name, values, names, n);
-}
-
-void ISNewBLOB(const char * dev, const char * name, int sizes[], int blobsizes[], char * blobs[], char * formats[],
-               char * names[], int n)
-{
-    ccdsim->ISNewBLOB(dev, name, sizes, blobsizes, blobs, formats, names, n);
-}
-
-void ISSnoopDevice(XMLEle * root)
-{
-    ccdsim->ISSnoopDevice(root);
-}
-
 CCDSim::CCDSim() : INDI::FilterInterface(this)
 {
     currentRA  = RA;
@@ -123,7 +92,7 @@ bool CCDSim::Connect()
     streamPredicate = 0;
     terminateThread = false;
     pthread_create(&primary_thread, nullptr, &streamVideoHelper, this);
-    SetTimer(POLLMS);
+    SetTimer(getCurrentPollingPeriod());
     return true;
 }
 
@@ -169,8 +138,8 @@ bool CCDSim::initProperties()
 
     // RGB Simulation
     IUFillSwitch(&SimulateBayerS[INDI_ENABLED], "INDI_ENABLED", "Enabled", ISS_OFF);
-    IUFillSwitch(&SimulateBayerS[INDI_DISABLED], "INDI_DISABLED", "Dsiabled", ISS_ON);
-    IUFillSwitchVector(&SimulateRgbSP, SimulateBayerS, 2, getDeviceName(), "SIMULATE_BAYER", "Bayer", SIMULATOR_TAB, IP_RW,
+    IUFillSwitch(&SimulateBayerS[INDI_DISABLED], "INDI_DISABLED", "Disabled", ISS_ON);
+    IUFillSwitchVector(&SimulateBayerSP, SimulateBayerS, 2, getDeviceName(), "SIMULATE_BAYER", "Bayer", SIMULATOR_TAB, IP_RW,
                        ISR_1OFMANY, 60, IPS_IDLE);
 
     // Simulate focusing
@@ -202,7 +171,7 @@ bool CCDSim::initProperties()
                        ISR_1OFMANY, 0, IPS_IDLE);
 
     // Gain
-    IUFillNumber(&GainN[0], "GAIN", "value", "%.f", 0, 100, 10, 50);
+    IUFillNumber(&GainN[0], "GAIN", "value", "%.f", 0, 100, 10, 90);
     IUFillNumberVector(&GainNP, GainN, 1, getDeviceName(), "CCD_GAIN", "Gain", MAIN_CONTROL_TAB, IP_RW, 60, IPS_IDLE);
 
     // Offset
@@ -287,11 +256,11 @@ void CCDSim::ISGetProperties(const char * dev)
 {
     INDI::CCD::ISGetProperties(dev);
 
-    defineNumber(&SimulatorSettingsNP);
-    defineNumber(&EqPENP);
-    defineNumber(&FocusSimulationNP);
-    defineSwitch(&SimulateRgbSP);
-    defineSwitch(&CrashSP);
+    defineProperty(&SimulatorSettingsNP);
+    defineProperty(&EqPENP);
+    defineProperty(&FocusSimulationNP);
+    defineProperty(&SimulateBayerSP);
+    defineProperty(&CrashSP);
 }
 
 bool CCDSim::updateProperties()
@@ -301,13 +270,13 @@ bool CCDSim::updateProperties()
     if (isConnected())
     {
         if (HasCooler())
-            defineSwitch(&CoolerSP);
+            defineProperty(&CoolerSP);
 
-        defineNumber(&GainNP);
-        defineNumber(&OffsetNP);
+        defineProperty(&GainNP);
+        defineProperty(&OffsetNP);
 
-        defineText(&DirectoryTP);
-        defineSwitch(&DirectorySP);
+        defineProperty(&DirectoryTP);
+        defineProperty(&DirectorySP);
 
         setupParameters();
 
@@ -432,7 +401,7 @@ float CCDSim::CalcTimeLeft(timeval start, float req)
 
 void CCDSim::TimerHit()
 {
-    uint32_t nextTimer = POLLMS;
+    uint32_t nextTimer = getCurrentPollingPeriod();
 
     //  No need to reset timer if we are not connected anymore
     if (!isConnected())
@@ -517,21 +486,16 @@ void CCDSim::TimerHit()
 
     if (TemperatureNP.s == IPS_BUSY)
     {
-        if (fabs(TemperatureRequest - TemperatureN[0].value) <= 0.5)
-        {
-            LOGF_INFO("Temperature reached requested value %.2f degrees C", TemperatureRequest);
-            TemperatureN[0].value = TemperatureRequest;
-            TemperatureNP.s       = IPS_OK;
-        }
+        if (TemperatureRequest < TemperatureN[0].value)
+            TemperatureN[0].value = std::max(TemperatureRequest, TemperatureN[0].value - 0.5);
         else
-        {
-            if (TemperatureRequest < TemperatureN[0].value)
-                TemperatureN[0].value -= 0.5;
-            else
-                TemperatureN[0].value += 0.5;
-        }
+            TemperatureN[0].value = std::min(TemperatureRequest, TemperatureN[0].value + 0.5);
 
-        IDSetNumber(&TemperatureNP, nullptr);
+        if (std::abs(TemperatureN[0].value - m_LastTemperature) > 0.1)
+        {
+            m_LastTemperature = TemperatureN[0].value;
+            IDSetNumber(&TemperatureNP, nullptr);
+        }
 
         // Above 20, cooler is off
         if (TemperatureN[0].value >= 20)
@@ -547,6 +511,15 @@ void CCDSim::TimerHit()
     SetTimer(nextTimer);
 }
 
+double CCDSim::flux(double mag) const
+{
+    // The limiting magnitude provides zero ADU whatever the exposure
+    // The saturation magnitude provides max ADU in one second
+    double const z = m_LimitingMag;
+    double const k = 2.5 * log10(m_MaxVal) / (m_LimitingMag - m_SaturationMag);
+    return pow(10, (z - mag) * k / 2.5);
+}
+
 int CCDSim::DrawCcdFrame(INDI::CCDChip * targetChip)
 {
     //  CCD frame is 16 bit data
@@ -667,22 +640,22 @@ int CCDSim::DrawCcdFrame(INDI::CCDChip * targetChip)
             currentRA  = RA;
             currentDE = Dec;
 
-            ln_equ_posn epochPos { 0, 0 }, J2000Pos { 0, 0 };
+            INDI::IEquatorialCoordinates epochPos { 0, 0 }, J2000Pos { 0, 0 };
 
             double jd = ln_get_julian_from_sys();
 
-            epochPos.ra  = currentRA * 15.0;
-            epochPos.dec = currentDE;
+            epochPos.rightascension  = currentRA;
+            epochPos.declination = currentDE;
 
             // Convert from JNow to J2000
-            LibAstro::ObservedToJ2000(&epochPos, jd, &J2000Pos);
+            INDI::ObservedToJ2000(&epochPos, jd, &J2000Pos);
 
-            currentRA  = J2000Pos.ra / 15.0;
-            currentDE = J2000Pos.dec;
+            currentRA  = J2000Pos.rightascension;
+            currentDE = J2000Pos.declination;
 
             //LOGF_DEBUG("DrawCcdFrame JNow %f, %f J2000 %f, %f", epochPos.ra, epochPos.dec, J2000Pos.ra, J2000Pos.dec);
-            //ln_equ_posn jnpos;
-            //LibAstro::J2000toObserved(&J2000Pos, jd, &jnpos);
+            //INDI::IEquatorialCoordinates jnpos;
+            //INDI::J2000toObserved(&J2000Pos, jd, &jnpos);
             //LOGF_DEBUG("J2000toObserved JNow %f, %f J2000 %f, %f", jnpos.ra, jnpos.dec, J2000Pos.ra, J2000Pos.dec);
 
             currentDE += guideNSOffset;
@@ -718,8 +691,8 @@ int CCDSim::DrawCcdFrame(INDI::CCDChip * targetChip)
         //  and a limitingmag produces a one adu level in one second
         //  solve for zero point and system gain
 
-        k = (m_SaturationMag - m_LimitingMag) / ((-2.5 * log(m_MaxVal)) - (-2.5 * log(1.0 / 2.0)));
-        z = m_SaturationMag - k * (-2.5 * log(m_MaxVal));
+        //k = (m_SaturationMag - m_LimitingMag) / ((-2.5 * log(m_MaxVal)) - (-2.5 * log(1.0 / 2.0)));
+        //z = m_SaturationMag - k * (-2.5 * log(m_MaxVal));
 
         //  Should probably do some math here to figure out the dimmest
         //  star we can see on this exposure
@@ -838,7 +811,6 @@ int CCDSim::DrawCcdFrame(INDI::CCDChip * targetChip)
 
         if (ftype == INDI::CCDChip::LIGHT_FRAME || ftype == INDI::CCDChip::FLAT_FRAME)
         {
-            float skyflux;
             //  calculate flux from our zero point and gain values
             float glow = m_SkyGlow;
 
@@ -852,10 +824,8 @@ int CCDSim::DrawCcdFrame(INDI::CCDChip * targetChip)
 
             //fprintf(stderr,"Using glow %4.2f\n",glow);
 
-            skyflux = pow(10, ((glow - z) * k / -2.5));
-            //  ok, flux represents one second now
-            //  scale up linearly for exposure time
-            skyflux = skyflux * exposure_time;
+            // Flux represents one second, scale up linearly for exposure time
+            float const skyflux = flux(glow) * exposure_time;
 
             uint16_t * pt = reinterpret_cast<uint16_t *>(targetChip->getFrameBuffer());
 
@@ -864,43 +834,31 @@ int CCDSim::DrawCcdFrame(INDI::CCDChip * targetChip)
 
             for (int y = 0; y < nheight; y++)
             {
+                float const sy = nheight / 2 - y;
+
                 for (int x = 0; x < nwidth; x++)
                 {
-                    float dc; //  distance from center
-                    float fp; //  flux this pixel;
-                    float sx, sy;
-                    float vig;
+                    float const sx = nwidth / 2 - x;
 
-                    sx = nwidth / 2 - x;
-                    sy = nheight / 2 - y;
+                    // Vignetting parameter in arcsec
+                    float const vig = std::min(nwidth, nheight) * ImageScalex;
 
-                    vig = nwidth;
-                    vig = vig * ImageScalex;
-                    //  need to make this account for actual pixel size
-                    dc = std::sqrt(sx * sx * ImageScalex * ImageScalex + sy * sy * ImageScaley * ImageScaley);
-                    //  now we have the distance from center, in arcseconds
-                    //  now lets plot a gaussian falloff to the edges
-                    //
-                    float fa;
-                    fa = exp(-2.0 * 0.7 * (dc * dc) / vig / vig);
+                    // Squared distance to center in arcsec (need to make this account for actual pixel size)
+                    float const dc2 = sx * sx * ImageScalex * ImageScalex + sy * sy * ImageScaley * ImageScaley;
 
-                    //  get the current value
-                    fp = pt[0];
+                    // Gaussian falloff to the edges of the frame
+                    float const fa = exp(-2.0 * 0.7 * dc2 / (vig * vig));
 
-                    //  Add the sky glow
-                    fp += skyflux;
+                    // Get the current value of the pixel, add the sky glow and scale for vignetting
+                    float fp = (pt[0] + skyflux) * fa;
 
-                    //  now scale it for the vignetting
-                    fp = fa * fp;
+                    // Clamp to limits, store minmax
+                    if (fp > m_MaxVal) fp = m_MaxVal;
+                    if (fp < pt[0]) fp = pt[0];
+                    if (fp > maxpix) maxpix = fp;
+                    if (fp < minpix) minpix = fp;
 
-                    //  clamp to limits
-                    if (fp > m_MaxVal)
-                        fp = m_MaxVal;
-                    if (fp > maxpix)
-                        maxpix = fp;
-                    if (fp < minpix)
-                        minpix = fp;
-                    //  and put it back
+                    // And put it back
                     pt[0] = fp;
                     pt++;
                 }
@@ -967,7 +925,7 @@ int CCDSim::DrawImageStar(INDI::CCDChip * targetChip, float mag, float x, float
     }
 
     //  calculate flux from our zero point and gain values
-    flux = pow(10, ((mag - z) * k / -2.5));
+    flux = this->flux(mag);
 
     //  ok, flux represents one second now
     //  scale up linearly for exposure time
@@ -991,17 +949,18 @@ int CCDSim::DrawImageStar(INDI::CCDChip * targetChip, float mag, float x, float
         for (sx = -boxsizey; sx <= boxsizey; sx++)
         {
             int rc;
-            float dc; //  distance from center
-            float fp; //  flux this pixel;
 
-            //  need to make this account for actual pixel size
-            dc = std::sqrt(sx * sx * ImageScalex * ImageScalex + sy * sy * ImageScaley * ImageScaley);
-            //  now we have the distance from center, in arcseconds
-            //  This should be gaussian, but, for now we'll just go with
-            //  a simple linear function
-            float fa = exp(-2.0 * 0.7 * (dc * dc) / seeing / seeing);
+            // Squared distance to center in arcsec (need to make this account for actual pixel size)
+            float const dc2 = sx * sx * ImageScalex * ImageScalex + sy * sy * ImageScaley * ImageScaley;
 
-            fp = fa * flux;
+            // Use a gaussian of unitary integral, scale it with the source flux
+            // f(x) = 1/(sqrt(2*pi)*sigma) * exp( -x² / (2*sigma²) )
+            // FWHM = 2*sqrt(2*log(2))*sigma => sigma = seeing/(2*sqrt(2*log(2)))
+            float const sigma = seeing / ( 2 * sqrt(2 * log(2)));
+            float const fa = 1 / (sigma * sqrt(2 * 3.1416)) * exp( -dc2 / (2 * sigma * sigma));
+
+            // The source contribution is the gaussian value, stretched by seeing/FWHM
+            float fp = fa * flux;
 
             if (fp < 0)
                 fp = 0;
@@ -1141,6 +1100,7 @@ bool CCDSim::ISNewNumber(const char * dev, const char * name, double values[], c
             //  Reset our parameters now
             setupParameters();
             IDSetNumber(&SimulatorSettingsNP, nullptr);
+            saveConfig(true, SimulatorSettingsNP.name);
             return true;
         }
         // Record PE EQ to simulate different position in the sky than actual mount coordinate
@@ -1150,17 +1110,16 @@ bool CCDSim::ISNewNumber(const char * dev, const char * name, double values[], c
             IUUpdateNumber(&EqPENP, values, names, n);
             EqPENP.s = IPS_OK;
 
-            ln_equ_posn epochPos { 0, 0 }, J2000Pos { 0, 0 };
-            epochPos.ra  = EqPEN[AXIS_RA].value * 15.0;
-            epochPos.dec = EqPEN[AXIS_DE].value;
+            INDI::IEquatorialCoordinates epochPos { 0, 0 }, J2000Pos { 0, 0 };
+            epochPos.rightascension  = EqPEN[AXIS_RA].value;
+            epochPos.declination = EqPEN[AXIS_DE].value;
 
             RA = EqPEN[AXIS_RA].value;
             Dec = EqPEN[AXIS_DE].value;
 
-            LibAstro::ObservedToJ2000(&epochPos, ln_get_julian_from_sys(), &J2000Pos);
-            //ln_get_equ_prec2(&epochPos, ln_get_julian_from_sys(), JD2000, &J2000Pos);
-            currentRA  = J2000Pos.ra / 15.0;
-            currentDE = J2000Pos.dec;
+            INDI::ObservedToJ2000(&epochPos, ln_get_julian_from_sys(), &J2000Pos);
+            currentRA  = J2000Pos.rightascension;
+            currentDE = J2000Pos.declination;
             usePE = true;
 
             IDSetNumber(&EqPENP, nullptr);
@@ -1188,15 +1147,15 @@ bool CCDSim::ISNewSwitch(const char * dev, const char * name, ISState * states,
     if (dev != nullptr && strcmp(dev, getDeviceName()) == 0)
     {
         // Simulate RGB
-        if (!strcmp(name, SimulateRgbSP.name))
+        if (!strcmp(name, SimulateBayerSP.name))
         {
-            IUUpdateSwitch(&SimulateRgbSP, states, names, n);
-            int index = IUFindOnSwitchIndex(&SimulateRgbSP);
+            IUUpdateSwitch(&SimulateBayerSP, states, names, n);
+            int index = IUFindOnSwitchIndex(&SimulateBayerSP);
             if (index == -1)
             {
-                SimulateRgbSP.s = IPS_ALERT;
+                SimulateBayerSP.s = IPS_ALERT;
                 LOG_INFO("Cannot determine whether RGB simulation should be switched on or off.");
-                IDSetSwitch(&SimulateRgbSP, nullptr);
+                IDSetSwitch(&SimulateBayerSP, nullptr);
                 return false;
             }
 
@@ -1205,8 +1164,8 @@ bool CCDSim::ISNewSwitch(const char * dev, const char * name, ISState * states,
 
             SimulateBayerS[INDI_ENABLED].s = m_SimulateBayer ? ISS_ON : ISS_OFF;
             SimulateBayerS[INDI_DISABLED].s = m_SimulateBayer ? ISS_OFF : ISS_ON;
-            SimulateRgbSP.s   = IPS_OK;
-            IDSetSwitch(&SimulateRgbSP, nullptr);
+            SimulateBayerSP.s   = IPS_OK;
+            IDSetSwitch(&SimulateBayerSP, nullptr);
 
             return true;
         }
@@ -1218,9 +1177,11 @@ bool CCDSim::ISNewSwitch(const char * dev, const char * name, ISState * states,
                 CoolerSP.s = IPS_BUSY;
             else
             {
-                CoolerSP.s         = IPS_IDLE;
-                TemperatureRequest = 20;
-                TemperatureNP.s    = IPS_BUSY;
+                CoolerSP.s          = IPS_IDLE;
+                m_TargetTemperature = 20;
+                TemperatureNP.s     = IPS_BUSY;
+                m_TemperatureCheckTimer.start();
+                m_TemperatureElapsedTimer.start();
             }
 
             IDSetSwitch(&CoolerSP, nullptr);
@@ -1235,6 +1196,15 @@ bool CCDSim::ISNewSwitch(const char * dev, const char * name, ISState * states,
             if (DirectoryS[INDI_ENABLED].s == ISS_ON)
             {
                 DIR* dirp = opendir(DirectoryT[0].text);
+                if (dirp == nullptr)
+                {
+                    DirectoryS[INDI_ENABLED].s = ISS_OFF;
+                    DirectoryS[INDI_DISABLED].s = ISS_ON;
+                    DirectorySP.s = IPS_ALERT;
+                    LOGF_ERROR("Cannot monitor invalid directory %s", DirectoryT[0].text);
+                    IDSetSwitch(&DirectorySP, nullptr);
+                    return true;
+                }
                 struct dirent * dp;
                 std::string d_dir = std::string(DirectoryT[0].text);
                 if (DirectoryT[0].text[strlen(DirectoryT[0].text) - 1] != '/')
@@ -1352,7 +1322,7 @@ bool CCDSim::ISSnoopDevice(XMLEle * root)
 
         if (rc_ra == 0 && rc_de == 0 && ((newra != raPE) || (newdec != decPE)))
         {
-            ln_equ_posn epochPos { 0, 0 }, J2000Pos { 0, 0 };
+            INDI::IEquatorialCoordinates epochPos { 0, 0 }, J2000Pos { 0, 0 };
             epochPos.ra  = newra * 15.0;
             epochPos.dec = newdec;
             ln_get_equ_prec2(&epochPos, ln_get_julian_from_sys(), JD2000, &J2000Pos);
@@ -1392,8 +1362,8 @@ bool CCDSim::saveConfigItems(FILE * fp)
     // Directory
     IUSaveConfigText(fp, &DirectoryTP);
 
-    // RGB
-    IUSaveConfigSwitch(fp, &SimulateRgbSP);
+    // Bayer
+    IUSaveConfigSwitch(fp, &SimulateBayerSP);
 
     // Focus simulation
     IUSaveConfigNumber(fp, &FocusSimulationNP);
@@ -1553,6 +1523,8 @@ bool CCDSim::loadNextImage()
 
     if (ndim < 3)
         naxes[2] = 1;
+    else
+        PrimaryCCD.setNAxis(3);
     int samples_per_channel = naxes[0] * naxes[1];
     int channels = naxes[2];
     int elements = samples_per_channel * channels;
