@@ -34,39 +34,6 @@ static pthread_mutex_t condMutex = PTHREAD_MUTEX_INITIALIZER;
 // We declare an auto pointer to GuideSim.
 static std::unique_ptr<GuideSim> ccd(new GuideSim());
 
-void ISPoll(void * p);
-
-void ISGetProperties(const char * dev)
-{
-    ccd->ISGetProperties(dev);
-}
-
-void ISNewSwitch(const char * dev, const char * name, ISState * states, char * names[], int n)
-{
-    ccd->ISNewSwitch(dev, name, states, names, n);
-}
-
-void ISNewText(const char * dev, const char * name, char * texts[], char * names[], int n)
-{
-    ccd->ISNewText(dev, name, texts, names, n);
-}
-
-void ISNewNumber(const char * dev, const char * name, double values[], char * names[], int n)
-{
-    ccd->ISNewNumber(dev, name, values, names, n);
-}
-
-void ISNewBLOB(const char * dev, const char * name, int sizes[], int blobsizes[], char * blobs[], char * formats[],
-               char * names[], int n)
-{
-    ccd->ISNewBLOB(dev, name, sizes, blobsizes, blobs, formats, names, n);
-}
-
-void ISSnoopDevice(XMLEle * root)
-{
-    ccd->ISSnoopDevice(root);
-}
-
 GuideSim::GuideSim()
 {
     currentRA  = RA;
@@ -79,9 +46,6 @@ GuideSim::GuideSim()
     guiderFocalLength  = 300;
 
     time(&RunStart);
-
-    SimulatorSettingsNV = new INumberVectorProperty;
-
 }
 
 bool GuideSim::SetupParms()
@@ -127,7 +91,7 @@ bool GuideSim::Connect()
     streamPredicate = 0;
     terminateThread = false;
     pthread_create(&primary_thread, nullptr, &streamVideoHelper, this);
-    SetTimer(POLLMS);
+    SetTimer(getCurrentPollingPeriod());
     return true;
 }
 
@@ -172,14 +136,14 @@ bool GuideSim::initProperties()
     IUFillNumber(&SimulatorSettingsN[15], "SIM_KING_THETA", "hour hangle, deg", "%4.1f", 0, 360, 0, 0);
     IUFillNumber(&SimulatorSettingsN[16], "SIM_TIME_FACTOR", "Time Factor (x)", "%.2f", 0.01, 100, 0, 1);
 
-    IUFillNumberVector(SimulatorSettingsNV, SimulatorSettingsN, 17, getDeviceName(), "SIMULATOR_SETTINGS",
-                       "Simulator Settings", "Simulator Config", IP_RW, 60, IPS_IDLE);
+    IUFillNumberVector(&SimulatorSettingsNP, SimulatorSettingsN, 17, getDeviceName(), "SIMULATOR_SETTINGS",
+                       "Config", SIMULATOR_TAB, IP_RW, 60, IPS_IDLE);
 
     // RGB Simulation
     IUFillSwitch(&SimulateRgbS[0], "SIMULATE_YES", "Yes", ISS_OFF);
     IUFillSwitch(&SimulateRgbS[1], "SIMULATE_NO", "No", ISS_ON);
     IUFillSwitchVector(&SimulateRgbSP, SimulateRgbS, 2, getDeviceName(), "SIMULATE_RGB", "Simulate RGB",
-                       "Simulator Config", IP_RW, ISR_1OFMANY, 60, IPS_IDLE);
+                       SIMULATOR_TAB, IP_RW, ISR_1OFMANY, 60, IPS_IDLE);
 
     IUFillNumber(&FWHMN[0], "SIM_FWHM", "FWHM (arcseconds)", "%4.2f", 0, 60, 0, 7.5);
     IUFillNumberVector(&FWHMNP, FWHMN, 1, ActiveDeviceT[ACTIVE_FOCUSER].text, "FWHM", "FWHM", OPTIONS_TAB, IP_RO, 60, IPS_IDLE);
@@ -195,9 +159,14 @@ bool GuideSim::initProperties()
 
     IUFillNumber(&EqPEN[0], "RA_PE", "RA (hh:mm:ss)", "%010.6m", 0, 24, 0, 0);
     IUFillNumber(&EqPEN[1], "DEC_PE", "DEC (dd:mm:ss)", "%010.6m", -90, 90, 0, 0);
-    IUFillNumberVector(&EqPENP, EqPEN, 2, getDeviceName(), "EQUATORIAL_PE", "EQ PE", "Simulator Config", IP_RW, 60,
+    IUFillNumberVector(&EqPENP, EqPEN, 2, getDeviceName(), "EQUATORIAL_PE", "EQ PE", SIMULATOR_TAB, IP_RW, 60,
                        IPS_IDLE);
 
+    // Timeout
+    ToggleTimeoutSP[INDI_ENABLED].fill("INDI_ENABLED", "Enabled", ISS_OFF);
+    ToggleTimeoutSP[INDI_DISABLED].fill("INDI_DISABLED", "Disabled", ISS_ON);
+    ToggleTimeoutSP.fill(getDeviceName(), "CCD_TIMEOUT", "Timeout", SIMULATOR_TAB, IP_RW, ISR_1OFMANY, 60, IPS_IDLE);
+
 #ifdef USE_EQUATORIAL_PE
     IDSnoopDevice(ActiveDeviceT[0].text, "EQUATORIAL_PE");
 #else
@@ -256,9 +225,10 @@ void GuideSim::ISGetProperties(const char * dev)
 {
     INDI::CCD::ISGetProperties(dev);
 
-    defineNumber(SimulatorSettingsNV);
-    defineNumber(&EqPENP);
-    defineSwitch(&SimulateRgbSP);
+    defineProperty(&SimulatorSettingsNP);
+    defineProperty(&EqPENP);
+    defineProperty(&SimulateRgbSP);
+    defineProperty(&ToggleTimeoutSP);
 }
 
 bool GuideSim::updateProperties()
@@ -268,9 +238,9 @@ bool GuideSim::updateProperties()
     if (isConnected())
     {
         if (HasCooler())
-            defineSwitch(&CoolerSP);
+            defineProperty(&CoolerSP);
 
-        defineNumber(&GainNP);
+        defineProperty(&GainNP);
 
         SetupParms();
 
@@ -279,7 +249,6 @@ bool GuideSim::updateProperties()
             SetGuiderParams(500, 290, 16, 9.8, 12.6);
             GuideCCD.setFrameBufferSize(GuideCCD.getXRes() * GuideCCD.getYRes() * 2);
         }
-
     }
     else
     {
@@ -333,17 +302,6 @@ bool GuideSim::StartExposure(float duration)
     return true;
 }
 
-bool GuideSim::StartGuideExposure(float n)
-{
-    GuideExposureRequest = n;
-    AbortGuideFrame      = false;
-    GuideCCD.setExposureDuration(n);
-    DrawCcdFrame(&GuideCCD);
-    gettimeofday(&GuideExpStart, nullptr);
-    InGuideExposure = true;
-    return true;
-}
-
 bool GuideSim::AbortExposure()
 {
     if (!InExposure)
@@ -354,15 +312,6 @@ bool GuideSim::AbortExposure()
     return true;
 }
 
-bool GuideSim::AbortGuideExposure()
-{
-    //IDLog("Enter AbortGuideExposure\n");
-    if (!InGuideExposure)
-        return true; //  no need to abort if we aren't doing one
-    AbortGuideFrame = true;
-    return true;
-}
-
 float GuideSim::CalcTimeLeft(timeval start, float req)
 {
     double timesince;
@@ -382,13 +331,13 @@ float GuideSim::CalcTimeLeft(timeval start, float req)
 
 void GuideSim::TimerHit()
 {
-    uint32_t nextTimer = POLLMS;
+    uint32_t nextTimer = getCurrentPollingPeriod();
 
     //  No need to reset timer if we are not connected anymore
     if (!isConnected())
         return;
 
-    if (InExposure)
+    if (InExposure && ToggleTimeoutSP.findOnSwitchIndex() == INDI_DISABLED)
     {
         if (AbortPrimaryFrame)
         {
@@ -423,67 +372,13 @@ void GuideSim::TimerHit()
         }
     }
 
-    if (InGuideExposure)
-    {
-        float timeleft;
-        timeleft = CalcTimeLeft(GuideExpStart, GuideExposureRequest);
-
-        //IDLog("GUIDE Exposure left: %g - Requset: %g\n", timeleft, GuideExposureRequest);
-
-        if (timeleft < 0)
-            timeleft = 0;
-
-        //ImageExposureN[0].value = timeleft;
-        //IDSetNumber(ImageExposureNP, nullptr);
-        GuideCCD.setExposureLeft(timeleft);
-
-        if (timeleft < 1.0)
-        {
-            if (timeleft <= 0.001)
-            {
-                InGuideExposure = false;
-                if (!AbortGuideFrame)
-                {
-                    GuideCCD.binFrame();
-                    ExposureComplete(&GuideCCD);
-                    if (InGuideExposure)
-                    {
-                        //  the call to complete triggered another exposure
-                        timeleft = CalcTimeLeft(GuideExpStart, GuideExposureRequest);
-                        if (timeleft < 1.0)
-                        {
-                            nextTimer = timeleft * 1000;
-                        }
-                    }
-                }
-                else
-                {
-                    //IDLog("Not sending guide frame cuz of abort\n");
-                }
-                AbortGuideFrame = false;
-            }
-            else
-            {
-                nextTimer = timeleft * 1000; //  set a shorter timer
-            }
-        }
-    }
-
     if (TemperatureNP.s == IPS_BUSY)
     {
-        if (fabs(TemperatureRequest - TemperatureN[0].value) <= 0.5)
-        {
-            LOGF_INFO("Temperature reached requested value %.2f degrees C", TemperatureRequest);
-            TemperatureN[0].value = TemperatureRequest;
-            TemperatureNP.s       = IPS_OK;
-        }
+        if (TemperatureRequest < TemperatureN[0].value)
+            TemperatureN[0].value = std::max(TemperatureRequest, TemperatureN[0].value - 0.5);
         else
-        {
-            if (TemperatureRequest < TemperatureN[0].value)
-                TemperatureN[0].value -= 0.5;
-            else
-                TemperatureN[0].value += 0.5;
-        }
+            TemperatureN[0].value = std::min(TemperatureRequest, TemperatureN[0].value + 0.5);
+
 
         IDSetNumber(&TemperatureNP, nullptr);
 
@@ -509,9 +404,7 @@ int GuideSim::DrawCcdFrame(INDI::CCDChip * targetChip)
 
     uint16_t * ptr = reinterpret_cast<uint16_t *>(targetChip->getFrameBuffer());
 
-    if (targetChip->getXRes() == 500)
-        exposure_time = GuideExposureRequest * 4;
-    else if (Streamer->isStreaming())
+    if (Streamer->isStreaming())
         exposure_time = (ExposureRequest < 1) ? (ExposureRequest * 100) : ExposureRequest * 2;
     else
         exposure_time = ExposureRequest;
@@ -618,18 +511,11 @@ int GuideSim::DrawCcdFrame(INDI::CCDChip * targetChip)
             currentRA  = RA;
             currentDE = Dec;
 
-            ln_equ_posn epochPos { 0, 0 }, J2000Pos { 0, 0 };
-
-            epochPos.ra  = currentRA * 15.0;
-            epochPos.dec = currentDE;
-
+            INDI::IEquatorialCoordinates epochPos { currentRA, currentDE }, J2000Pos { 0, 0 };
             // Convert from JNow to J2000
-            LibAstro::ObservedToJ2000(&epochPos, ln_get_julian_from_sys(), &J2000Pos);
-            //ln_get_equ_prec2(&epochPos, ln_get_julian_from_sys(), JD2000, &J2000Pos);
-
-            currentRA  = J2000Pos.ra / 15.0;
-            currentDE = J2000Pos.dec;
-
+            INDI::ObservedToJ2000(&epochPos, ln_get_julian_from_sys(), &J2000Pos);
+            currentRA  = J2000Pos.rightascension;
+            currentDE = J2000Pos.declination;
             currentDE += guideNSOffset;
             currentRA += guideWEOffset;
 #ifdef USE_EQUATORIAL_PE
@@ -705,7 +591,7 @@ int GuideSim::DrawCcdFrame(INDI::CCDChip * targetChip)
             // tra, tdec are at the center of the projection center for the simulated
             // images
             //double J2ra = J2000Pos.ra;  // J2000Pos: 0,360, RA: 0,24
-            double J2dec = J2000Pos.dec;
+            double J2dec = J2000Pos.declination;
 
             //double J2rar = J2ra * 0.0174532925;
             double J2decr = J2dec * 0.0174532925;
@@ -1149,12 +1035,12 @@ bool GuideSim::ISNewNumber(const char * dev, const char * name, double values[],
 
         if (strcmp(name, "SIMULATOR_SETTINGS") == 0)
         {
-            IUUpdateNumber(SimulatorSettingsNV, values, names, n);
-            SimulatorSettingsNV->s = IPS_OK;
+            IUUpdateNumber(&SimulatorSettingsNP, values, names, n);
+            SimulatorSettingsNP.s = IPS_OK;
 
             //  Reset our parameters now
             SetupParms();
-            IDSetNumber(SimulatorSettingsNV, nullptr);
+            IDSetNumber(&SimulatorSettingsNP, nullptr);
 
             maxnoise      = SimulatorSettingsN[8].value;
             skyglow       = SimulatorSettingsN[9].value;
@@ -1181,23 +1067,14 @@ bool GuideSim::ISNewNumber(const char * dev, const char * name, double values[],
             IUUpdateNumber(&EqPENP, values, names, n);
             EqPENP.s = IPS_OK;
 
-            ln_equ_posn epochPos { 0, 0 }, J2000Pos { 0, 0 };
-            epochPos.ra  = EqPEN[AXIS_RA].value * 15.0;
-            epochPos.dec = EqPEN[AXIS_DE].value;
-
-            RA = EqPEN[AXIS_RA].value;
-            Dec = EqPEN[AXIS_DE].value;
-
-            LibAstro::ObservedToJ2000(&epochPos, ln_get_julian_from_sys(), &J2000Pos);
-            //ln_get_equ_prec2(&epochPos, ln_get_julian_from_sys(), JD2000, &J2000Pos);
-            currentRA  = J2000Pos.ra / 15.0;
-            currentDE = J2000Pos.dec;
+            INDI::IEquatorialCoordinates epochPos { EqPEN[AXIS_RA].value, EqPEN[AXIS_DE].value }, J2000Pos { 0, 0 };
+            INDI::ObservedToJ2000(&epochPos, ln_get_julian_from_sys(), &J2000Pos);
+            currentRA  = J2000Pos.rightascension;
+            currentDE = J2000Pos.declination;
             usePE = true;
-
             IDSetNumber(&EqPENP, nullptr);
             return true;
         }
-
     }
 
     return INDI::CCD::ISNewNumber(dev, name, values, names, n);
@@ -1248,6 +1125,14 @@ bool GuideSim::ISNewSwitch(const char * dev, const char * name, ISState * states
 
             return true;
         }
+
+        if (ToggleTimeoutSP.isNameMatch(name))
+        {
+            ToggleTimeoutSP.update(states, names, n);
+            ToggleTimeoutSP.setState(IPS_OK);
+            ToggleTimeoutSP.apply();
+            return true;
+        }
     }
 
     //  Nobody has claimed this, so, ignore it
@@ -1295,7 +1180,7 @@ bool GuideSim::ISSnoopDevice(XMLEle * root)
 
         if (rc_ra == 0 && rc_de == 0 && ((newra != raPE) || (newdec != decPE)))
         {
-            ln_equ_posn epochPos { 0, 0 }, J2000Pos { 0, 0 };
+            INDI::IEquatorialCoordinates epochPos { 0, 0 }, J2000Pos { 0, 0 };
             epochPos.ra  = newra * 15.0;
             epochPos.dec = newdec;
             ln_get_equ_prec2(&epochPos, ln_get_julian_from_sys(), JD2000, &J2000Pos);
@@ -1324,7 +1209,7 @@ bool GuideSim::saveConfigItems(FILE * fp)
 
 
     // Save CCD Simulator Config
-    IUSaveConfigNumber(fp, SimulatorSettingsNV);
+    IUSaveConfigNumber(fp, &SimulatorSettingsNP);
 
     // Gain
     IUSaveConfigNumber(fp, &GainNP);
