@@ -20,6 +20,7 @@
 
 #include "indiccd.h"
 #include "indifilterinterface.h"
+#include "indipropertyswitch.h"
 
 /**
  * @brief The GuideSim class provides an advanced simulator for a CCD that includes a dedicated on-board guide chip.
@@ -64,10 +65,8 @@ class GuideSim : public INDI::CCD
         bool Disconnect() override;
 
         bool StartExposure(float duration) override;
-        bool StartGuideExposure(float) override;
 
         bool AbortExposure() override;
-        bool AbortGuideExposure() override;
 
         void TimerHit() override;
 
@@ -99,7 +98,7 @@ class GuideSim : public INDI::CCD
         // Turns on/off Bayer RGB simulation.
         void setRGB(bool onOff);
 
-        float TemperatureRequest { 0 };
+        double TemperatureRequest { 0 };
 
         float ExposureRequest { 0 };
         struct timeval ExpStart
@@ -107,11 +106,6 @@ class GuideSim : public INDI::CCD
             0, 0
         };
 
-        float GuideExposureRequest { 0 };
-        struct timeval GuideExpStart
-        {
-            0, 0
-        };
 
         int testvalue { 0 };
         bool ShowStarField { true };
@@ -137,7 +131,6 @@ class GuideSim : public INDI::CCD
         float k { 0 };
         float z { 0 };
 
-        bool AbortGuideFrame { false };
         bool AbortPrimaryFrame { false };
 
         /// Guide rate is 7 arcseconds per second
@@ -166,7 +159,7 @@ class GuideSim : public INDI::CCD
 
         //  And this lives in our simulator settings page
 
-        INumberVectorProperty *SimulatorSettingsNV;
+        INumberVectorProperty SimulatorSettingsNP;
         INumber SimulatorSettingsN[17];
 
         ISwitchVectorProperty SimulateRgbSP;
@@ -184,4 +177,8 @@ class GuideSim : public INDI::CCD
 
         INumber GainN[1];
         INumberVectorProperty GainNP;
+
+        INDI::PropertySwitch ToggleTimeoutSP {2};
+
+        static constexpr const char *SIMULATOR_TAB {"Simulator Settings"};
 };
