@@ -48,44 +48,6 @@ typedef enum
 
 static std::unique_ptr<DomeScript> scope_script(new DomeScript());
 
-void ISGetProperties(const char *dev)
-{
-    scope_script->ISGetProperties(dev);
-}
-
-void ISNewSwitch(const char *dev, const char *name, ISState *states, char *names[], int n)
-{
-    scope_script->ISNewSwitch(dev, name, states, names, n);
-}
-
-void ISNewText(const char *dev, const char *name, char *texts[], char *names[], int n)
-{
-    scope_script->ISNewText(dev, name, texts, names, n);
-}
-
-void ISNewNumber(const char *dev, const char *name, double values[], char *names[], int n)
-{
-    scope_script->ISNewNumber(dev, name, values, names, n);
-}
-
-void ISNewBLOB(const char *dev, const char *name, int sizes[], int blobsizes[], char *blobs[], char *formats[],
-               char *names[], int n)
-{
-    INDI_UNUSED(dev);
-    INDI_UNUSED(name);
-    INDI_UNUSED(sizes);
-    INDI_UNUSED(blobsizes);
-    INDI_UNUSED(blobs);
-    INDI_UNUSED(formats);
-    INDI_UNUSED(names);
-    INDI_UNUSED(n);
-}
-
-void ISSnoopDevice(XMLEle *root)
-{
-    scope_script->ISSnoopDevice(root);
-}
-
 DomeScript::DomeScript()
 {
     SetDomeCapability(DOME_CAN_PARK | DOME_CAN_ABORT | DOME_CAN_ABS_MOVE | DOME_HAS_SHUTTER);
@@ -133,7 +95,7 @@ bool DomeScript::saveConfigItems(FILE *fp)
 void DomeScript::ISGetProperties(const char *dev)
 {
     INDI::Dome::ISGetProperties(dev);
-    defineText(&ScriptsTP);
+    defineProperty(&ScriptsTP);
     loadConfig(true, "SCRIPTS");
 }
 
@@ -320,7 +282,7 @@ void DomeScript::TimerHit()
     {
         LOG_ERROR("Failed to read status");
     }
-    SetTimer(POLLMS);
+    SetTimer(getCurrentPollingPeriod());
     if (!isParked() && TimeSinceUpdate++ > 4)
     {
         TimeSinceUpdate = 0;
