@@ -30,46 +30,6 @@ static std::unique_ptr<DomeSim> domeSim(new DomeSim());
 #define DOME_SPEED    10.0 /* 10 degrees per second, constant */
 #define SHUTTER_TIMER 5.0  /* Shutter closes/open in 5 seconds */
 
-void ISPoll(void *p);
-
-void ISGetProperties(const char *dev)
-{
-    domeSim->ISGetProperties(dev);
-}
-
-void ISNewSwitch(const char *dev, const char *name, ISState *states, char *names[], int n)
-{
-    domeSim->ISNewSwitch(dev, name, states, names, n);
-}
-
-void ISNewText(const char *dev, const char *name, char *texts[], char *names[], int n)
-{
-    domeSim->ISNewText(dev, name, texts, names, n);
-}
-
-void ISNewNumber(const char *dev, const char *name, double values[], char *names[], int n)
-{
-    domeSim->ISNewNumber(dev, name, values, names, n);
-}
-
-void ISNewBLOB(const char *dev, const char *name, int sizes[], int blobsizes[], char *blobs[], char *formats[],
-               char *names[], int n)
-{
-    INDI_UNUSED(dev);
-    INDI_UNUSED(name);
-    INDI_UNUSED(sizes);
-    INDI_UNUSED(blobsizes);
-    INDI_UNUSED(blobs);
-    INDI_UNUSED(formats);
-    INDI_UNUSED(names);
-    INDI_UNUSED(n);
-}
-
-void ISSnoopDevice(XMLEle *root)
-{
-    domeSim->ISSnoopDevice(root);
-}
-
 DomeSim::DomeSim()
 {
     targetAz        = 0;
