@@ -32,44 +32,6 @@
 
 static std::unique_ptr<SteelDriveII> steelDrive(new SteelDriveII());
 
-void ISGetProperties(const char *dev)
-{
-    steelDrive->ISGetProperties(dev);
-}
-
-void ISNewSwitch(const char *dev, const char *name, ISState *states, char *names[], int n)
-{
-    steelDrive->ISNewSwitch(dev, name, states, names, n);
-}
-
-void ISNewText(const char *dev, const char *name, char *texts[], char *names[], int n)
-{
-    steelDrive->ISNewText(dev, name, texts, names, n);
-}
-
-void ISNewNumber(const char *dev, const char *name, double values[], char *names[], int n)
-{
-    steelDrive->ISNewNumber(dev, name, values, names, n);
-}
-
-void ISNewBLOB(const char *dev, const char *name, int sizes[], int blobsizes[], char *blobs[], char *formats[],
-               char *names[], int n)
-{
-    INDI_UNUSED(dev);
-    INDI_UNUSED(name);
-    INDI_UNUSED(sizes);
-    INDI_UNUSED(blobsizes);
-    INDI_UNUSED(blobs);
-    INDI_UNUSED(formats);
-    INDI_UNUSED(names);
-    INDI_UNUSED(n);
-}
-
-void ISSnoopDevice(XMLEle *root)
-{
-    steelDrive->ISSnoopDevice(root);
-}
-
 /////////////////////////////////////////////////////////////////////////////
 ///
 /////////////////////////////////////////////////////////////////////////////
@@ -148,14 +110,14 @@ bool SteelDriveII::updateProperties()
     {
         getStartupValues();
 
-        defineText(&InfoTP);
-        defineSwitch(&OperationSP);
+        defineProperty(&InfoTP);
+        defineProperty(&OperationSP);
 
-        defineSwitch(&TemperatureCompensationSP);
-        defineSwitch(&TemperatureStateSP);
-        defineNumber(&TemperatureSettingsNP);
-        defineNumber(&TemperatureSensorNP);
-        defineNumber(&StepperDriveNP);
+        defineProperty(&TemperatureCompensationSP);
+        defineProperty(&TemperatureStateSP);
+        defineProperty(&TemperatureSettingsNP);
+        defineProperty(&TemperatureSensorNP);
+        defineProperty(&StepperDriveNP);
     }
     else
     {
@@ -437,7 +399,8 @@ void SteelDriveII::TimerHit()
         FocusMotionS[FOCUS_INWARD].s = (m_State == GOING_DOWN) ? ISS_ON : ISS_OFF;
         FocusMotionS[FOCUS_OUTWARD].s = (m_State == GOING_DOWN) ? ISS_OFF : ISS_ON;
         FocusMotionSP.s = IPS_BUSY;
-        FocusAbsPosNP.s = FocusRelPosNP.s = IPS_BUSY;
+        FocusAbsPosNP.s = IPS_BUSY;
+        FocusRelPosNP.s = IPS_BUSY;
         FocusAbsPosN[0].value = summaryPosition;
 
         IDSetSwitch(&FocusMotionSP, nullptr);
@@ -496,7 +459,7 @@ void SteelDriveII::TimerHit()
         IDSetNumber(&TemperatureSensorNP, nullptr);
     }
 
-    SetTimer(POLLMS);
+    SetTimer(getCurrentPollingPeriod());
 }
 
 /////////////////////////////////////////////////////////////////////////////
