@@ -35,43 +35,6 @@
 // We declare an auto pointer to TCFS.
 static std::unique_ptr<TCFS> tcfs(new TCFS());
 
-void ISGetProperties(const char *dev)
-{
-    tcfs->ISGetProperties(dev);
-}
-
-void ISNewSwitch(const char *dev, const char *name, ISState *states, char *names[], int n)
-{
-    tcfs->ISNewSwitch(dev, name, states, names, n);
-}
-
-void ISNewText(const char *dev, const char *name, char *texts[], char *names[], int n)
-{
-    tcfs->ISNewText(dev, name, texts, names, n);
-}
-
-void ISNewNumber(const char *dev, const char *name, double values[], char *names[], int n)
-{
-    tcfs->ISNewNumber(dev, name, values, names, n);
-}
-
-void ISNewBLOB(const char *dev, const char *name, int sizes[], int blobsizes[], char *blobs[], char *formats[],
-               char *names[], int n)
-{
-    INDI_UNUSED(dev);
-    INDI_UNUSED(name);
-    INDI_UNUSED(sizes);
-    INDI_UNUSED(blobsizes);
-    INDI_UNUSED(blobs);
-    INDI_UNUSED(formats);
-    INDI_UNUSED(names);
-    INDI_UNUSED(n);
-}
-void ISSnoopDevice(XMLEle *root)
-{
-    tcfs->ISSnoopDevice(root);
-}
-
 /****************************************************************
 **
 **
@@ -98,24 +61,24 @@ bool TCFS::initProperties()
     {
         isTCFS3 = true;
 
-        FocusMaxPosN[0].max = 9999;
-
-        FocusAbsPosN[0].max  = 9999;
-        FocusRelPosN[0].max  = 2000;
-        FocusRelPosN[0].step = FocusAbsPosN[0].step = 100;
-        FocusRelPosN[0].value                       = 0;
+        FocusMaxPosN[0].max   = 9999;
+        FocusAbsPosN[0].max   = 9999;
+        FocusRelPosN[0].max   = 2000;
+        FocusRelPosN[0].step  = 100;
+        FocusAbsPosN[0].step  = 100;
+        FocusRelPosN[0].value = 0;
         LOG_DEBUG("TCF-S3 detected. Updating maximum position value to 9999.");
     }
     else
     {
         isTCFS3 = false;
 
-        FocusMaxPosN[0].max = 7000;
-
-        FocusAbsPosN[0].max  = 7000;
-        FocusRelPosN[0].max  = 2000;
-        FocusRelPosN[0].step = FocusAbsPosN[0].step = 100;
-        FocusRelPosN[0].value                       = 0;
+        FocusMaxPosN[0].max   = 7000;
+        FocusAbsPosN[0].max   = 7000;
+        FocusRelPosN[0].max   = 2000;
+        FocusRelPosN[0].step  = 100;
+        FocusAbsPosN[0].step  = 100;
+        FocusRelPosN[0].value = 0;
         LOG_DEBUG("TCF-S detected. Updating maximum position value to 7000.");
     }
 
@@ -181,14 +144,14 @@ bool TCFS::updateProperties()
 
     if (isConnected())
     {
-        defineSwitch(&FocusGotoSP);
-        defineNumber(&FocusTemperatureNP);
-        defineSwitch(&FocusPowerSP);
-        defineSwitch(&FocusModeSP);
-        defineSwitch(&FocusTelemetrySP);
-        defineSwitch(&FocusStartModeSP);
-        defineNumber(&FocusModeANP);
-        defineNumber(&FocusModeBNP);;
+        defineProperty(&FocusGotoSP);
+        defineProperty(&FocusTemperatureNP);
+        defineProperty(&FocusPowerSP);
+        defineProperty(&FocusModeSP);
+        defineProperty(&FocusTelemetrySP);
+        defineProperty(&FocusStartModeSP);
+        defineProperty(&FocusModeANP);
+        defineProperty(&FocusModeBNP);;
         GetFocusParams();
     }
     else
@@ -679,7 +642,8 @@ bool TCFS::ISNewSwitch(const char *dev, const char *name, ISState *states, char
             else if (!strcmp(sp->name, "FOCUS_CENTER"))
             {
                 dispatch_command(FCENTR);
-                FocusAbsPosNP.s = FocusRelPosNP.s = IPS_BUSY;
+                FocusAbsPosNP.s = IPS_BUSY;
+                FocusRelPosNP.s = IPS_BUSY;
                 IDSetNumber(&FocusAbsPosNP, nullptr);
                 IDSetNumber(&FocusRelPosNP, nullptr);
                 IDSetSwitch(&FocusGotoSP, "Moving focuser to center position %d...", isTCFS3 ? 5000 : 3500);
@@ -962,7 +926,7 @@ void TCFS::TimerHit()
 
     if (!isConnected())
     {
-        SetTimer(POLLMS);
+        SetTimer(getCurrentPollingPeriod());
         return;
     }
 
@@ -974,7 +938,7 @@ void TCFS::TimerHit()
         LOGF_DEBUG("%s Motion in Progress...", __FUNCTION__);
         if (read_tcfs(response, true) == false)
         {
-            SetTimer(POLLMS);
+            SetTimer(getCurrentPollingPeriod());
             return;
         }
         LOGF_DEBUG("%s READY %s", __FUNCTION__, response );
@@ -999,7 +963,7 @@ void TCFS::TimerHit()
                     IUResetSwitch(&FocusModeSP);
                     FocusModeSP.s = IPS_ALERT;
                     IDSetSwitch(&FocusModeSP, "Error switching to Auto Mode %s. No reply from TCF-S. Try again.", mode);
-                    SetTimer(POLLMS);
+                    SetTimer(getCurrentPollingPeriod());
                     return;
                 }
                 FocusModeSP.s = IPS_OK;
@@ -1007,7 +971,7 @@ void TCFS::TimerHit()
                 currentMode = (FocusModeSP.sp[1].s == ISS_ON) ? MODE_A : MODE_B;
                 IDSetSwitch(&FocusModeSP, nullptr);
             }
-            SetTimer(POLLMS);
+            SetTimer(getCurrentPollingPeriod());
             return;
         }
     }
@@ -1020,14 +984,14 @@ void TCFS::TimerHit()
         if (FocusTelemetrySP.sp[1].s == ISS_ON)
         {
             LOGF_DEBUG("%s %s", __FUNCTION__, "Telemetry is off");
-            SetTimer(POLLMS);
+            SetTimer(getCurrentPollingPeriod());
             return;
         }
         for(int i = 0; i < 2; i++)
         {
             if (read_tcfs(response, true) == false)
             {
-                SetTimer(POLLMS);
+                SetTimer(getCurrentPollingPeriod());
                 return;
             }
             LOGF_DEBUG("%s Received %s", __FUNCTION__, response);
@@ -1051,7 +1015,7 @@ void TCFS::TimerHit()
                 }
             }
         }
-        SetTimer(POLLMS);
+        SetTimer(getCurrentPollingPeriod());
         return;
     }
 
@@ -1065,7 +1029,7 @@ void TCFS::TimerHit()
 
             if (!rc)
             {
-                SetTimer(POLLMS);
+                SetTimer(getCurrentPollingPeriod());
                 return;
             }
 
@@ -1094,7 +1058,7 @@ void TCFS::TimerHit()
 
             if (read_tcfs(response) == false)
             {
-                SetTimer(POLLMS);
+                SetTimer(getCurrentPollingPeriod());
                 return;
             }
 
@@ -1114,7 +1078,7 @@ void TCFS::TimerHit()
         case IPS_BUSY:
             if (read_tcfs(response, true) == false)
             {
-                SetTimer(POLLMS);
+                SetTimer(getCurrentPollingPeriod());
                 return;
             }
 
@@ -1122,7 +1086,7 @@ void TCFS::TimerHit()
             if (strstr(response, "ER") != nullptr)
             {
                 LOGF_DEBUG("Received error: %s", response);
-                SetTimer(POLLMS);
+                SetTimer(getCurrentPollingPeriod());
                 return;
             }
 
@@ -1164,7 +1128,7 @@ void TCFS::TimerHit()
             IDSetNumber(&FocusTemperatureNP, nullptr);
             LOG_ERROR("Failed to read temperature. Is sensor connected?");
 
-            SetTimer(POLLMS);
+            SetTimer(getCurrentPollingPeriod());
             return;
         }
 
@@ -1191,7 +1155,7 @@ void TCFS::TimerHit()
         }
     }
 
-    SetTimer(POLLMS);
+    SetTimer(getCurrentPollingPeriod());
 }
 
 bool TCFS::read_tcfs(char *response, bool silent)
