@@ -36,44 +36,6 @@
 
 static std::unique_ptr<USBFocusV3> usbFocusV3(new USBFocusV3());
 
-void ISGetProperties(const char *dev)
-{
-    usbFocusV3->ISGetProperties(dev);
-}
-
-void ISNewSwitch(const char *dev, const char *name, ISState *states, char *names[], int n)
-{
-    usbFocusV3->ISNewSwitch(dev, name, states, names, n);
-}
-
-void ISNewText(const char *dev, const char *name, char *texts[], char *names[], int n)
-{
-    usbFocusV3->ISNewText(dev, name, texts, names, n);
-}
-
-void ISNewNumber(const char *dev, const char *name, double values[], char *names[], int n)
-{
-    usbFocusV3->ISNewNumber(dev, name, values, names, n);
-}
-
-void ISNewBLOB(const char *dev, const char *name, int sizes[], int blobsizes[], char *blobs[], char *formats[],
-               char *names[], int n)
-{
-    INDI_UNUSED(dev);
-    INDI_UNUSED(name);
-    INDI_UNUSED(sizes);
-    INDI_UNUSED(blobsizes);
-    INDI_UNUSED(blobs);
-    INDI_UNUSED(formats);
-    INDI_UNUSED(names);
-    INDI_UNUSED(n);
-}
-
-void ISSnoopDevice(XMLEle *root)
-{
-    usbFocusV3->ISSnoopDevice(root);
-}
-
 USBFocusV3::USBFocusV3()
 {
     // Can move in Absolute & Relative motions, can AbortFocuser motion, and has variable speed.
@@ -194,18 +156,18 @@ bool USBFocusV3::updateProperties()
 
     if (isConnected())
     {
-        defineNumber(&TemperatureNP);
-        defineNumber(&MaxPositionNP);
-        defineSwitch(&StepModeSP);
-        defineSwitch(&RotDirSP);
-        defineNumber(&MaxPositionNP);
-        defineNumber(&TemperatureSettingNP);
-        defineSwitch(&TempCompSignSP);
-        defineSwitch(&TemperatureCompensateSP);
-        //        defineSwitch(&BacklashDirectionSP);
-        //        defineNumber(&BacklashSettingNP);
-        defineSwitch(&ResetSP);
-        defineNumber(&FWversionNP);
+        defineProperty(&TemperatureNP);
+        defineProperty(&MaxPositionNP);
+        defineProperty(&StepModeSP);
+        defineProperty(&RotDirSP);
+        defineProperty(&MaxPositionNP);
+        defineProperty(&TemperatureSettingNP);
+        defineProperty(&TempCompSignSP);
+        defineProperty(&TemperatureCompensateSP);
+        //        defineProperty(&BacklashDirectionSP);
+        //        defineProperty(&BacklashSettingNP);
+        defineProperty(&ResetSP);
+        defineProperty(&FWversionNP);
 
         GetFocusParams();
 
@@ -1219,7 +1181,7 @@ void USBFocusV3::TimerHit()
         }
     }
 
-    SetTimer(POLLMS);
+    SetTimer(getCurrentPollingPeriod());
 }
 
 bool USBFocusV3::AbortFocuser()
