@@ -84,7 +84,7 @@ LX200_10MICRON::LX200_10MICRON() : LX200Generic()
 // Note that getDriverName calls ::getDefaultName which returns LX200 Generic
 const char *LX200_10MICRON::getDefaultName()
 {
-    return (const char *)"10micron";
+    return "10micron";
 }
 
 // Called by INDI::Telescope::callHandshake, either TCP Connect or Serial Port Connect
@@ -147,7 +147,7 @@ bool LX200_10MICRON::initProperties()
         IUFillSwitch(&AlignmentStateS[ALIGN_START], "Start", "Start new model", ISS_OFF);
         IUFillSwitch(&AlignmentStateS[ALIGN_END], "End", "End new model", ISS_OFF);
         IUFillSwitch(&AlignmentStateS[ALIGN_DELETE_CURRENT], "Del", "Delete current model", ISS_OFF);
-        IUFillSwitchVector(&AlignmentSP, AlignmentStateS, ALIGN_COUNT, getDeviceName(),
+        IUFillSwitchVector(&AlignmentStateSP, AlignmentStateS, ALIGN_COUNT, getDeviceName(),
                            ALIGNMENT_STATE, "Alignment", ALIGNMENT_TAB, IP_RW, ISR_1OFMANY, 60, IPS_IDLE);
 
         IUFillNumber(&MiniNewAlpRON[MALPRO_MRA], "MRA", "Mount RA (hh:mm:ss)", "%010.6m", 0, 24, 0, 0);
@@ -200,19 +200,19 @@ bool LX200_10MICRON::updateProperties()
 
     if (isConnected())
     {
-        defineSwitch(&UnattendedFlipSP);
+        defineProperty(&UnattendedFlipSP);
         // getMountInfo defines ProductTP
-        defineNumber(&RefractionModelTemperatureNP);
-        defineNumber(&RefractionModelPressureNP);
-        defineNumber(&ModelCountNP);
-        defineNumber(&AlignmentPointsNP);
-        defineSwitch(&AlignmentSP);
-        defineNumber(&MiniNewAlpRONP);
-        defineNumber(&MiniNewAlpNP);
-        defineNumber(&NewAlpNP);
-        defineNumber(&NewAlignmentPointsNP);
-        defineText(&NewModelNameTP);
-        defineNumber(&TLEfromDatabaseNP);
+        defineProperty(&RefractionModelTemperatureNP);
+        defineProperty(&RefractionModelPressureNP);
+        defineProperty(&ModelCountNP);
+        defineProperty(&AlignmentPointsNP);
+        defineProperty(&AlignmentStateSP);
+        defineProperty(&MiniNewAlpRONP);
+        defineProperty(&MiniNewAlpNP);
+        defineProperty(&NewAlpNP);
+        defineProperty(&NewAlignmentPointsNP);
+        defineProperty(&NewModelNameTP);
+        defineProperty(&TLEfromDatabaseNP);
 
         // read UnAttendedFlip setting from config and apply if available
         int readit = 0;
@@ -248,7 +248,7 @@ bool LX200_10MICRON::updateProperties()
         deleteProperty(RefractionModelPressureNP.name);
         deleteProperty(ModelCountNP.name);
         deleteProperty(AlignmentPointsNP.name);
-        deleteProperty(AlignmentSP.name);
+        deleteProperty(AlignmentStateSP.name);
         deleteProperty(MiniNewAlpRONP.name);
         deleteProperty(MiniNewAlpNP.name);
         deleteProperty(NewAlpNP.name);
@@ -281,14 +281,14 @@ void LX200_10MICRON::getBasicData()
         else
         {
             LOGF_INFO("Tracking frequency is %.1f Hz", TrackFreqN[0].value);
-            IDSetNumber(&TrackingFreqNP, nullptr);
+            IDSetNumber(&TrackFreqNP, nullptr);
         }
 
         char RefractionModelTemperature[80];
         getCommandString(PortFD, RefractionModelTemperature, "#:GRTMP#");
         float rmtemp;
         sscanf(RefractionModelTemperature, "%f#", &rmtemp);
-        RefractionModelTemperatureN[0].value = (double) rmtemp;
+        RefractionModelTemperatureN[0].value = rmtemp;
         LOGF_INFO("RefractionModelTemperature is %0+6.1f degrees C", RefractionModelTemperatureN[0].value);
         IDSetNumber(&RefractionModelTemperatureNP, nullptr);
 
@@ -296,20 +296,20 @@ void LX200_10MICRON::getBasicData()
         getCommandString(PortFD, RefractionModelPressure, "#:GRPRS#");
         float rmpres;
         sscanf(RefractionModelPressure, "%f#", &rmpres);
-        RefractionModelPressureN[0].value = (double) rmpres;
+        RefractionModelPressureN[0].value = rmpres;
         LOGF_INFO("RefractionModelPressure is %06.1f hPa", RefractionModelPressureN[0].value);
         IDSetNumber(&RefractionModelPressureNP, nullptr);
 
         int ModelCount;
         getCommandInt(PortFD, &ModelCount, "#:modelcnt#");
         ModelCountN[0].value = (double) ModelCount;
-        LOGF_INFO("%d Alignment Models", (int) ModelCountN[0].value);
+        LOGF_INFO("%d Alignment Models", static_cast<int>(ModelCountN[0].value));
         IDSetNumber(&ModelCountNP, nullptr);
 
         int AlignmentPoints;
         getCommandInt(PortFD, &AlignmentPoints, "#:getalst#");
-        AlignmentPointsN[0].value = (double) AlignmentPoints;
-        LOGF_INFO("%d Alignment Stars in active model", (int) AlignmentPointsN[0].value);
+        AlignmentPointsN[0].value = AlignmentPoints;
+        LOGF_INFO("%d Alignment Stars in active model", static_cast<int>(AlignmentPointsN[0].value));
         IDSetNumber(&AlignmentPointsNP, nullptr);
 
         if (false == getUnattendedFlipSetting())
@@ -376,7 +376,7 @@ bool LX200_10MICRON::getMountInfo()
     IUFillTextVector(&ProductTP, ProductT, PRODUCT_COUNT, getDeviceName(),
                      PRODUCT_INFO, "Product", PRODUCT_TAB, IP_RO, 60, IPS_IDLE);
 
-    defineText(&ProductTP);
+    defineProperty(&ProductTP);
 
     return true;
 }
@@ -541,6 +541,8 @@ bool LX200_10MICRON::Park()
     {
         return false;
     }
+
+    TrackState = SCOPE_PARKING;
     // postpone SetParked(true) for ReadScopeStatus so that we know it is actually correct
     return true;
 }
@@ -555,6 +557,8 @@ bool LX200_10MICRON::UnPark()
     {
         return false;
     }
+
+    TrackState = SCOPE_IDLE;
     SetParked(false);
     return true;
 }
@@ -711,10 +715,10 @@ bool LX200_10MICRON::SetTLEtoFollow(const char *tle)
         {
             LOG_ERROR("Invalid formatting of TLE, trying to split:");
             char *pch = strtok ((char*) tle, "\n");
-            while (pch != NULL)
+            while (pch != nullptr)
             {
                 LOGF_INFO("%s\n", pch);
-                pch = strtok (NULL, "\n");
+                pch = strtok (nullptr, "\n");
             }
             return 1;
         }
@@ -722,10 +726,10 @@ bool LX200_10MICRON::SetTLEtoFollow(const char *tle)
     else
     {
         char *pch = strtok ((char*) tle, "\n");
-        while (pch != NULL)
+        while (pch != nullptr)
         {
             LOGF_INFO("%s\n", pch);
-            pch = strtok (NULL, "\n");
+            pch = strtok (nullptr, "\n");
         }
     }
     return 0;
@@ -794,7 +798,7 @@ bool LX200_10MICRON::CalculateSatTrajectory(std::string start_pass_isodatetime,
     double JD_end;
     JD_start = ln_get_julian_day(&start_pass);
     JD_end = ln_get_julian_day(&end_pass);
-    int nextPassInMinutes = (int) ceil((JD_end - JD_start) * 24 * 60);
+    int nextPassInMinutes = static_cast<int>(ceil((JD_end - JD_start) * 24 * 60));
     int nextPassinMinutesUpTo1440 = std::min(nextPassInMinutes,  1440);
     int nextPassinMinutesBetween1and1440 = std::max(nextPassinMinutesUpTo1440, 1);
 
@@ -928,7 +932,7 @@ int LX200_10MICRON::AddSyncPoint(double MRa, double MDec, double MSide, double P
     fs_sexa(MDec_str, MDec, 0, 3600);
 
     char MSide_char;
-    ((int)MSide == 0) ? MSide_char = 'E' : MSide_char = 'W';
+    (static_cast<int>(MSide) == 0) ? MSide_char = 'E' : MSide_char = 'W';
 
     char PRa_str[32], PDec_str[32];
     fs_sexa(PRa_str, PRa, 0, 36000);
@@ -1095,10 +1099,10 @@ bool LX200_10MICRON::ISNewSwitch(const char *dev, const char *name, ISState *sta
 {
     if (dev != nullptr && strcmp(dev, getDeviceName()) == 0)
     {
-        if (strcmp(AlignmentSP.name, name) == 0)
+        if (strcmp(AlignmentStateSP.name, name) == 0)
         {
-            IUUpdateSwitch(&AlignmentSP, states, names, n);
-            int index    = IUFindOnSwitchIndex(&AlignmentSP);
+            IUUpdateSwitch(&AlignmentStateSP, states, names, n);
+            int index    = IUFindOnSwitchIndex(&AlignmentStateSP);
 
             switch (index)
             {
@@ -1116,8 +1120,8 @@ bool LX200_10MICRON::ISNewSwitch(const char *dev, const char *name, ISState *sta
                     if (0 != setStandardProcedureAndExpect(fd, "#:newalig#", "V"))
                     {
                         LOG_ERROR("New alignment start error");
-                        AlignmentSP.s = IPS_ALERT;
-                        IDSetSwitch(&AlignmentSP, nullptr);
+                        AlignmentStateSP.s = IPS_ALERT;
+                        IDSetSwitch(&AlignmentStateSP, nullptr);
                         return false;
                     }
                     LOG_INFO("New Alignment started");
@@ -1136,8 +1140,8 @@ bool LX200_10MICRON::ISNewSwitch(const char *dev, const char *name, ISState *sta
                     if (0 != setStandardProcedureAndExpect(fd, "#:endalig#", "V"))
                     {
                         LOG_ERROR("New alignment end error");
-                        AlignmentSP.s = IPS_ALERT;
-                        IDSetSwitch(&AlignmentSP, nullptr);
+                        AlignmentStateSP.s = IPS_ALERT;
+                        IDSetSwitch(&AlignmentStateSP, nullptr);
                         return false;
                     }
                     LOG_INFO("New Alignment ended");
@@ -1152,8 +1156,8 @@ bool LX200_10MICRON::ISNewSwitch(const char *dev, const char *name, ISState *sta
                     if (0 != setStandardProcedureAndExpect(fd, "#:delalig#", "#"))
                     {
                         LOG_ERROR("Delete current alignment error");
-                        AlignmentSP.s = IPS_ALERT;
-                        IDSetSwitch(&AlignmentSP, nullptr);
+                        AlignmentStateSP.s = IPS_ALERT;
+                        IDSetSwitch(&AlignmentStateSP, nullptr);
                         return false;
                     }
                     LOG_INFO("Current Alignment deleted");
@@ -1161,14 +1165,14 @@ bool LX200_10MICRON::ISNewSwitch(const char *dev, const char *name, ISState *sta
                     break;
 
                 default:
-                    AlignmentSP.s = IPS_ALERT;
-                    IDSetSwitch(&AlignmentSP, "Unknown alignment index %d", index);
+                    AlignmentStateSP.s = IPS_ALERT;
+                    IDSetSwitch(&AlignmentStateSP, "Unknown alignment index %d", index);
                     AlignmentState = ALIGN_IDLE;
                     return false;
             }
 
-            AlignmentSP.s = IPS_OK;
-            IDSetSwitch(&AlignmentSP, nullptr);
+            AlignmentStateSP.s = IPS_OK;
+            IDSetSwitch(&AlignmentStateSP, nullptr);
             return true;
         }
         if (strcmp(TrackSatSP.name, name) == 0)
@@ -1284,8 +1288,6 @@ bool LX200_10MICRON::ISNewText(const char *dev, const char *name, char *texts[],
                 LOG_ERROR("TLE was not correctly uploaded");
                 return false;
             }
-
-            return true;
         }
         if (strcmp(name, "SAT_PASS_WINDOW") == 0)
         {
@@ -1304,8 +1306,6 @@ bool LX200_10MICRON::ISNewText(const char *dev, const char *name, char *texts[],
                 LOG_ERROR("Trajectory could not be calculated");
                 return false;
             }
-
-            return true;
         }
     }
     return LX200Generic::ISNewText(dev, name, texts, names, n);
