@@ -1,6 +1,6 @@
 /*
 
-LX200_TeenAstro 
+LX200_TeenAstro
 
 Based on LX200_OnStep and others
 François Desvallées https://github.com/fdesvallees
@@ -53,40 +53,6 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110 - 1301  USA
 static std::unique_ptr<LX200_TeenAstro> teenAstro(new LX200_TeenAstro());
 extern std::mutex lx200CommsLock;
 
-void ISGetProperties(const char *dev)
-{
-    teenAstro->ISGetProperties(dev);
-}
-void ISNewSwitch(const char *dev, const char *name, ISState *states, char *names[], int n)
-{
-    teenAstro->ISNewSwitch(dev, name, states, names, n);
-}
-void ISNewText(const char *dev, const char *name, char *texts[], char *names[], int n)
-{
-    teenAstro->ISNewText(dev, name, texts, names, n);
-}
-void ISNewNumber(const char *dev, const char *name, double values[], char *names[], int n)
-{
-    teenAstro->ISNewNumber(dev, name, values, names, n);
-}
-void ISNewBLOB(const char *dev, const char *name, int sizes[], int blobsizes[], char *blobs[], char *formats[],
-               char *names[], int n)
-{
-    INDI_UNUSED(dev);
-    INDI_UNUSED(name);
-    INDI_UNUSED(sizes);
-    INDI_UNUSED(blobsizes);
-    INDI_UNUSED(blobs);
-    INDI_UNUSED(formats);
-    INDI_UNUSED(names);
-    INDI_UNUSED(n);
-}
-void ISSnoopDevice(XMLEle *root)
-{
-    teenAstro->ISSnoopDevice(root);
-}
-
-
 /*
  * LX200 TeenAstro constructor
  */
@@ -130,13 +96,14 @@ bool LX200_TeenAstro::initProperties()
     // ============== MAIN_CONTROL_TAB
 
     // Tracking Mode
-     AddTrackMode("TRACK_SIDEREAL", "Sidereal", true);
-     AddTrackMode("TRACK_SOLAR", "Solar");
-     AddTrackMode("TRACK_LUNAR", "Lunar");
+    AddTrackMode("TRACK_SIDEREAL", "Sidereal", true);
+    AddTrackMode("TRACK_SOLAR", "Solar");
+    AddTrackMode("TRACK_LUNAR", "Lunar");
 
     // Error Status
     IUFillText(&ErrorStatusT[0], "Error code", "", "");
-    IUFillTextVector(&ErrorStatusTP, ErrorStatusT, 1, getDeviceName(), "Mount Status", "", MAIN_CONTROL_TAB, IP_RO, 0, IPS_IDLE);
+    IUFillTextVector(&ErrorStatusTP, ErrorStatusT, 1, getDeviceName(), "Mount Status", "", MAIN_CONTROL_TAB, IP_RO, 0,
+                     IPS_IDLE);
 
 
     // ============== MOTION_TAB
@@ -146,15 +113,16 @@ bool LX200_TeenAstro::initProperties()
     IUFillSwitch(&SlewRateS[2], "Medium", "Medium", ISS_OFF);
     IUFillSwitch(&SlewRateS[3], "Fast", "Fast", ISS_ON);
     IUFillSwitch(&SlewRateS[4], "Max", "Max", ISS_OFF);
-    IUFillSwitchVector(&SlewRateSP, SlewRateS, 5, getDeviceName(), "TELESCOPE_SLEW_RATE", "Centering Rate", 
-                        MOTION_TAB, IP_RW, ISR_1OFMANY, 0, IPS_IDLE);
+    IUFillSwitchVector(&SlewRateSP, SlewRateS, 5, getDeviceName(), "TELESCOPE_SLEW_RATE", "Centering Rate",
+                       MOTION_TAB, IP_RW, ISR_1OFMANY, 0, IPS_IDLE);
 
     // ============== GUIDE_TAB
     // Motion speed of axis when guiding
     IUFillSwitch(&GuideRateS[0], "25", "0.25x", ISS_OFF);
     IUFillSwitch(&GuideRateS[1], "50", "0.5x", ISS_ON);
     IUFillSwitch(&GuideRateS[2], "100", "1.0x", ISS_OFF);
-    IUFillSwitchVector(&GuideRateSP, GuideRateS, 3, getDeviceName(), "TELESCOPE_GUIDE_RATE", "Guide Rate", GUIDE_TAB, IP_RW, ISR_1OFMANY, 0, IPS_IDLE);
+    IUFillSwitchVector(&GuideRateSP, GuideRateS, 3, getDeviceName(), "TELESCOPE_GUIDE_RATE", "Guide Rate", GUIDE_TAB, IP_RW,
+                       ISR_1OFMANY, 0, IPS_IDLE);
     initGuiderProperties(getDeviceName(), GUIDE_TAB);
 
     // ============== OPTIONS_TAB
@@ -185,16 +153,6 @@ bool LX200_TeenAstro::initProperties()
     addAuxControls();
     setDriverInterface(getDriverInterface() | GUIDER_INTERFACE);
 
-#if 0
-    // dont' need to get from config file - mount already has location
-    double longitude = 0, latitude = 90;
-    // Get value from config file if it exists.
-    IUGetConfigNumber(getDeviceName(), "GEOGRAPHIC_COORD", "LONG", &longitude);
-    currentRA  = get_local_sidereal_time(longitude);
-    IUGetConfigNumber(getDeviceName(), "GEOGRAPHIC_COORD", "LAT", &latitude);
-    currentDEC = latitude > 0 ? 90 : -90;
-#endif
-
     return true;
 }
 void LX200_TeenAstro::ISGetProperties(const char *dev)
@@ -217,27 +175,27 @@ bool LX200_TeenAstro::updateProperties()
         deleteProperty("DOME_POLICY");
         deleteProperty("TELESCOPE_HAS_TRACK_RATE");
         // Main Control
-        defineNumber(&SlewAccuracyNP);
-        defineText(&ErrorStatusTP);
+        defineProperty(&SlewAccuracyNP);
+        defineProperty(&ErrorStatusTP);
         // Connection
         // Options
         // Motion Control
-        defineSwitch(&SlewRateSP);
-        defineSwitch(&GuideRateSP);
+        defineProperty(&SlewRateSP);
+        defineProperty(&GuideRateSP);
 
         // Site Management
-        defineSwitch(&ParkOptionSP);
-        defineSwitch(&SetHomeSP);
+        defineProperty(&ParkOptionSP);
+        defineProperty(&SetHomeSP);
 
-        defineSwitch(&SiteSP);
-        defineText(&SiteNameTP);
+        defineProperty(&SiteSP);
+        defineProperty(&SiteNameTP);
 
         // Guide
-        defineNumber(&GuideNSNP);
-        defineNumber(&GuideWENP);
+        defineProperty(&GuideNSNP);
+        defineProperty(&GuideWENP);
 
         // Firmware Data
-        defineText(&VersionTP);
+        defineProperty(&VersionTP);
         getBasicData();
     }
     else
@@ -245,7 +203,7 @@ bool LX200_TeenAstro::updateProperties()
         // Main Control
         deleteProperty(SlewAccuracyNP.name);
         deleteProperty(ErrorStatusTP.name);
-      // Connection
+        // Connection
         // Options
         // Motion Control
         deleteProperty(SlewRateSP.name);
@@ -340,7 +298,7 @@ bool LX200_TeenAstro::ReadScopeStatus()
  */
 void LX200_TeenAstro::handleStatusChange(void)
 {
-    LOGF_DEBUG ("Status Change: %s", OSStat);        
+    LOGF_DEBUG ("Status Change: %s", OSStat);
 
     if (OSStat[0] != OldOSStat[0])
     {
@@ -369,7 +327,7 @@ void LX200_TeenAstro::handleStatusChange(void)
         else
         {
             SetParked(false);
-//            SetTrackEnabled(false);     //disable since TeenAstro enables it by default            
+            //            SetTrackEnabled(false);     //disable since TeenAstro enables it by default
         }
     }
     // Byte 13 is pier side
@@ -386,14 +344,15 @@ void LX200_TeenAstro::handleStatusChange(void)
 
 /*
  * Mount Error status
- * 0:ERR_NONE,  1: ERR_MOTOR_FAULT, 2: ERR_ALT, 3: ERR_LIMIT_SENSE 
+ * 0:ERR_NONE,  1: ERR_MOTOR_FAULT, 2: ERR_ALT, 3: ERR_LIMIT_SENSE
  * 4: ERR_AXIS2,5: ERR_AZM, 6: ERR_UNDER_POLE, 7: ERR_MERIDIAN, 8: ERR_SYNC
  */
 void LX200_TeenAstro::updateMountStatus(char status)
 {
-    static const char *errCodes[9] = {"ERR_NONE",  "ERR_MOTOR_FAULT", "ERR_ALT", "ERR_LIMIT_SENSE", 
-                                "ERR_AXIS2", "ERR_AZM", "ERR_UNDER_POLE", "ERR_MERIDIAN", "ERR_SYNC"};
-    
+    static const char *errCodes[9] = {"ERR_NONE",  "ERR_MOTOR_FAULT", "ERR_ALT", "ERR_LIMIT_SENSE",
+                                      "ERR_AXIS2", "ERR_AZM", "ERR_UNDER_POLE", "ERR_MERIDIAN", "ERR_SYNC"
+                                     };
+
     if (status < '0' || status > '9')
     {
         return;
@@ -405,15 +364,15 @@ void LX200_TeenAstro::updateMountStatus(char status)
     else
     {
         ErrorStatusTP.s = IPS_ALERT;
-        TrackState = SCOPE_IDLE;     // Tell Ekos mount is not tracking anymore            
+        TrackState = SCOPE_IDLE;     // Tell Ekos mount is not tracking anymore
     }
-    IUSaveText(&ErrorStatusT[0], errCodes[status-'0']);
-    IDSetText(&ErrorStatusTP, nullptr);        
+    IUSaveText(&ErrorStatusT[0], errCodes[status - '0']);
+    IDSetText(&ErrorStatusTP, nullptr);
 }
 
 /*
- *  Goto target 
- *  Use standard lx200driver command (:Sr   #) 
+ *  Goto target
+ *  Use standard lx200driver command (:Sr   #)
  *  Set state to slewing
  */
 bool LX200_TeenAstro::Goto(double r, double d)
@@ -490,7 +449,7 @@ bool LX200_TeenAstro::SetTrackMode(uint8_t mode)
 }
 
 /*
- * Sync - synchronizes the telescope with its currently selected database object coordinates 
+ * Sync - synchronizes the telescope with its currently selected database object coordinates
  */
 bool LX200_TeenAstro::Sync(double ra, double dec)
 {
@@ -592,8 +551,9 @@ bool LX200_TeenAstro::Park()
 
         if (MovementNSSP.s == IPS_BUSY || MovementWESP.s == IPS_BUSY)
         {
-            MovementNSSP.s = MovementWESP.s = IPS_IDLE;
-            EqNP.s                          = IPS_IDLE;
+            MovementNSSP.s = IPS_IDLE;
+            MovementWESP.s = IPS_IDLE;
+            EqNP.s = IPS_IDLE;
             IUResetSwitch(&MovementNSSP);
             IUResetSwitch(&MovementWESP);
 
@@ -659,18 +619,19 @@ void LX200_TeenAstro::getBasicData()
             currentSiteNum = currentSiteIndex + 1;
             LOGF_INFO("Site number %d", currentSiteNum);
             getSiteName(PortFD, SiteNameTP.tp[0].text, currentSiteNum);
-            SiteNameTP.s = SiteSP.s = IPS_OK;
+            SiteNameTP.s = IPS_OK;
+            SiteSP.s = IPS_OK;
             IDSetText(&SiteNameTP, nullptr);
             IDSetSwitch(&SiteSP, nullptr);
             getLocation();                  // read site from TeenAstro
         }
         else
         {
-             LOG_ERROR("Error reading current site number");
+            LOG_ERROR("Error reading current site number");
         }
 
         // Get initial state and set switches
-        for (unsigned i=0;i<sizeof(OldOSStat);i++)
+        for (unsigned i = 0; i < sizeof(OldOSStat); i++)
             OldOSStat[i] = 'x';                         // reset old OS stat to force re-evaluation
         getCommandString(PortFD, OSStat, statusCommand);       // returns a string containing controller status
         handleStatusChange();
@@ -680,13 +641,13 @@ void LX200_TeenAstro::getBasicData()
         if (getSlewRate(&slewRateIndex))
         {
             LOGF_INFO("current slew rate : %d", slewRateIndex);
-            SlewRateS[slewRateIndex].s = ISS_ON; 
+            SlewRateS[slewRateIndex].s = ISS_ON;
             SlewRateSP.s = IPS_OK;
             IDSetSwitch(&SlewRateSP, nullptr);
         }
         else
         {
-             LOG_ERROR("Error reading current slew rate");
+            LOG_ERROR("Error reading current slew rate");
         }
 
         // Turn off tracking. (too much interaction with telescope.cpp if we try to keep the mount's current track state)
@@ -695,7 +656,7 @@ void LX200_TeenAstro::getBasicData()
             SetTrackEnabled(false);
         }
 
- 
+
         if (InitPark())
         {
             // If loading parking data is successful, we just set the default parking values.
@@ -711,7 +672,7 @@ void LX200_TeenAstro::getBasicData()
 
 /*
  * ISNewNumber: callback from user interface
- * 
+ *
  * when user has entered a number, handle it to store corresponding driver parameter
  *
  */
@@ -744,7 +705,7 @@ bool LX200_TeenAstro::ISNewNumber(const char *dev, const char *name, double valu
 
 /*
  * ISNewSwitch: callback from user interface
- * 
+ *
  * when user has entered a switch, handle it to store corresponding driver parameter
  *
  */
@@ -778,7 +739,7 @@ bool LX200_TeenAstro::ISNewSwitch(const char *dev, const char *name, ISState *st
             SetGuideRate(index);
             IDSetSwitch(&GuideRateSP, nullptr);
         }
-         // Sites
+        // Sites
         if (!strcmp(name, SiteSP.name))
         {
             if (IUUpdateSwitch(&SiteSP, states, names, n) < 0)
@@ -805,8 +766,9 @@ bool LX200_TeenAstro::ISNewSwitch(const char *dev, const char *name, ISState *st
             getLocation();
 
             LOGF_INFO("Setting site number %d", currentSiteNum);
-            SiteS[currentSiteNum-1].s = ISS_ON;
-            SiteNameTP.s = SiteSP.s = IPS_OK;
+            SiteS[currentSiteNum - 1].s = ISS_ON;
+            SiteNameTP.s = IPS_OK;
+            SiteSP.s = IPS_OK;
 
             IDSetText(&SiteNameTP, nullptr);
             IDSetSwitch(&SiteSP, nullptr);
@@ -845,7 +807,7 @@ bool LX200_TeenAstro::ISNewText(const char *dev, const char *name, char *texts[]
 
 
 /*
- * getLocalDate() to sendScopeTime() are copied from lx200telescope.cpp 
+ * getLocalDate() to sendScopeTime() are copied from lx200telescope.cpp
  */
 bool LX200_TeenAstro::getLocalDate(char *dateString)
 {
@@ -856,7 +818,7 @@ bool LX200_TeenAstro::getLocalDate(char *dateString)
     }
     else
     {
-       getCalendarDate(PortFD, dateString);
+        getCalendarDate(PortFD, dateString);
     }
 
     return true;
@@ -971,8 +933,8 @@ bool LX200_TeenAstro::sendScopeTime()
     return true;
 }
 /*
- * Called by INDI - not sure when 
- */ 
+ * Called by INDI - not sure when
+ */
 
 bool LX200_TeenAstro::sendScopeLocation()
 {
@@ -1016,7 +978,8 @@ bool LX200_TeenAstro::sendScopeLocation()
         else
             LocationNP.np[LOCATION_LONGITUDE].value = (dd - mm / 60.0) * -1.0;
     }
-    LOGF_DEBUG("Mount Controller Latitude: %g Longitude: %g", LocationN[LOCATION_LATITUDE].value, LocationN[LOCATION_LONGITUDE].value);
+    LOGF_DEBUG("Mount Controller Latitude: %g Longitude: %g", LocationN[LOCATION_LATITUDE].value,
+               LocationN[LOCATION_LONGITUDE].value);
 
     if (getSiteElevation(&elev))
     {
@@ -1037,7 +1000,7 @@ bool LX200_TeenAstro::sendScopeLocation()
  */
 bool LX200_TeenAstro::getSiteElevation(int *elevationP)
 {
-    if (getCommandInt(PortFD, elevationP, ":Ge#") !=0)
+    if (getCommandInt(PortFD, elevationP, ":Ge#") != 0)
         return false;
     return true;
 }
@@ -1047,7 +1010,7 @@ bool LX200_TeenAstro::getSiteElevation(int *elevationP)
  */
 bool LX200_TeenAstro::getSiteIndex(int *ndxP)
 {
-    if (getCommandInt(PortFD, ndxP, ":W?#") !=0)
+    if (getCommandInt(PortFD, ndxP, ":W?#") != 0)
         return false;
     return true;
 }
@@ -1058,7 +1021,7 @@ bool LX200_TeenAstro::getSiteIndex(int *ndxP)
  */
 bool LX200_TeenAstro::getSlewRate(int *ndxP)
 {
-    if (getCommandInt(PortFD, ndxP, ":GXRD#") !=0)
+    if (getCommandInt(PortFD, ndxP, ":GXRD#") != 0)
         return false;
     return true;
 }
@@ -1071,21 +1034,21 @@ bool LX200_TeenAstro::getSlewRate(int *ndxP)
 bool LX200_TeenAstro::setSite(int num)
 {
     char buf[10];
-    snprintf (buf, sizeof(buf), ":W%d#", num-1);
+    snprintf (buf, sizeof(buf), ":W%d#", num - 1);
     sendCommand(buf);
     return true;
 }
 
- /*
-  * setSiteElevation - not in Meade standard
-  */
+/*
+ * setSiteElevation - not in Meade standard
+ */
 bool LX200_TeenAstro::setSiteElevation(double elevation)
 {
     char buf[20];
     snprintf (buf, sizeof(buf), ":Se%+4d#", static_cast<int>(elevation));
     sendCommand(buf);
     return true;
-}       
+}
 
 
 /*
@@ -1226,8 +1189,12 @@ bool LX200_TeenAstro::Move(TDirection dir, TelescopeMotionCommand cmd)
 {
     switch (cmd)
     {
-        case MOTION_START: MoveTo(PortFD, dir);         break;
-        case MOTION_STOP:  HaltMovement(PortFD, dir);   break;
+        case MOTION_START:
+            MoveTo(PortFD, dir);
+            break;
+        case MOTION_STOP:
+            HaltMovement(PortFD, dir);
+            break;
     }
     return true;
 }
@@ -1325,7 +1292,7 @@ void LX200_TeenAstro::slewError(int slewCode)
         IDSetNumber(&EqNP, "Slew failed.");
 }
 /*
- *  Enable or disable sidereal tracking (events handled by inditelescope) 
+ *  Enable or disable sidereal tracking (events handled by inditelescope)
  */
 bool LX200_TeenAstro::SetTrackEnabled(bool enabled)
 {
@@ -1357,7 +1324,7 @@ bool LX200_TeenAstro::selectSlewRate(int index)
 
 /*
  * Used instead of getCommandString when response is not terminated with '#'
- * 
+ *
  */
 void LX200_TeenAstro::sendCommand(const char *cmd)
 {
