@@ -2,6 +2,7 @@
  * Standard LX200 implementation.
 
    Copyright (C) 2003 - 2018 Jasem Mutlaq (mutlaqja@ikarustech.com)
+   Minor changes Copyright (C) 2021 James Lancaster
 
    This library is free software;
    you can redistribute it and / or
@@ -102,22 +103,27 @@ bool LX200Telescope::initProperties()
     {
         IUFillNumber(&TrackFreqN[0], "trackFreq", "Freq", "%g", 56.4, 60.1, 0.1, 60.1);
     }
-    IUFillNumberVector(&TrackingFreqNP, TrackFreqN, 1, getDeviceName(), "Tracking Frequency", "", MOTION_TAB, IP_RW, 0,
+    IUFillNumberVector(&TrackFreqNP, TrackFreqN, 1, getDeviceName(), "Tracking Frequency", "", MOTION_TAB, IP_RW, 0,
                        IPS_IDLE);
 
-    IUFillSwitch(&UsePulseCmdS[0], "Off", "", ISS_OFF);
-    IUFillSwitch(&UsePulseCmdS[1], "On", "", ISS_ON);
+    IUFillSwitch(&UsePulseCmdS[0], "Off", "Off", ISS_OFF);
+    IUFillSwitch(&UsePulseCmdS[1], "On", "On", ISS_ON);
     IUFillSwitchVector(&UsePulseCmdSP, UsePulseCmdS, 2, getDeviceName(), "Use Pulse Cmd", "", MAIN_CONTROL_TAB, IP_RW,
                        ISR_1OFMANY, 0, IPS_IDLE);
 
-    IUFillSwitch(&SiteS[0], "Site 1", "", ISS_ON);
-    IUFillSwitch(&SiteS[1], "Site 2", "", ISS_OFF);
-    IUFillSwitch(&SiteS[2], "Site 3", "", ISS_OFF);
-    IUFillSwitch(&SiteS[3], "Site 4", "", ISS_OFF);
+    int selectedSite = 0;
+    IUGetConfigOnSwitchIndex(getDeviceName(), "Sites", &selectedSite);
+    IUFillSwitch(&SiteS[0], "Site 1", "Site 1", selectedSite == 0 ? ISS_ON : ISS_OFF);
+    IUFillSwitch(&SiteS[1], "Site 2", "Site 2", selectedSite == 1 ? ISS_ON : ISS_OFF);
+    IUFillSwitch(&SiteS[2], "Site 3", "Site 3", selectedSite == 2 ? ISS_ON : ISS_OFF);
+    IUFillSwitch(&SiteS[3], "Site 4", "Site 4", selectedSite == 3 ? ISS_ON : ISS_OFF);
     IUFillSwitchVector(&SiteSP, SiteS, 4, getDeviceName(), "Sites", "", SITE_TAB, IP_RW, ISR_1OFMANY, 0, IPS_IDLE);
 
-    IUFillText(&SiteNameT[0], "Name", "", "");
-    IUFillTextVector(&SiteNameTP, SiteNameT, 1, getDeviceName(), "Site Name", "", SITE_TAB, IP_RW, 0, IPS_IDLE);
+    char siteName[64] = {"NA"};
+    IUGetConfigText(getDeviceName(), "Site Name", "Name", siteName, 64);
+    IUFillText(&SiteNameT[0], "Name", "Name", siteName);
+    IUFillTextVector(&SiteNameTP, SiteNameT, 1, getDeviceName(), "Site Name", "Site Name", SITE_TAB, IP_RW, 0, IPS_IDLE);
+
 
     if (genericCapability & LX200_HAS_FOCUS)
     {
@@ -145,13 +151,10 @@ bool LX200Telescope::initProperties()
 
     setDriverInterface(getDriverInterface() | GUIDER_INTERFACE);
 
-    double longitude = 0, latitude = 90;
-    // Get value from config file if it exists.
-    IUGetConfigNumber(getDeviceName(), "GEOGRAPHIC_COORD", "LONG", &longitude);
-    currentRA  = get_local_sidereal_time(longitude);
-    IUGetConfigNumber(getDeviceName(), "GEOGRAPHIC_COORD", "LAT", &latitude);
-    currentDEC = latitude > 0 ? 90 : -90;
-
+    if (m_Location.longitude > 0)
+        currentRA  = get_local_sidereal_time(m_Location.longitude);
+    if (m_Location.latitude != 0)
+        currentDEC = m_Location.latitude > 0 ? 90 : -90;
     return true;
 }
 
@@ -166,28 +169,28 @@ void LX200Telescope::ISGetProperties(const char *dev)
     if (isConnected())
     {
         if (genericCapability & LX200_HAS_ALIGNMENT_TYPE)
-            defineSwitch(&AlignmentSP);
+            defineProperty(&AlignmentSP);
 
         if (genericCapability & LX200_HAS_TRACKING_FREQ)
-            defineNumber(&TrackingFreqNP);
+            defineProperty(&TrackFreqNP);
 
         if (genericCapability & LX200_HAS_PULSE_GUIDING)
-            defineSwitch(&UsePulseCmdSP);
+            defineProperty(&UsePulseCmdSP);
 
         if (genericCapability & LX200_HAS_SITES)
         {
-            defineSwitch(&SiteSP);
-            defineText(&SiteNameTP);
+            defineProperty(&SiteSP);
+            defineProperty(&SiteNameTP);
         }
 
-        defineNumber(&GuideNSNP);
-        defineNumber(&GuideWENP);
+        defineProperty(&GuideNSNP);
+        defineProperty(&GuideWENP);
 
         if (genericCapability & LX200_HAS_FOCUS)
         {
-            defineSwitch(&FocusMotionSP);
-            defineNumber(&FocusTimerNP);
-            defineSwitch(&FocusModeSP);
+            defineProperty(&FocusMotionSP);
+            defineProperty(&FocusTimerNP);
+            defineProperty(&FocusModeSP);
         }
     }
     */
@@ -200,27 +203,27 @@ bool LX200Telescope::updateProperties()
     if (isConnected())
     {
         if (genericCapability & LX200_HAS_ALIGNMENT_TYPE)
-            defineSwitch(&AlignmentSP);
+            defineProperty(&AlignmentSP);
 
         if (genericCapability & LX200_HAS_TRACKING_FREQ)
-            defineNumber(&TrackingFreqNP);
+            defineProperty(&TrackFreqNP);
 
         if (genericCapability & LX200_HAS_PULSE_GUIDING)
-            defineSwitch(&UsePulseCmdSP);
+            defineProperty(&UsePulseCmdSP);
 
         if (genericCapability & LX200_HAS_SITES)
         {
-            defineSwitch(&SiteSP);
-            defineText(&SiteNameTP);
+            defineProperty(&SiteSP);
+            defineProperty(&SiteNameTP);
         }
 
-        defineNumber(&GuideNSNP);
-        defineNumber(&GuideWENP);
+        defineProperty(&GuideNSNP);
+        defineProperty(&GuideWENP);
 
         if (genericCapability & LX200_HAS_FOCUS)
         {
             FI::updateProperties();
-            //defineSwitch(&FocusModeSP);
+            //defineProperty(&FocusModeSP);
         }
 
         getBasicData();
@@ -231,7 +234,7 @@ bool LX200Telescope::updateProperties()
             deleteProperty(AlignmentSP.name);
 
         if (genericCapability & LX200_HAS_TRACKING_FREQ)
-            deleteProperty(TrackingFreqNP.name);
+            deleteProperty(TrackFreqNP.name);
 
         if (genericCapability & LX200_HAS_PULSE_GUIDING)
             deleteProperty(UsePulseCmdSP.name);
@@ -362,8 +365,9 @@ bool LX200Telescope::Goto(double ra, double dec)
 
         if (MovementNSSP.s == IPS_BUSY || MovementWESP.s == IPS_BUSY)
         {
-            MovementNSSP.s = MovementWESP.s = IPS_IDLE;
-            EqNP.s                          = IPS_IDLE;
+            MovementNSSP.s = IPS_IDLE;
+            MovementWESP.s = IPS_IDLE;
+            EqNP.s = IPS_IDLE;
             IUResetSwitch(&MovementNSSP);
             IUResetSwitch(&MovementWESP);
             IDSetSwitch(&MovementNSSP, nullptr);
@@ -454,8 +458,9 @@ bool LX200Telescope::Park()
 
             if (MovementNSSP.s == IPS_BUSY || MovementWESP.s == IPS_BUSY)
             {
-                MovementNSSP.s = MovementWESP.s = IPS_IDLE;
-                EqNP.s                          = IPS_IDLE;
+                MovementNSSP.s = IPS_IDLE;
+                MovementWESP.s = IPS_IDLE;
+                EqNP.s = IPS_IDLE;
                 IUResetSwitch(&MovementNSSP);
                 IUResetSwitch(&MovementWESP);
 
@@ -475,7 +480,6 @@ bool LX200Telescope::Park()
         }
     }
 
-    ParkSP.s   = IPS_BUSY;
     TrackState = SCOPE_PARKING;
     LOG_INFO("Parking telescope in progress...");
     return true;
@@ -637,19 +641,24 @@ bool LX200Telescope::updateLocation(double latitude, double longitude, double el
 {
     INDI_UNUSED(elevation);
 
-    if (isSimulation())
-        return true;
+    // JM 2021-04-10: MUST convert from INDI longitude to standard longitude.
+    // DO NOT REMOVE
+    if (longitude > 180)
+        longitude = longitude - 360;
 
-    if (!isSimulation() && setSiteLongitude(PortFD, longitude) < 0)
+    if (!isSimulation())
     {
-        LOG_ERROR("Error setting site longitude coordinates");
-        return false;
-    }
+        if (setSiteLongitude(PortFD, longitude) < 0)
+        {
+            LOG_ERROR("Error setting site longitude coordinates");
+            return false;
+        }
 
-    if (!isSimulation() && setSiteLatitude(PortFD, latitude) < 0)
-    {
-        LOG_ERROR("Error setting site latitude coordinates");
-        return false;
+        if (setSiteLatitude(PortFD, latitude) < 0)
+        {
+            LOG_ERROR("Error setting site latitude coordinates");
+            return false;
+        }
     }
 
     char l[MAXINDINAME] = {0}, L[MAXINDINAME] = {0};
@@ -657,7 +666,8 @@ bool LX200Telescope::updateLocation(double latitude, double longitude, double el
     fs_sexa(L, longitude, 2, 36000);
 
     // Choose WGS 84, also known as EPSG:4326 for latitude/longitude ordering
-    LOGF_INFO("Site location in the mount updated to Latitude %.12s (%g) Longitude %.12s (%g) (Longitude sign in carthography format)", l, latitude, L, longitude);
+    LOGF_INFO("Site location in the mount updated to Latitude %.12s (%g) Longitude %.12s (%g) (Longitude sign in carthography format)",
+              l, latitude, L, longitude);
 
     return true;
 }
@@ -696,20 +706,20 @@ bool LX200Telescope::ISNewNumber(const char *dev, const char *name, double value
         }
 
         // Update Frequency
-        if (!strcmp(name, TrackingFreqNP.name))
+        if (!strcmp(name, TrackFreqNP.name))
         {
             LOGF_DEBUG("Trying to set track freq of: %04.1f", values[0]);
             if (genericCapability & LX200_HAS_PRECISE_TRACKING_FREQ)
             {
                 if (!isSimulation() && setPreciseTrackFreq(PortFD, values[0]) < 0)
                 {
-                    TrackingFreqNP.s = IPS_ALERT;
-                    IDSetNumber(&TrackingFreqNP, "Error setting tracking frequency");
+                    TrackFreqNP.s = IPS_ALERT;
+                    IDSetNumber(&TrackFreqNP, "Error setting tracking frequency");
                     return false;
                 }
-                TrackingFreqNP.s           = IPS_OK;
-                TrackingFreqNP.np[0].value = values[0];
-                IDSetNumber(&TrackingFreqNP, "Tracking frequency set to %8.5f", values[0]);
+                TrackFreqNP.s           = IPS_OK;
+                TrackFreqNP.np[0].value = values[0];
+                IDSetNumber(&TrackFreqNP, "Tracking frequency set to %8.5f", values[0]);
             }
             else
             {
@@ -722,18 +732,18 @@ bool LX200Telescope::ISNewNumber(const char *dev, const char *name, double value
                     LOGF_DEBUG("Trying to set track freq of: %f\n", values[0]);
                     if (!isSimulation() && setTrackFreq(PortFD, values[0]) < 0)
                     {
-                        TrackingFreqNP.s = IPS_ALERT;
-                        IDSetNumber(&TrackingFreqNP, "Error setting tracking frequency");
+                        TrackFreqNP.s = IPS_ALERT;
+                        IDSetNumber(&TrackFreqNP, "Error setting tracking frequency");
                         return false;
                     }
-                    TrackingFreqNP.s           = IPS_OK;
-                    IDSetNumber(&TrackingFreqNP, "Error setting tracking frequency");
+                    TrackFreqNP.s           = IPS_OK;
+                    IDSetNumber(&TrackFreqNP, "Error setting tracking frequency");
                     return false;
                 }
 
-                TrackingFreqNP.s           = IPS_OK;
-                TrackingFreqNP.np[0].value = values[0];
-                IDSetNumber(&TrackingFreqNP, "Tracking frequency set to %04.1f", values[0]);
+                TrackFreqNP.s           = IPS_OK;
+                TrackFreqNP.np[0].value = values[0];
+                IDSetNumber(&TrackFreqNP, "Tracking frequency set to %04.1f", values[0]);
             }
 
             if (trackingMode != LX200_TRACK_MANUAL)
@@ -820,16 +830,21 @@ bool LX200Telescope::ISNewSwitch(const char *dev, const char *name, ISState *sta
                 IDSetSwitch(&SiteSP, "Error selecting sites.");
                 return false;
             }
-
+            char siteName[64] = {0};
             if (isSimulation())
+            {
                 IUSaveText(&SiteNameTP.tp[0], "Sample Site");
+            }
             else
-                getSiteName(PortFD, SiteNameTP.tp[0].text, currentSiteNum);
-
+            {
+                getSiteName(PortFD, siteName, currentSiteNum);
+                IUSaveText(&SiteNameT[0], siteName);
+            }
             if (GetTelescopeCapability() & TELESCOPE_HAS_LOCATION)
                 sendScopeLocation();
 
-            SiteNameTP.s = SiteSP.s = IPS_OK;
+            SiteNameTP.s = IPS_OK;
+            SiteSP.s = IPS_OK;
 
             IDSetText(&SiteNameTP, nullptr);
             IDSetSwitch(&SiteSP, nullptr);
@@ -940,7 +955,7 @@ bool LX200Telescope::SetTrackMode(uint8_t mode)
     if (rc &&  (genericCapability & LX200_HAS_TRACKING_FREQ))
     {
         getTrackFreq(PortFD, &TrackFreqN[0].value);
-        IDSetNumber(&TrackingFreqNP, nullptr);
+        IDSetNumber(&TrackFreqNP, nullptr);
     }
     return rc;
 }
@@ -1171,12 +1186,16 @@ void LX200Telescope::getBasicData()
 
         if (genericCapability & LX200_HAS_SITES)
         {
-            SiteNameT[0].text = new char[64];
-
-            if (getSiteName(PortFD, SiteNameT[0].text, currentSiteNum) < 0)
+            char siteName[64] = {0};
+            if (getSiteName(PortFD, siteName, currentSiteNum) < 0)
+            {
                 LOG_ERROR("Failed to get site name from device");
+            }
             else
+            {
+                IUSaveText(&SiteNameT[0], siteName);
                 IDSetText(&SiteNameTP, nullptr);
+            }
         }
 
 
@@ -1185,7 +1204,7 @@ void LX200Telescope::getBasicData()
             if (getTrackFreq(PortFD, &TrackFreqN[0].value) < 0)
                 LOG_ERROR("Failed to get tracking frequency from device.");
             else
-                IDSetNumber(&TrackingFreqNP, nullptr);
+                IDSetNumber(&TrackFreqNP, nullptr);
         }
 
     }
@@ -1379,7 +1398,7 @@ bool LX200Telescope::sendScopeLocation()
     }
     else
     {
-        snprintf(lat_sexagesimal, MAXINDIFORMAT,"%02d:%02d:%02.1lf", lat_dd, lat_mm, lat_ssf);
+        snprintf(lat_sexagesimal, MAXINDIFORMAT, "%02d:%02d:%02.1lf", lat_dd, lat_mm, lat_ssf);
         f_scansexa(lat_sexagesimal, &(LocationNP.np[LOCATION_LATITUDE].value));
     }
 
@@ -1390,7 +1409,7 @@ bool LX200Telescope::sendScopeLocation()
     }
     else
     {
-        snprintf(lng_sexagesimal, MAXINDIFORMAT,"%02d:%02d:%02.1lf", long_dd, long_mm, long_ssf);
+        snprintf(lng_sexagesimal, MAXINDIFORMAT, "%02d:%02d:%02.1lf", long_dd, long_mm, long_ssf);
         f_scansexa(lng_sexagesimal, &(LocationNP.np[LOCATION_LONGITUDE].value));
     }
 
@@ -1655,3 +1674,4 @@ bool LX200Telescope::SetFocuserSpeed(int speed)
 {
     return (setFocuserSpeedMode(PortFD, speed) == 0);
 }
+
