@@ -31,43 +31,6 @@
 
 static std::unique_ptr<Rainbow> scope(new Rainbow());
 
-void ISGetProperties(const char *dev)
-{
-    scope->ISGetProperties(dev);
-}
-
-void ISNewSwitch(const char *dev, const char *name, ISState *states, char *names[], int n)
-{
-    scope->ISNewSwitch(dev, name, states, names, n);
-}
-
-void ISNewText(const char *dev, const char *name, char *texts[], char *names[], int n)
-{
-    scope->ISNewText(dev, name, texts, names, n);
-}
-
-void ISNewNumber(const char *dev, const char *name, double values[], char *names[], int n)
-{
-    scope->ISNewNumber(dev, name, values, names, n);
-}
-
-void ISNewBLOB(const char *dev, const char *name, int sizes[], int blobsizes[], char *blobs[], char *formats[],
-               char *names[], int n)
-{
-    INDI_UNUSED(dev);
-    INDI_UNUSED(name);
-    INDI_UNUSED(sizes);
-    INDI_UNUSED(blobsizes);
-    INDI_UNUSED(blobs);
-    INDI_UNUSED(formats);
-    INDI_UNUSED(names);
-    INDI_UNUSED(n);
-}
-void ISSnoopDevice(XMLEle *root)
-{
-    scope->ISSnoopDevice(root);
-}
-
 Rainbow::Rainbow() : INDI::Telescope ()
 {
     setVersion(1, 1);
@@ -107,6 +70,38 @@ bool Rainbow::initProperties()
     IUFillSwitchVector(&HomeSP, HomeS, 1, getDeviceName(), "HOME", "Homing", MAIN_CONTROL_TAB, IP_RW, ISR_ATMOST1, 60,
                        IPS_IDLE);
 
+    // Star Alignment on Sync
+    IUFillSwitch(&SaveAlignBeforeSyncS[STAR_ALIGNMENT_ENABLED], "STAR_ALIGNMENT_ENABLED", "Enabled", ISS_OFF);
+    IUFillSwitch(&SaveAlignBeforeSyncS[STAR_ALIGNMENT_DISABLED], "STAR_ALIGNMENT_DISABLED", "Disabled", ISS_ON);
+    IUFillSwitchVector(&SaveAlignBeforeSyncSP, SaveAlignBeforeSyncS, 2, getDeviceName(),
+                           "STAR_ALIGNMENT", "Star Alignment", ALIGNMENT_TAB, IP_RW, ISR_1OFMANY, 60, IPS_IDLE);
+
+    // Mount's versions
+    IUFillText(&RSTVersionsT[FIRMWARE], "FIRMWARE", "Firmware Version", "");
+    IUFillText(&RSTVersionsT[SERIALNUMBER], "SERIALNUMBER", "Serial Number", "");
+    IUFillTextVector(&RSTVersionsTP, RSTVersionsT, 2, getDeviceName(), "RST_VERSIONS", "Versions", GENERAL_INFO_TAB, IP_RO, 0,
+                       IPS_IDLE);
+
+    // Pull Voltage & Temperatures (possible to disable to reduce load on Serial bus)
+    IUFillSwitch(&PullVoltTempS[PULL_VOLTTEMP_ENABLED], "PULL_VOLTTEMP_ENABLED", "Enabled", ISS_OFF);
+    IUFillSwitch(&PullVoltTempS[PULL_VOLTTEMP_DISABLED], "PULL_VOLTTEMP_DISABLED", "Disabled", ISS_ON);
+    IUFillSwitchVector(&PullVoltTempSP, PullVoltTempS, 2, getDeviceName(),
+                           "PULL_VOLTTEMP", "Pull V. & T.", GENERAL_INFO_TAB, IP_RW, ISR_1OFMANY, 60, IPS_IDLE);
+
+    // Voltage & Temperatures
+    IUFillNumber(&RSTVoltTempN[VOLTAGE], "VOLTAGE", "Input Voltage (V)", "%2.1f", 0, 20., 0., 0.);
+    IUFillNumber(&RSTVoltTempN[BOARD_TEMPERATURE], "BOARD_TEMPERATURE", "Board Temp. (°C)", "%2.1f", -50, 70., 0., 0.);
+    IUFillNumber(&RSTVoltTempN[RA_M_TEMPERATURE], "RA_M_TEMPERATURE", "RA-Motor Temp. (°C)", "%2.1f", -50, 70., 0., 0.);
+    IUFillNumber(&RSTVoltTempN[DE_M_TEMPERATURE], "DE_M_TEMPERATURE", "DEC-Motor Temp. (°C)", "%2.1f", -50, 70., 0., 0.);
+    IUFillNumberVector(&RSTVoltTempNP, RSTVoltTempN, 4, getDeviceName(), "RST_VOLT_TEMP", "Volt. & Temp.", GENERAL_INFO_TAB, IP_RO, 0,
+                       IPS_IDLE);
+
+    // Motor powers
+    IUFillNumber(&RSTMotorPowN[RA_M_POWER], "RA_M_POWER", "RA-Motor (%)", "%3.1f", 0, 100., 0., 0.);
+    IUFillNumber(&RSTMotorPowN[DE_M_POWER], "DE_M_POWER", "DE-Motor (%)", "%3.1f", 0, 100., 0., 0.);
+    IUFillNumberVector(&RSTMotorPowNP, RSTMotorPowN, 2, getDeviceName(), "RST_MOTOR_POW", "Motor Power", GENERAL_INFO_TAB, IP_RO, 0,
+                       IPS_IDLE);
+
     // Horizontal Coords
     IUFillNumber(&HorizontalCoordsN[AXIS_AZ], "AZ", "Az D:M:S", "%10.6m", 0.0, 360.0, 0.0, 0);
     IUFillNumber(&HorizontalCoordsN[AXIS_ALT], "ALT", "Alt  D:M:S", "%10.6m", -90., 90.0, 0.0, 0);
@@ -122,6 +117,12 @@ bool Rainbow::initProperties()
     IUFillNumberVector(&GuideRateNP, GuideRateN, 1, getDeviceName(), "GUIDE_RATE", "Guiding Rate", MOTION_TAB, IP_RW, 0,
                        IPS_IDLE);
 
+    IUFillNumber(&SlewSpeedsN[SLEW_SPEED_MAX], "SLEW_SPEED_MAX", "Max (x Siderial)", "%g", 0, 2000, 0, 0);
+    IUFillNumber(&SlewSpeedsN[SLEW_SPEED_FIND], "SLEW_SPEED_FIND", "Find (x Siderial)", "%g", 0, 2000, 0, 0);
+    IUFillNumber(&SlewSpeedsN[SLEW_SPEED_CENTERING], "SLEW_SPEED_CENTERING", "Centering (x Siderial)", "%g", 0, 2000, 0, 0);
+    IUFillNumberVector(&SlewSpeedsNP, SlewSpeedsN, 3, getDeviceName(), "SLEW_SPEED", "Slew speed", MOTION_TAB, IP_RW, 0,
+                       IPS_IDLE);
+
     setDriverInterface(getDriverInterface() | GUIDER_INTERFACE);
 
     initGuiderProperties(getDeviceName(), MOTION_TAB);
@@ -140,15 +141,21 @@ bool Rainbow::updateProperties()
 
     if (isConnected())
     {
+        defineProperty(&HorizontalCoordsNP);
+        defineProperty(&HomeSP);
+
+        defineProperty(&GuideNSNP);
+        defineProperty(&GuideWENP);
+        defineProperty(&GuideRateNP);
+        defineProperty(&SlewSpeedsNP);
+        
+        defineProperty(&SaveAlignBeforeSyncSP);
+        defineProperty(&RSTVersionsTP);
+        defineProperty(&PullVoltTempSP);
+        defineProperty(&RSTVoltTempNP);
+        defineProperty(&RSTMotorPowNP);
+
         getStartupStatus();
-
-        defineNumber(&HorizontalCoordsNP);
-        defineSwitch(&HomeSP);
-
-        defineNumber(&GuideNSNP);
-        defineNumber(&GuideWENP);
-        defineNumber(&GuideRateNP);
-
     }
     else
     {
@@ -158,6 +165,13 @@ bool Rainbow::updateProperties()
         deleteProperty(GuideNSNP.name);
         deleteProperty(GuideWENP.name);
         deleteProperty(GuideRateNP.name);
+        deleteProperty(SlewSpeedsNP.name);
+
+        deleteProperty(SaveAlignBeforeSyncSP.name);
+        deleteProperty(RSTVersionsTP.name);
+        deleteProperty(PullVoltTempSP.name);
+        deleteProperty(RSTVoltTempNP.name);
+        deleteProperty(RSTMotorPowNP.name);
     }
 
     return true;
@@ -230,6 +244,24 @@ bool Rainbow::ISNewNumber(const char *dev, const char *name, double values[], ch
             IDSetNumber(&GuideRateNP, nullptr);
             return true;
         }
+        // Slew speeds
+        else if (!strcmp(name, SlewSpeedsNP.name))
+        {
+
+                if (setSlewSpeedVal(SLEW_SPEED_MAX, values[SLEW_SPEED_MAX])
+                    && setSlewSpeedVal(SLEW_SPEED_FIND, values[SLEW_SPEED_FIND])
+                    && setSlewSpeedVal(SLEW_SPEED_CENTERING, values[SLEW_SPEED_CENTERING])
+                )
+                {
+                    IUUpdateNumber(&SlewSpeedsNP, values, names, n);
+                    SlewSpeedsNP.s = IPS_OK;
+                    LOG_INFO("Slew speeds updated.");
+                }
+                else
+                    SlewSpeedsNP.s = IPS_ALERT;
+            IDSetNumber(&SlewSpeedsNP, nullptr);
+            return true;
+        }
         else
             processGuiderProperties(name, values, names, n);
     }
@@ -267,6 +299,41 @@ bool Rainbow::ISNewSwitch(const char *dev, const char *name, ISState *states, ch
             IDSetSwitch(&HomeSP, nullptr);
             return true;
         }
+        // Star Align
+        else if (!strcmp(SaveAlignBeforeSyncSP.name, name)) {
+
+            IUUpdateSwitch(&SaveAlignBeforeSyncSP, states, names, n);
+            SaveAlignBeforeSyncSP.s = IPS_OK;
+            saveConfig(true, SaveAlignBeforeSyncSP.name);
+            IDSetSwitch(&SaveAlignBeforeSyncSP, nullptr);
+            return true;
+        }
+        // Pull RST's Voltage and Temperatures
+        else if (!strcmp(PullVoltTempSP.name, name)) {
+            IUUpdateSwitch(&PullVoltTempSP, states, names, n);
+            if (PullVoltTempS[PULL_VOLTTEMP_DISABLED].s == ISS_ON)
+            {
+                RSTVoltTempN[VOLTAGE].value = 0.;
+                RSTVoltTempN[BOARD_TEMPERATURE].value = 0.;
+                RSTVoltTempN[RA_M_TEMPERATURE].value = 0.;
+                RSTVoltTempN[DE_M_TEMPERATURE].value = 0.;
+                RSTMotorPowN[RA_M_POWER].value = 0.;
+                RSTMotorPowN[DE_M_POWER].value = 0.;
+                RSTVoltTempNP.s = IPS_IDLE;
+                IDSetNumber(&RSTVoltTempNP, nullptr);
+                RSTMotorPowNP.s = IPS_IDLE;
+                IDSetNumber(&RSTMotorPowNP, nullptr);
+                PullVoltTempSP.s = IPS_IDLE;
+                LOG_INFO("Pulling RST's Voltage and Temperatures set to: off");
+            }
+            else
+            {
+                PullVoltTempSP.s = IPS_OK;
+                LOG_INFO("Pulling RST's Voltage and Temperatures set to: on");
+            }
+            IDSetSwitch(&PullVoltTempSP, nullptr);
+            return true;
+        }
 
     }
 
@@ -278,12 +345,15 @@ bool Rainbow::ISNewSwitch(const char *dev, const char *name, ISState *states, ch
 /////////////////////////////////////////////////////////////////////////////////////
 void Rainbow::getStartupStatus()
 {
+    getFirmwareVersion();
     LOGF_INFO("Detected firmware %s", m_Version.c_str());
 
     if (getTrackingState())
         IDSetSwitch(&TrackStateSP, nullptr);
     if (getGuideRate())
         IDSetNumber(&GuideRateNP, nullptr);
+    if (getSlewSpeedVal(SLEW_SPEED_MAX) && (getSlewSpeedVal(SLEW_SPEED_FIND) && getSlewSpeedVal(SLEW_SPEED_CENTERING)))
+        IDSetNumber(&SlewSpeedsNP, nullptr);
 
     double longitude = 0, latitude = 90;
     // Get value from config file if it exists.
@@ -363,12 +433,29 @@ bool Rainbow::Handshake()
 bool Rainbow::getFirmwareVersion()
 {
     char res[DRIVER_LEN] = {0};
+    char fw[6], sn[6];
 
     if (sendCommand(":AV#", res) == false)
         return false;
 
     // :AV190905 --> 190905
     m_Version = std::string(res + 3);
+
+    // Write Firmware and Serial Number to INDI Control Panel
+    if (isConnected()) // executed onlyafter properties are updated
+    {
+        sscanf(res + 3, "%6s", fw);
+        memset(res, 0, sizeof res);
+        IUSaveText(&RSTVersionsT[FIRMWARE],fw);
+
+        if (sendCommand(":AS#", res) == false)
+            return false;
+        sscanf(res + 3, "%6s", sn);
+        IUSaveText(&RSTVersionsT[SERIALNUMBER],sn);
+
+        RSTVersionsTP.s = IPS_OK;
+        IDSetText(&RSTVersionsTP, nullptr);
+    }
     return true;
 }
 
@@ -431,7 +518,41 @@ bool Rainbow::getGuideRate()
     memcpy(rate, res + 5, 3);
 
     GuideRateN[0].value = std::stod(rate);
+    GuideRateNP.s = IPS_OK;
+    return true;
+}
 
+/////////////////////////////////////////////////////////////////////////////////////
+///
+/////////////////////////////////////////////////////////////////////////////////////
+bool Rainbow::setSlewSpeedVal(int speedtype, double rate)
+{
+    char cmd[DRIVER_LEN] = {0};
+    snprintf(cmd, DRIVER_LEN, ":Cu%d=%g#",speedtype == SLEW_SPEED_MAX ? 3 : (speedtype == SLEW_SPEED_FIND ? 2 : 1) , rate);
+    LOGF_INFO("slew speed set to enum %d and value %g", speedtype, rate);
+    return sendCommand(cmd);
+}
+
+/////////////////////////////////////////////////////////////////////////////////////
+///
+/////////////////////////////////////////////////////////////////////////////////////
+bool Rainbow::getSlewSpeedVal(int speedtype)
+{
+    char res[DRIVER_LEN] = {0};
+    char rate[4] = {0};
+    char cmd[DRIVER_LEN] = {0};
+    snprintf(cmd, DRIVER_LEN, ":CU%d#", speedtype == SLEW_SPEED_MAX ? 3 : (speedtype == SLEW_SPEED_FIND ? 2 : 1));
+    if (sendCommand(cmd, res))
+            {
+                memcpy(rate, res + 5, 4);
+                SlewSpeedsN[speedtype].value = std::stod(rate);
+                SlewSpeedsNP.s = IPS_OK;
+            }
+            else
+            {
+                SlewSpeedsNP.s = IPS_ALERT;
+                return false;
+            }
     return true;
 }
 
@@ -632,6 +753,38 @@ bool Rainbow::ReadScopeStatus()
     IDSetNumber(&HorizontalCoordsNP, nullptr);
 
     NewRaDec(m_CurrentRA, m_CurrentDE);
+
+    if (PullVoltTempS[PULL_VOLTTEMP_ENABLED].s == ISS_ON)
+    {
+        // Get Voltage and Temperatures
+        char res[DRIVER_LEN] = {0};
+        float v, bT, rT, dT;
+        if (sendCommand(":Cv#", res) == false)
+            return false;
+        sscanf(res + 3, "%f", &v);
+        memset(res, 0, sizeof res);
+        RSTVoltTempN[VOLTAGE].value = v;
+        if (sendCommand(":CT#", res) == false)
+            return false;
+        sscanf(res + 3, "%f|%f|%f", &bT, &rT, &dT);
+        memset(res, 0, sizeof res);
+        RSTVoltTempN[BOARD_TEMPERATURE].value = bT;
+        RSTVoltTempN[RA_M_TEMPERATURE].value = rT;
+        RSTVoltTempN[DE_M_TEMPERATURE].value = dT;
+        RSTVoltTempNP.s = IPS_OK;
+        IDSetNumber(&RSTVoltTempNP, nullptr);
+
+        // Get Motor Powers
+        float rP, dP;
+        if (sendCommand(":CP#", res) == false)
+            return false;
+        sscanf(res + 3, "%f|%f", &dP, &rP);
+        memset(res, 0, sizeof res);
+        RSTMotorPowN[RA_M_POWER].value = rP;
+        RSTMotorPowN[DE_M_POWER].value = dP;
+        RSTMotorPowNP.s = IPS_OK;
+        IDSetNumber(&RSTMotorPowNP, nullptr);
+    }
     return true;
 }
 
@@ -663,8 +816,9 @@ bool Rainbow::Goto(double ra, double dec)
 
         if (MovementNSSP.s == IPS_BUSY || MovementWESP.s == IPS_BUSY)
         {
-            MovementNSSP.s = MovementWESP.s = IPS_IDLE;
-            EqNP.s                          = IPS_IDLE;
+            MovementNSSP.s = IPS_IDLE;
+            MovementWESP.s = IPS_IDLE;
+            EqNP.s = IPS_IDLE;
             IUResetSwitch(&MovementNSSP);
             IUResetSwitch(&MovementWESP);
             IDSetSwitch(&MovementNSSP, nullptr);
@@ -958,19 +1112,25 @@ bool Rainbow::Abort()
 /////////////////////////////////////////////////////////////////////////////
 bool Rainbow::Sync(double ra, double dec)
 {
-    char cmd[DRIVER_LEN] = {0};
 
-    snprintf(cmd, DRIVER_LEN, ":Ck%07.3f%c%06.3f#", ra * 15.0, dec >= 0 ? '+' : '-', std::fabs(dec));
+    char cmd[DRIVER_LEN] = {0};
+    if (SaveAlignBeforeSyncS[STAR_ALIGNMENT_ENABLED].s == ISS_ON)
+    {
+        snprintf(cmd, DRIVER_LEN, ":CN%07.3f%c%06.3f#", ra * 15.0, dec >= 0 ? '+' : '-', std::fabs(dec));
+    }
+    else
+    {
+        snprintf(cmd, DRIVER_LEN, ":Ck%07.3f%c%06.3f#", ra * 15.0, dec >= 0 ? '+' : '-', std::fabs(dec));
+    }
 
     if (sendCommand(cmd))
     {
         char RAStr[64] = {0}, DecStr[64] = {0};
         fs_sexa(RAStr, ra, 2, 36000);
         fs_sexa(DecStr, dec, 2, 36000);
-        LOGF_INFO("Synced to RA %s DE %s", RAStr, DecStr);
+        LOGF_INFO("Synced to RA %s DE %s%s",RAStr, DecStr, SaveAlignBeforeSyncS[STAR_ALIGNMENT_ENABLED].s == ISS_ON?", and saved as alignment point.":"");
         return true;
     }
-
     return false;
 }
 
@@ -1254,7 +1414,7 @@ void Rainbow::addGuideTimer(Direction direction, uint32_t ms)
 }
 
 /////////////////////////////////////////////////////////////////////////////
-/// Send Time
+/// Get Time from mount
 /////////////////////////////////////////////////////////////////////////////
 bool Rainbow::getLocalTime(char *timeString)
 {
@@ -1265,10 +1425,16 @@ bool Rainbow::getLocalTime(char *timeString)
     }
     else
     {
-        double ctime = 0;
         int h, m, s;
-        getLocalTime24(PortFD, &ctime);
-        getSexComponents(ctime, &h, &m, &s);
+        char response[DRIVER_LEN] = {0};
+        if (!sendCommand(":GL#", response))
+            return false;
+
+        if (sscanf(response + 3, "%d:%d:%d", &h, &m, &s) != 3)
+        {
+            LOG_WARN("Failed to get time from device.");
+            return false;
+        }
         snprintf(timeString, MAXINDINAME, "%02d:%02d:%02d", h, m, s);
     }
 
@@ -1276,7 +1442,7 @@ bool Rainbow::getLocalTime(char *timeString)
 }
 
 /////////////////////////////////////////////////////////////////////////////
-/// Send Time
+/// Get Date from mount
 /////////////////////////////////////////////////////////////////////////////
 bool Rainbow::getLocalDate(char *dateString)
 {
@@ -1287,14 +1453,33 @@ bool Rainbow::getLocalDate(char *dateString)
     }
     else
     {
-        getCalendarDate(PortFD, dateString);
+        int dd, mm, yy;
+        char response[DRIVER_LEN] = {0};
+        char mell_prefix[3] = {0};
+        if (!sendCommand(":GC#", response))
+            return false;
+
+        if (sscanf(response + 3, "%d%*c%d%*c%d", &mm, &dd, &yy) != 3)
+        {
+            LOG_WARN("Failed to get date from device.");
+            return false;
+        }
+        else
+        {
+            if (yy > 50)
+                strncpy(mell_prefix, "19", 3);
+            else
+                strncpy(mell_prefix, "20", 3);
+            /* We need to have it in YYYY-MM-DD ISO format */
+            snprintf(dateString, 32, "%s%02d-%02d-%02d", mell_prefix, yy, mm, dd);
+        }
     }
 
     return true;
 }
 
 /////////////////////////////////////////////////////////////////////////////
-/// Send Time
+/// GET UTC offset from mount
 /////////////////////////////////////////////////////////////////////////////
 bool Rainbow::getUTFOffset(double *offset)
 {
@@ -1304,15 +1489,25 @@ bool Rainbow::getUTFOffset(double *offset)
         return true;
     }
 
-    int lx200_utc_offset = 0;
-    getUTCOffset(PortFD, &lx200_utc_offset);
+    int rst135_utc_offset = 0;
+
+    char response[DRIVER_LEN] = {0};
+    if (!sendCommand(":GG#", response))
+        return false;
+
+    if (sscanf(response + 3, "%d", &rst135_utc_offset) != 1)
+    {
+        LOG_WARN("Failed to get UTC offset from device.");
+        return false;
+    }
+
     // LX200 TimeT Offset is defined at the number of hours added to LOCAL TIME to get TimeT. This is contrary to the normal definition.
-    *offset = lx200_utc_offset * -1;
+    *offset = rst135_utc_offset * -1;
     return true;
 }
 
 /////////////////////////////////////////////////////////////////////////////
-/// Send Time
+/// Get Time and Date from mount
 /////////////////////////////////////////////////////////////////////////////
 bool Rainbow::sendScopeTime()
 {
@@ -1382,12 +1577,13 @@ bool Rainbow::sendScopeTime()
 }
 
 /////////////////////////////////////////////////////////////////////////////
-/// Send Location
+/// Get Location from mount
 /////////////////////////////////////////////////////////////////////////////
 bool Rainbow::sendScopeLocation()
 {
-    int dd = 0, mm = 0;
-    double ssf = 0.0;
+    double longitude {0}, latitude {0};
+    double dd = 0, mm = 0, ssf = 0;
+    char response[DRIVER_LEN] = {0};
 
     if (isSimulation())
     {
@@ -1399,7 +1595,11 @@ bool Rainbow::sendScopeLocation()
         return true;
     }
 
-    if (getSiteLatitude(PortFD, &dd, &mm, &ssf) < 0)
+    // Latitude
+    if (!sendCommand(":Gt#", response))
+        return false;
+
+    if (sscanf(response + 3, "%lf%*[^0-9]%lf%*[^0-9]%lf", &dd, &mm, &ssf) != 3)
     {
         LOG_WARN("Failed to get site latitude from device.");
         return false;
@@ -1407,12 +1607,16 @@ bool Rainbow::sendScopeLocation()
     else
     {
         if (dd > 0)
-            LocationNP.np[0].value = dd + mm / 60.0;
+            latitude = dd + mm / 60.0 + ssf / 3600.0;
         else
-            LocationNP.np[0].value = dd - mm / 60.0;
+            latitude = dd - mm / 60.0 - ssf / 3600.0;
     }
 
-    if (getSiteLongitude(PortFD, &dd, &mm, &ssf) < 0)
+    // Longitude
+    if (!sendCommand(":Gg#", response))
+        return false;
+
+    if (sscanf(response + 3, "%lf%*[^0-9]%lf%*[^0-9]%lf", &dd, &mm, &ssf) != 3)
     {
         LOG_WARN("Failed to get site longitude from device.");
         return false;
@@ -1420,18 +1624,24 @@ bool Rainbow::sendScopeLocation()
     else
     {
         if (dd > 0)
-            LocationNP.np[1].value = 360.0 - (dd + mm / 60.0);
+            longitude = 360.0 - (dd + mm / 60.0 + ssf / 3600.0);
         else
-            LocationNP.np[1].value = (dd - mm / 60.0) * -1.0;
+            longitude = (dd - mm / 60.0 - ssf / 3600.0) * -1.0;
 
     }
 
-    LOGF_DEBUG("Mount Controller Latitude: %.3f Longitude: %.3f", LocationN[LOCATION_LATITUDE].value,
-               LocationN[LOCATION_LONGITUDE].value);
-
-    IDSetNumber(&LocationNP, nullptr);
-
-    //saveConfig(true, "GEOGRAPHIC_COORD");
+    // Only update if different from current values
+    // and then immediately save to config.
+    if (std::abs(LocationN[LOCATION_LONGITUDE].value - longitude) > 0.001 ||
+            std::abs(LocationN[LOCATION_LATITUDE].value - latitude) > 0.001)
+    {
+        LocationN[LOCATION_LATITUDE].value = latitude;
+        LocationN[LOCATION_LONGITUDE].value = longitude;
+        LOGF_DEBUG("Mount Controller Latitude: %.3f Longitude: %.3f", LocationN[LOCATION_LATITUDE].value,
+                   LocationN[LOCATION_LONGITUDE].value);
+        IDSetNumber(&LocationNP, nullptr);
+        saveConfig(true, LocationNP.name);
+    }
 
     return true;
 }
