@@ -24,13 +24,15 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110 - 1301  USA
 #endif
 
 #include <cmath>
+#include <algorithm>
 #include "v4l2driver.h"
 #include "indistandardproperty.h"
 #include "lx/Lx.h"
 
 // Pixel size info for different cameras
-typedef struct
+typedef struct PixelSizeInfo
 {
+    const char * deviceLabel; // Device label used by INDI
     const char * deviceName; // device name reported by V4L
     const char * commonName; // if null, use device name
     float pixelSizeX;
@@ -38,28 +40,66 @@ typedef struct
     bool tested; //if False print please report message
 } PixelSizeInfo;
 
-static const PixelSizeInfo pixelSizeInfo[] =
+static const PixelSizeInfo CameraDatabase[] =
 {
-    { "NexImage 5", nullptr, 2.2f, -1, true },
-    { "UVC Camera (046d:0809)", "Logitech Webcam Pro 9000", 3.3f, -1, true },
-    { "SVBONY SV105: SVBONY SV105", "SVBONY SV105", 3.0f, -1, true },
-    { "SVBONY SV205: SVBONY SV205", "SVBONY SV205", 4.0f, -1, true },
-    { "NexImage 10", nullptr, 1.67f, -1, true },
-    { "NexImage Burst Color", nullptr, 3.75f, -1, false },
-    { "NexImage Burst Mono", nullptr, 3.75f, -1, false },
-    { "Skyris 132C", nullptr, 3.75f, -1, false },
-    { "Skyris 132M", nullptr, 3.75f, -1, false },
-    { "Skyris 236C", nullptr, 2.8f, -1, false },
-    { "Skyris 236M", nullptr, 2.8f, -1, false },
-    { "iOptron iPolar: iOptron iPolar", nullptr, 3.75f, -1, true },
-    { "iOptron iGuider: iOptron iGuider", nullptr, 3.75f, -1, true },
-    { "mmal service 16.1", "Raspberry Pi High Quality Camera", 1.55f, -1, true },
-    { "UVC Camera (046d:0825)", "Logitech HD C270", 2.8f, -1, true },
-    { "0c45:6366 Microdia", "Spinel 2MP Full HD Low Light WDR H264 USB Camera Module IMX290", 2.9f, -1, true },
-    { "Microsoft® LifeCam Cinema(TM):", "Microsoft® LifeCam Cinema(TM)", 3.0f, -1, false },
-    { nullptr, nullptr, 5.6f, -1, false}  // sentinel and default pixel size, needs to be last
+    { "NexImage 5", "NexImage 5", nullptr, 2.2f, -1, true },
+    { "Logitech Webcam Pro 9000", "UVC Camera (046d:0809)", "Logitech Webcam Pro 9000", 3.3f, -1, true },
+    { "SVBONY SV105", "SVBONY SV105: SVBONY SV105", "SVBONY SV105", 3.0f, -1, true },
+    { "SVBONY SV205", "SVBONY SV205: SVBONY SV205", "SVBONY SV205", 4.0f, -1, true },
+    { "NexImage 10", "NexImage 10", nullptr, 1.67f, -1, true },
+    { "NexImage Burst Color", "NexImage Burst Color", nullptr, 3.75f, -1, false },
+    { "NexImage Burst Mono", "NexImage Burst Mono", nullptr, 3.75f, -1, false },
+    { "Skyris 132C", "Skyris 132C", nullptr, 3.75f, -1, false },
+    { "Skyris 132M", "Skyris 132M", nullptr, 3.75f, -1, false },
+    { "Skyris 236C", "Skyris 236C", nullptr, 2.8f, -1, false },
+    { "Skyris 236M", "Skyris 236M", nullptr, 2.8f, -1, false },
+    { "iOptron iPolar", "iOptron iPolar: iOptron iPolar", nullptr, 3.75f, -1, true },
+    { "iOptron iPolar", "iOptron iPolar", nullptr, 3.75f, -1, true },
+    { "iOptron iGuider", "iOptron iGuider: iOptron iGuide", nullptr, 3.75f, -1, true },
+    { "iOptron iGuider", "iOptron iGuider 1", nullptr, 3.75f, -1, true },
+    { "Raspberry Pi High Quality Camera", "mmal service 16.1", "Raspberry Pi High Quality Camera", 1.55f, -1, true },
+    { "Logitech HD C270", "UVC Camera (046d:0825)", "Logitech HD C270", 2.8f, -1, true },
+    { "IMX290 Camera", "USB 2.0 Camera: USB Camera", "USB 2.0 IMX290 Board", 2.9f, -1, true },
+    { "IMX290 H264 Camera", "0c45:6366 Microdia", "Spinel 2MP Full HD Low Light WDR H264 USB Camera Module IMX290", 2.9f, -1, true },
+    { "Microsoft LifeCam Cinema", "Microsoft® LifeCam Cinema(TM):", "Microsoft® LifeCam Cinema(TM)", 3.0f, -1, false },
+    { nullptr, nullptr, nullptr, 5.6f, -1, false}  // sentinel and default pixel size, needs to be last
 };
 
+V4L2_Driver::V4L2_Driver(std::string label, std::string path)
+{
+    setDeviceName(label.c_str());
+    strncpy(defaultVideoPort, path.c_str(), 256);
+    strncpy(configPort, path.c_str(), 256);
+
+    setVersion(1, 0);
+
+    allocateBuffers();
+
+    divider = 128.;
+
+    is_capturing = false;
+    non_capture_frames = 0;
+
+    Options          = nullptr;
+    v4loptions       = 0;
+    AbsExposureN     = nullptr;
+    ManualExposureSP = nullptr;
+    CaptureSizesSP.sp = nullptr;
+    CaptureSizesNP.np = nullptr;
+    FrameRatesSP.sp = nullptr;
+
+    frame_received.tv_sec = 0;
+    frame_received.tv_usec = 0;
+
+    v4l_capture_started = false;
+
+    stackMode = STACK_NONE;
+
+    lx       = new Lx();
+    lxtimer  = -1;
+    stdtimer = -1;
+}
+
 V4L2_Driver::V4L2_Driver()
 {
     setVersion(1, 0);
@@ -69,6 +109,7 @@ V4L2_Driver::V4L2_Driver()
     divider = 128.;
 
     is_capturing = false;
+    non_capture_frames = 0;
 
     Options          = nullptr;
     v4loptions       = 0;
@@ -78,6 +119,11 @@ V4L2_Driver::V4L2_Driver()
     CaptureSizesNP.np = nullptr;
     FrameRatesSP.sp = nullptr;
 
+    frame_received.tv_sec = 0;
+    frame_received.tv_usec = 0;
+
+    v4l_capture_started = false;
+
     stackMode = STACK_NONE;
 
     lx       = new Lx();
@@ -109,17 +155,20 @@ bool V4L2_Driver::initProperties()
     addDebugControl();
 
     /* Port */
-    char configPort[256] = {0};
-    if (IUGetConfigText(getDeviceName(), PortTP.name, PortT[0].name, configPort, 256) == 0)
+    // Only load config port if it was empty. If it was already initialized, then we have an explicitly defined device
+    // with defaultVideoPort and we shouldn't mess this up.
+    if (configPort[0] == 0 && IUGetConfigText(getDeviceName(), PortTP.name, PortT[0].name, configPort, 256) == 0)
         IUFillText(&PortT[0], "PORT", "Port", configPort);
     else
-        IUFillText(&PortT[0], "PORT", "Port", "/dev/video0");
+        IUFillText(&PortT[0], "PORT", "Port", defaultVideoPort);
     IUFillTextVector(&PortTP, PortT, NARRAY(PortT), getDeviceName(), INDI::SP::DEVICE_PORT, "Ports", OPTIONS_TAB, IP_RW, 0,
                      IPS_IDLE);
 
     /* Color space */
-    IUFillSwitch(&ImageColorS[IMAGE_GRAYSCALE], "CCD_COLOR_GRAY", "Gray", ISS_ON);
-    IUFillSwitch(&ImageColorS[1], "CCD_COLOR_RGB", "Color", ISS_OFF);
+    int configColor = IMAGE_GRAYSCALE;
+    IUGetConfigOnSwitchIndex(getDeviceName(), "CCD_COLOR_SPACE", &configColor);
+    IUFillSwitch(&ImageColorS[IMAGE_GRAYSCALE], "CCD_COLOR_GRAY", "Gray", configColor == IMAGE_GRAYSCALE ? ISS_ON : ISS_OFF);
+    IUFillSwitch(&ImageColorS[IMAGE_COLOR], "CCD_COLOR_RGB", "Color", configColor == IMAGE_COLOR ? ISS_ON : ISS_OFF);
     IUFillSwitchVector(&ImageColorSP, ImageColorS, NARRAY(ImageColorS), getDeviceName(), "CCD_COLOR_SPACE",
                        "Image Type", IMAGE_SETTINGS_TAB, IP_RW, ISR_1OFMANY, 0, IPS_IDLE);
 
@@ -208,32 +257,32 @@ void V4L2_Driver::ISGetProperties(const char * dev)
 
     INDI::CCD::ISGetProperties(dev);
 
-    defineText(&PortTP);
+    defineProperty(&PortTP);
     loadConfig(true, INDI::SP::DEVICE_PORT);
 
     if (isConnected())
     {
-        defineText(&camNameTP);
+        defineProperty(&camNameTP);
 
-        defineSwitch(&ImageColorSP);
-        defineSwitch(&InputsSP);
-        defineSwitch(&CaptureFormatsSP);
+        defineProperty(&ImageColorSP);
+        defineProperty(&InputsSP);
+        defineProperty(&CaptureFormatsSP);
 
         if (CaptureSizesSP.sp != nullptr)
-            defineSwitch(&CaptureSizesSP);
+            defineProperty(&CaptureSizesSP);
         else if (CaptureSizesNP.np != nullptr)
-            defineNumber(&CaptureSizesNP);
+            defineProperty(&CaptureSizesNP);
         if (FrameRatesSP.sp != nullptr)
-            defineSwitch(&FrameRatesSP);
+            defineProperty(&FrameRatesSP);
         else if (FrameRateNP.np != nullptr)
-            defineNumber(&FrameRateNP);
+            defineProperty(&FrameRateNP);
 
-        defineSwitch(&StackModeSP);
+        defineProperty(&StackModeSP);
 
 #ifdef WITH_V4L2_EXPERIMENTS
-        defineSwitch(&ImageDepthSP);
-        defineSwitch(&ColorProcessingSP);
-        defineText(&CaptureColorSpaceTP);
+        defineProperty(&ImageDepthSP);
+        defineProperty(&ColorProcessingSP);
+        defineProperty(&CaptureColorSpaceTP);
 #endif
     }
 }
@@ -253,35 +302,44 @@ bool V4L2_Driver::updateProperties()
         FrameNP = getNumber("CCD_FRAME");
         FrameN  = FrameNP->np;
 
-        defineText(&camNameTP);
+        defineProperty(&camNameTP);
         getBasicData();
 
-        defineSwitch(&ImageColorSP);
-        defineSwitch(&InputsSP);
-        defineSwitch(&CaptureFormatsSP);
+        defineProperty(&ImageColorSP);
+        defineProperty(&InputsSP);
+        defineProperty(&CaptureFormatsSP);
 
         if (CaptureSizesSP.sp != nullptr)
-            defineSwitch(&CaptureSizesSP);
+            defineProperty(&CaptureSizesSP);
         else if (CaptureSizesNP.np != nullptr)
-            defineNumber(&CaptureSizesNP);
+            defineProperty(&CaptureSizesNP);
         if (FrameRatesSP.sp != nullptr)
-            defineSwitch(&FrameRatesSP);
+            defineProperty(&FrameRatesSP);
         else if (FrameRateNP.np != nullptr)
-            defineNumber(&FrameRateNP);
+            defineProperty(&FrameRateNP);
 
-        defineSwitch(&StackModeSP);
+        defineProperty(&StackModeSP);
 
 #ifdef WITH_V4L2_EXPERIMENTS
-        defineSwitch(&ImageDepthSP);
-        defineSwitch(&ColorProcessingSP);
-        defineText(&CaptureColorSpaceTP);
+        defineProperty(&ImageDepthSP);
+        defineProperty(&ColorProcessingSP);
+        defineProperty(&CaptureColorSpaceTP);
 #endif
 
         // Check if we have pixel size info
-        const PixelSizeInfo * info = pixelSizeInfo;
+        const PixelSizeInfo * info = CameraDatabase;
+        std::string deviceName = std::string(v4l_base->getDeviceName());
+        // to lower case.
+        std::transform(deviceName.begin(), deviceName.end(), deviceName.begin(), ::tolower);
         while (info->deviceName)
         {
-            if (!strcmp(v4l_base->getDeviceName(), info->deviceName))
+            std::string infoDeviceName = std::string(info->deviceName);
+            std::transform(infoDeviceName.begin(), infoDeviceName.end(), infoDeviceName.begin(), ::tolower);
+            std::string infoDeviceLabel = std::string(info->deviceLabel);
+            std::transform(infoDeviceLabel.begin(), infoDeviceLabel.end(), infoDeviceLabel.begin(), ::tolower);
+
+            // Case insensitive comparision
+            if (infoDeviceName == deviceName || infoDeviceLabel == deviceName)
                 break;
             ++info;
         }
@@ -400,7 +458,7 @@ bool V4L2_Driver::ISNewSwitch(const char * dev, const char * name, ISState * sta
 
             deleteProperty(CaptureFormatsSP.name);
             v4l_base->getcaptureformats(&CaptureFormatsSP);
-            defineSwitch(&CaptureFormatsSP);
+            defineProperty(&CaptureFormatsSP);
             if (CaptureSizesSP.sp != nullptr)
                 deleteProperty(CaptureSizesSP.name);
             else if (CaptureSizesNP.np != nullptr)
@@ -409,9 +467,9 @@ bool V4L2_Driver::ISNewSwitch(const char * dev, const char * name, ISState * sta
             v4l_base->getcapturesizes(&CaptureSizesSP, &CaptureSizesNP);
 
             if (CaptureSizesSP.sp != nullptr)
-                defineSwitch(&CaptureSizesSP);
+                defineProperty(&CaptureSizesSP);
             else if (CaptureSizesNP.np != nullptr)
-                defineNumber(&CaptureSizesNP);
+                defineProperty(&CaptureSizesNP);
             InputsSP.s = IPS_OK;
             IDSetSwitch(&InputsSP, nullptr);
             LOGF_INFO("Capture input: %d. %s", inputindex, InputsSP.sp[inputindex].name);
@@ -458,9 +516,9 @@ bool V4L2_Driver::ISNewSwitch(const char * dev, const char * name, ISState * sta
             v4l_base->getcapturesizes(&CaptureSizesSP, &CaptureSizesNP);
 
             if (CaptureSizesSP.sp != nullptr)
-                defineSwitch(&CaptureSizesSP);
+                defineProperty(&CaptureSizesSP);
             else if (CaptureSizesNP.np != nullptr)
-                defineNumber(&CaptureSizesNP);
+                defineProperty(&CaptureSizesNP);
             CaptureFormatsSP.s = IPS_OK;
 
 #ifdef WITH_V4L2_EXPERIMENTS
@@ -512,9 +570,9 @@ bool V4L2_Driver::ISNewSwitch(const char * dev, const char * name, ISState * sta
                 deleteProperty(FrameRateNP.name);
             v4l_base->getframerates(&FrameRatesSP, &FrameRateNP);
             if (FrameRatesSP.sp != nullptr)
-                defineSwitch(&FrameRatesSP);
+                defineProperty(&FrameRatesSP);
             else if (FrameRateNP.np != nullptr)
-                defineNumber(&FrameRateNP);
+                defineProperty(&FrameRateNP);
 
             PrimaryCCD.setFrame(0, 0, w, h);
             V4LFrame->width  = w;
@@ -864,6 +922,8 @@ bool V4L2_Driver::StartExposure(float duration)
                 "Cannot start new exposure until the current one completes (%.3f seconds left).",
                 getRemainingExposure());
             return !(GetCCDCapability() & CCD_CAN_ABORT);
+
+            return true;
         }
     }
 
@@ -879,9 +939,10 @@ bool V4L2_Driver::StartExposure(float duration)
         /* FIXME: exposure update timer has period hardcoded 1 second */
         if (is_capturing && 1.0f < duration)
         {
-            if (-1 != stdtimer)
-                IERmTimer(stdtimer);
-            stdtimer = IEAddTimer(1000, (IE_TCF *)stdtimerCallback, this);
+            //if (-1 != stdtimer)
+            //    IERmTimer(stdtimer);
+            //stdtimer = IEAddTimer(1000, (IE_TCF *)stdtimerCallback, this);
+            stdtimer = -1;
         }
         else
             stdtimer = -1;
@@ -912,11 +973,17 @@ bool V4L2_Driver::setShutter(double duration)
         exposure_duration.tv_sec  = (long) duration;
         exposure_duration.tv_usec = (long) ((duration - (double) exposure_duration.tv_sec) * 1000000.0f);
 
+        elapsed_exposure.tv_sec = 0;
+        elapsed_exposure.tv_usec = 0;
+
         gettimeofday(&capture_start, nullptr);
+
         frameCount    = 0;
         subframeCount = 0;
 
-        LOGF_INFO("Started %.3f-second manual exposure.", duration);
+        // Do not spam log for short exposures.
+        if (duration >= 3)
+            LOGF_INFO("Started %.3f-second manual exposure.", duration);
         return true;
     }
     else
@@ -944,9 +1011,26 @@ bool V4L2_Driver::setManualExposure(double duration)
         /* We don't have an absolute exposure control but we can stack gray frames until the exposure elapses */
         if (ImageColorS[IMAGE_GRAYSCALE].s == ISS_ON && stackMode != STACK_NONE && stackMode != STACK_RESET_DARK)
         {
-            LOGF_WARN("Absolute exposure duration control is undefined, stacking up to %.3f seconds using %.16s.",
-                      duration, StackModeS[stackMode].name);
-            return true;
+            //use frame interval as frame duration instead of max exposure time.
+            if(FrameRatesSP.sp != nullptr)
+            {
+                LOGF_WARN("Absolute exposure duration control is undefined, stacking up to %.3f seconds using %.16s.",
+                          duration, StackModeS[stackMode].name);
+                int index = IUFindOnSwitchIndex(&FrameRatesSP);
+                int fn, fd;
+                sscanf(FrameRatesSP.sp[index].name, "%d/%d", &fn, &fd);
+                IDLog("Interval = %d %d\n", fn, fd);
+                ticks = (long)(10000.0 * (float)fn / (float)fd + 0.5);
+                frame_duration.tv_sec  = ticks / 10000;
+                frame_duration.tv_usec = (ticks % 10000) * 100;
+                return true;
+            }
+            else
+            {
+                //ToDo: Same thing should be done for FrameRateNP
+                LOG_ERROR("Absolute exposure duration control is undefined and tacking is not supported");
+                return false;
+            }
         }
         /* We don't have an absolute exposure control and stacking is not configured, bail out */
         else
@@ -959,12 +1043,24 @@ bool V4L2_Driver::setManualExposure(double duration)
     /* Then if we have an exposure control, check the requested exposure duration */
     else if (AbsExposureN->max < ticks)
     {
+        if( ImageColorS[IMAGE_GRAYSCALE].s == ISS_ON && stackMode == STACK_NONE )
+        {
+            LOG_WARN("Requested manual exposure is out of device bounds auto set stackMode to ADDITIVE" );
+            stackMode = STACK_ADDITIVE;
+            StackModeSP.sp[ STACK_NONE ].s = ISS_OFF;
+            StackModeSP.sp[ STACK_ADDITIVE ].s = ISS_ON;
+            if(ManualExposureSP) IDSetSwitch(ManualExposureSP, nullptr);
+        }
+
         /* We can't expose as long as requested but we can stack gray frames until the exposure elapses */
         if (ImageColorS[IMAGE_GRAYSCALE].s == ISS_ON && stackMode != STACK_NONE && stackMode != STACK_RESET_DARK)
         {
-            LOGF_WARN("Requested manual exposure is out of device bounds [%.3f,%.3f], stacking up to %.3f seconds using %.16s.",
-                      (double) AbsExposureN->min / 10000.0f, (double) AbsExposureN->max / 10000.0f,
-                      duration, StackModeS[stackMode].name);
+            if( AbsExposureN->value != AbsExposureN->max )
+            {
+                LOGF_WARN("Requested manual exposure is out of device bounds [%.3f,%.3f], stacking up to %.3f seconds using %.16s.",
+                          (double) AbsExposureN->min / 10000.0f, (double) AbsExposureN->max / 10000.0f,
+                          duration, StackModeS[stackMode].name);
+            }
             ticks = AbsExposureN->max;
         }
         /* We can't expose as long as requested and stacking is not configured, bail out */
@@ -978,6 +1074,23 @@ bool V4L2_Driver::setManualExposure(double duration)
     }
     /* Lower-than-minimal exposure duration is left managed below */
 
+
+    frame_duration.tv_sec  = ticks / 10000;
+    frame_duration.tv_usec = (ticks % 10000) * 100;
+
+    if( v4l_capture_started )
+    {
+        if( AbsExposureN->value != ticks )
+        {
+            stop_capturing();
+        }
+        else
+        {
+            return true;
+        }
+    }
+
+
     /* At this point we do have an absolute exposure control and a valid exposure duration, so start exposing. */
 
     /* Manual mode should be set before changing Exposure (Auto), if possible.
@@ -1085,11 +1198,20 @@ bool V4L2_Driver::start_capturing(bool do_stream)
                   getRemainingExposure());
         return false;
     }
-    char errmsg[ERRMSGSIZ];
-    if (v4l_base->start_capturing(errmsg))
+
+    if( !v4l_capture_started )
     {
-        LOGF_WARN("V4L2 base failed starting capture (%s)", errmsg);
-        return false;
+        char errmsg[ERRMSGSIZ];
+        if (v4l_base->start_capturing(errmsg))
+        {
+            LOGF_WARN("V4L2 base failed starting capture (%s)", errmsg);
+            return false;
+        }
+        else
+        {
+            gettimeofday(&frame_received, nullptr);
+            v4l_capture_started = true;
+        }
     }
 
     //if (do_stream)
@@ -1101,7 +1223,7 @@ bool V4L2_Driver::start_capturing(bool do_stream)
 
 bool V4L2_Driver::stop_capturing()
 {
-    if (!is_capturing)
+    if (!is_capturing && !v4l_capture_started)
     {
         LOG_WARN("No exposure or streaming in progress");
         return true;
@@ -1116,14 +1238,14 @@ bool V4L2_Driver::stop_capturing()
     // FIXME what to do with doRecord?
     //if(Streamer->isDirectRecording())
     //v4l_base->doRecord(false);
-
     char errmsg[ERRMSGSIZ];
-
     if (v4l_base->stop_capturing(errmsg))
     {
         LOGF_WARN("V4L2 base failed stopping capture (%s)", errmsg);
     }
+
     is_capturing = false;
+    v4l_capture_started = false;
     return true;
 }
 
@@ -1149,7 +1271,7 @@ void V4L2_Driver::lxtimerCallback(void * userpointer)
     }
     IERmTimer(p->lxtimer);
     if (!p->v4l_base->isstreamactive())
-        p->start_capturing(false); // jump to new/updateFrame
+        p->is_capturing = p->start_capturing(false); // jump to new/updateFrame
     //p->v4l_base->start_capturing(errmsg); // jump to new/updateFrame
 }
 
@@ -1256,23 +1378,30 @@ void V4L2_Driver::stackFrame()
 
 struct timeval V4L2_Driver::getElapsedExposure() const
 {
-    struct timeval now = { .tv_sec = 0, .tv_usec = 0 }, elapsed = { .tv_sec = 0, .tv_usec = 0 };
-    gettimeofday(&now, nullptr);
-    timersub(&now, &capture_start, &elapsed);
-    return elapsed;
+    struct timeval now = { .tv_sec = 0, .tv_usec = 0 }, duration = { .tv_sec = 0, .tv_usec = 0 };
+    gettimeofday( &now, nullptr );
+    timersub(&now, &capture_start, &duration);
+    return duration;
 }
 
 float V4L2_Driver::getRemainingExposure() const
 {
-    struct timeval elapsed = getElapsedExposure(), remaining = { .tv_sec = 0, .tv_usec = 0 };
-    timersub(&exposure_duration, &elapsed, &remaining);
+    struct timeval remaining = { .tv_sec = 0, .tv_usec = 0 };
+    timersub(&exposure_duration, &elapsed_exposure, &remaining);
     return (float) remaining.tv_sec + (float) remaining.tv_usec / 1000000.0f;
 }
 
 void V4L2_Driver::newFrame()
 {
+    struct timeval current_frame_duration = frame_received;
+    gettimeofday(&frame_received, nullptr);
+    timersub(&frame_received, &current_frame_duration, &current_frame_duration);
+
+
     if (Streamer->isBusy())
     {
+        non_capture_frames = 0;
+
         int width             = v4l_base->getWidth();
         int height            = v4l_base->getHeight();
         int bpp               = v4l_base->getBpp();
@@ -1344,21 +1473,53 @@ void V4L2_Driver::newFrame()
         return;
     }
 
-    if (PrimaryCCD.isExposing())
+    if ( PrimaryCCD.isExposing() )
     {
+        non_capture_frames = 0;
+        if( !is_capturing )
+        {
+            LOG_DEBUG("Skip frame, setup not complete yet" );
+            return; //skip this frame
+        }
+
+        struct timeval capture_frame_dif = { .tv_sec = 0, .tv_usec = 0 };
+        timersub(&frame_received, &capture_start, &capture_frame_dif);
+
+        float cfd = (float) capture_frame_dif.tv_sec + (float) capture_frame_dif.tv_usec / 1000000.0f;
+        float fd = (float) frame_duration.tv_sec + (float) frame_duration.tv_usec / 1000000.0f;
+
+        if( cfd < fd * 0.9 )
+        {
+            LOGF_DEBUG("Skip early frame cfd = %ld.%06ld seconds.", capture_frame_dif.tv_sec, capture_frame_dif.tv_usec);
+            return;
+        }
+
+        timeradd(&elapsed_exposure, &frame_duration, &elapsed_exposure);
+
+        LOGF_DEBUG("Frame took %ld.%06ld s, e = %ld.%06ld s, t = %ld.%06ld s., cfd = %ld.%06ld s.",
+                   current_frame_duration.tv_sec, current_frame_duration.tv_usec,
+                   elapsed_exposure.tv_sec, elapsed_exposure.tv_usec,
+                   exposure_duration.tv_sec, exposure_duration.tv_usec,
+                   capture_frame_dif.tv_sec, capture_frame_dif.tv_usec
+                  );
+
+
+        float remaining = getRemainingExposure();
+        PrimaryCCD.setExposureLeft(remaining);
+
         // Stack Mono frames
         if ((stackMode) && !(lx->isEnabled()) && !(ImageColorS[1].s == ISS_ON))
         {
             stackFrame();
         }
 
-        struct timeval const current_exposure = getElapsedExposure();
-
         /* FIXME: stacking does not account for transfer time, so we'll miss the last frames probably */
         if ((stackMode) && !(lx->isEnabled()) && !(ImageColorS[1].s == ISS_ON) &&
-                (timercmp(&current_exposure, &exposure_duration, < )))
+                (timercmp(&elapsed_exposure, &exposure_duration, < )))
             return; // go on stacking
 
+        struct timeval const current_exposure = getElapsedExposure();
+
         //IDLog("Copying frame.\n");
         if (ImageColorS[IMAGE_GRAYSCALE].s == ISS_ON)
         {
@@ -1368,7 +1529,9 @@ void V4L2_Driver::newFrame()
                 src  = v4l_base->getY();
                 dest = (unsigned char *)PrimaryCCD.getFrameBuffer();
 
+                std::unique_lock<std::mutex> guard(ccdBufferLock);
                 memcpy(dest, src, frameBytes);
+                guard.unlock();
                 //for (i=0; i< frameBytes; i++)
                 //*(dest++) = *(src++);
 
@@ -1511,29 +1674,40 @@ void V4L2_Driver::newFrame()
             if (Streamer->isBusy() == false)
                 stop_capturing();
 
-            LOGF_INFO("Capture of LX frame took %ld.%06ld seconds.", current_exposure.tv_sec, current_exposure.tv_usec);
+            if (PrimaryCCD.getExposureDuration() >= 3)
+                LOGF_INFO("Capture of LX frame took %ld.%06ld seconds.", current_exposure.tv_sec, current_exposure.tv_usec);
             ExposureComplete(&PrimaryCCD);
         }
         else
         {
             //if (!is_streaming && !is_recording) stop_capturing();
             if (Streamer->isBusy() == false)
-                stop_capturing();
+            {
+                //just mark stop
+                is_capturing = false;
+            }
             else
-                IDLog("%s: streamer is busy, continue capturing\n", __FUNCTION__);
+                LOGF_DEBUG("%s: streamer is busy, continue capturing\n", __FUNCTION__);
 
-            LOGF_INFO("Capture of one frame (%d stacked frames) took %ld.%06ld seconds.",
-                      subframeCount, current_exposure.tv_sec, current_exposure.tv_usec);
+
+            if (PrimaryCCD.getExposureDuration() >= 3)
+                LOGF_INFO("Capture of one frame (%d stacked frames) took %ld.%06ld seconds.",  subframeCount, current_exposure.tv_sec,
+                          current_exposure.tv_usec);
             ExposureComplete(&PrimaryCCD);
         }
     }
     else
     {
-        /* If we arrive here, PrimaryCCD is not exposing anymore, we can't forward the frame and we can't be aborted neither, thus abort the exposure right now.
-         * That issue can be reproduced when clicking the "Set" button on the "Main Control" tab while an exposure is running.
-         * Note that the patch in StartExposure returning busy instead of error prevents the flow from coming here, so now it's only a safeguard. */
-        IDLog("%s: frame received while not exposing, force-aborting capture\n", __FUNCTION__);
-        AbortExposure();
+        non_capture_frames++;
+
+        if( non_capture_frames > 10 )
+        {
+            /* If we arrive here, PrimaryCCD is not exposing anymore, we can't forward the frame and we can't be aborted neither, thus abort the exposure right now.
+            * That issue can be reproduced when clicking the "Set" button on the "Main Control" tab while an exposure is running.
+            * Note that the patch in StartExposure returning busy instead of error prevents the flow from coming here, so now it's only a safeguard. */
+            IDLog("%s: frame received while not exposing, force-aborting capture\n", __FUNCTION__);
+            AbortExposure();
+        }
     }
 }
 
@@ -1562,15 +1736,18 @@ bool V4L2_Driver::Connect()
     {
         if (v4l_base->connectCam(PortT[0].text, errmsg) < 0)
         {
-            LOGF_ERROR("Error: unable to open device. %s", errmsg);
+            LOGF_ERROR("Error: unable to open device %s: %s", PortT[0].text, errmsg);
             return false;
         }
 
         /* Sucess! */
-        LOG_INFO("V4L2 CCD Device is online. Initializing properties.");
+        LOGF_INFO("%s is online.", getDeviceName());
+
+        // If port not stored in config already then save it.
+        if (strcmp(PortT[0].text, configPort))
+            saveConfig(true, PortTP.name);
 
         v4l_base->registerCallback(newFrame, this);
-
         lx->setCamerafd(v4l_base->fd);
 
         if (!(strcmp((const char *)v4l_base->cap.driver, "pwc")))
@@ -1666,7 +1843,7 @@ void V4L2_Driver::updateV4L2Controls()
     ImageAdjustNP.nnp = 0;
 
     //if (v4l_base->queryINTControls(&ImageAdjustNP) > 0)
-    //defineNumber(&ImageAdjustNP);
+    //defineProperty(&ImageAdjustNP);
     v4l_base->enumerate_ext_ctrl();
     useExtCtrl = false;
 
@@ -1679,7 +1856,7 @@ void V4L2_Driver::updateV4L2Controls()
     if (v4ladjustments > 0)
     {
         LOGF_DEBUG("Found %d V4L2 adjustments", v4ladjustments);
-        defineNumber(&ImageAdjustNP);
+        defineProperty(&ImageAdjustNP);
 
         for (int i = 0; i < ImageAdjustNP.nnp; i++)
         {
@@ -1696,7 +1873,7 @@ void V4L2_Driver::updateV4L2Controls()
     LOGF_DEBUG("Found %d V4L2 options", v4loptions);
     for (i = 0; i < v4loptions; i++)
     {
-        defineSwitch(&Options[i]);
+        defineProperty(&Options[i]);
 
         if (strcmp(Options[i].label, "Exposure, Auto") == 0 || strcmp(Options[i].label, "Auto Exposure") == 0)
         {
@@ -1707,7 +1884,9 @@ void V4L2_Driver::updateV4L2Controls()
     }
 
     if(!AbsExposureN)
+    {
         DEBUGF(INDI::Logger::DBG_WARNING, "Absolute exposure duration control is not possible on the device!", "");
+    }
 
     if(!ManualExposureSP)
         DEBUGF(INDI::Logger::DBG_WARNING, "Manual/auto exposure control is not possible on the device!", "");
@@ -1768,6 +1947,8 @@ bool V4L2_Driver::saveConfigItems(FILE * fp)
     INDI::CCD::saveConfigItems(fp);
 
     IUSaveConfigText(fp, &PortTP);
+    IUSaveConfigSwitch(fp, &StackModeSP);
+    IUSaveConfigSwitch(fp, &ImageColorSP);
 
     if (ImageAdjustNP.nnp > 0)
         IUSaveConfigNumber(fp, &ImageAdjustNP);
