@@ -17,6 +17,7 @@
 *******************************************************************************/
 
 #include "basedevice.h"
+#include "basedevice_p.h"
 
 #include "base64.h"
 #include "config.h"
@@ -24,11 +25,21 @@
 #include "indistandardproperty.h"
 #include "locale_compat.h"
 
+#include "indipropertytext.h"
+#include "indipropertynumber.h"
+#include "indipropertyswitch.h"
+#include "indipropertylight.h"
+#include "indipropertyblob.h"
+
 #include <cerrno>
+#include <cassert>
 #include <cstdlib>
 #include <cstring>
 #include <zlib.h>
 #include <sys/stat.h>
+#include <thread>
+#include <chrono>
+#include <algorithm>
 
 #if defined(_MSC_VER)
 #define snprintf _snprintf
@@ -40,124 +51,162 @@
 namespace INDI
 {
 
-BaseDevice::BaseDevice()
+BaseDevicePrivate::BaseDevicePrivate()
 {
-    mediator = nullptr;
-    lp       = newLilXML();
-    deviceID = new char[MAXINDIDEVICE];
-    memset(deviceID, 0, MAXINDIDEVICE);
-
-    char indidev[MAXINDIDEVICE];
-    strncpy(indidev, "INDIDEV=", MAXINDIDEVICE);
+    static char indidev[] = "INDIDEV=";
+    lp = newLilXML();
 
     if (getenv("INDIDEV") != nullptr)
     {
-        strncpy(deviceID, getenv("INDIDEV"), MAXINDIDEVICE);
+        deviceName = getenv("INDIDEV");
         putenv(indidev);
     }
 }
 
-BaseDevice::~BaseDevice()
+BaseDevicePrivate::~BaseDevicePrivate()
 {
     delLilXML(lp);
-    while (!pAll.empty())
-    {
-        delete pAll.back();
-        pAll.pop_back();
-    }
-    messageLog.clear();
-
-    delete[] deviceID;
+    pAll.clear();
 }
 
-INumberVectorProperty *BaseDevice::getNumber(const char *name)
+BaseDevice::BaseDevice()
+    : d_ptr(new BaseDevicePrivate)
+{ }
+
+BaseDevice::~BaseDevice()
+{ }
+
+BaseDevice::BaseDevice(BaseDevicePrivate &dd)
+    : d_ptr(&dd)
+{ }
+
+INDI::PropertyView<INumber> *BaseDevice::getNumber(const char *name) const
 {
-    return static_cast<INumberVectorProperty *>(getRawProperty(name, INDI_NUMBER));
+    return static_cast<PropertyView<INumber> *>(getRawProperty(name, INDI_NUMBER));
 }
 
-ITextVectorProperty *BaseDevice::getText(const char *name)
+INDI::PropertyView<IText> *BaseDevice::getText(const char *name) const
 {
-    return static_cast<ITextVectorProperty *>(getRawProperty(name, INDI_TEXT));
+    return static_cast<PropertyView<IText> *>(getRawProperty(name, INDI_TEXT));
 }
 
-ISwitchVectorProperty *BaseDevice::getSwitch(const char *name)
+INDI::PropertyView<ISwitch> *BaseDevice::getSwitch(const char *name) const
 {
-    return static_cast<ISwitchVectorProperty *>(getRawProperty(name, INDI_SWITCH));
+    return static_cast<PropertyView<ISwitch> *>(getRawProperty(name, INDI_SWITCH));
 }
 
-ILightVectorProperty *BaseDevice::getLight(const char *name)
+INDI::PropertyView<ILight> *BaseDevice::getLight(const char *name) const
 {
-    return static_cast<ILightVectorProperty *>(getRawProperty(name, INDI_LIGHT));
+    return static_cast<PropertyView<ILight> *>(getRawProperty(name, INDI_LIGHT));
 }
 
-IBLOBVectorProperty *BaseDevice::getBLOB(const char *name)
+INDI::PropertyView<IBLOB> *BaseDevice::getBLOB(const char *name) const
 {
-    return static_cast<IBLOBVectorProperty *>(getRawProperty(name, INDI_BLOB));
+    return static_cast<PropertyView<IBLOB> *>(getRawProperty(name, INDI_BLOB));
 }
 
-IPState BaseDevice::getPropertyState(const char *name)
+IPState BaseDevice::getPropertyState(const char *name) const
 {
-    for (const auto &oneProp : pAll)
-        if (!strcmp(name, oneProp->getName()))
-            return oneProp->getState();
+    for (const auto &oneProp : getProperties())
+        if (oneProp.isNameMatch(name))
+            return oneProp.getState();
 
     return IPS_IDLE;
 }
 
-IPerm BaseDevice::getPropertyPermission(const char *name)
+IPerm BaseDevice::getPropertyPermission(const char *name) const
 {
-    for (const auto &oneProp : pAll)
-        if (!strcmp(name, oneProp->getName()))
-            return oneProp->getPermission();
+    for (const auto &oneProp : getProperties())
+        if (oneProp.isNameMatch(name))
+            return oneProp.getPermission();
 
     return IP_RO;
 }
 
-void *BaseDevice::getRawProperty(const char *name, INDI_PROPERTY_TYPE type)
+void *BaseDevice::getRawProperty(const char *name, INDI_PROPERTY_TYPE type) const
 {
     INDI::Property *prop = getProperty(name, type);
     return prop != nullptr ? prop->getProperty() : nullptr;
 }
 
-INDI::Property *BaseDevice::getProperty(const char *name, INDI_PROPERTY_TYPE type)
+INDI::Property BaseDevice::getProperty(const char *name, INDI_PROPERTY_TYPE type) const
 {
-    for (const auto &oneProp : pAll)
+    D_PTR(const BaseDevice);
+    std::lock_guard<std::mutex> lock(d->m_Lock);
+
+    for (const auto &oneProp : getProperties())
     {
-        if (type != oneProp->getType() && type != INDI_UNKNOWN)
+        if (type != oneProp.getType() && type != INDI_UNKNOWN)
             continue;
 
-        if (!oneProp->getRegistered())
+        if (!oneProp.getRegistered())
             continue;
 
-        if (!strcmp(name, oneProp->getName()))
+        if (oneProp.isNameMatch(name))
             return oneProp;
     }
 
-    return nullptr;
+    return INDI::Property();
+}
+
+BaseDevice::Properties BaseDevice::getProperties()
+{
+    D_PTR(BaseDevice);
+    return d->pAll;
+}
+
+const BaseDevice::Properties BaseDevice::getProperties() const
+{
+    D_PTR(const BaseDevice);
+    return d->pAll;
 }
 
 int BaseDevice::removeProperty(const char *name, char *errmsg)
 {
-    for (auto orderi = pAll.begin(); orderi != pAll.end(); ++orderi)
+    D_PTR(BaseDevice);
+    int result = INDI_PROPERTY_INVALID;
+
+    std::lock_guard<std::mutex> lock(d->m_Lock);
+
+    d->pAll.erase_if([&name, &result](INDI::Property & prop) -> bool
     {
-        const auto &oneProp = *orderi;
-        if (!strcmp(name, oneProp->getName()))
+#if 0
+        if (prop.isNameMatch(name))
         {
-            //            if (mediator)
-            //                mediator->removeProperty(oneProp);
-
-            delete oneProp;
-            orderi = pAll.erase(orderi);
-            return 0;
+            // JM 2021-04-28: delete later. We perform the actual delete after 100ms to give clients a chance to remove the object.
+            // This is necessary when rapid define-delete-define sequences are made.
+            // This HACK is not ideal. We need to start using std::shared_ptr for this purpose soon, but this will be a major change to the
+            // interface. Perhaps for INDI 2.0
+            std::thread([prop]
+            {
+                std::this_thread::sleep_for(std::chrono::milliseconds(100));
+            }).detach();
+            result = 0;
+            return true;
         }
-    }
+        else
+        {
+            return false;
+        }
+#endif
+        if (prop.isNameMatch(name))
+        {
+            result = 0;
+            return true;
+        }
+        else
+            return false;
+    });
 
-    snprintf(errmsg, MAXRBUF, "Error: Property %s not found in device %s.", name, deviceID);
-    return INDI_PROPERTY_INVALID;
+    if (result != 0)
+        snprintf(errmsg, MAXRBUF, "Error: Property %s not found in device %s.", name, getDeviceName());
+
+    return result;
 }
 
 bool BaseDevice::buildSkeleton(const char *filename)
 {
+    D_PTR(BaseDevice);
     char errmsg[MAXRBUF];
     FILE *fp     = nullptr;
     XMLEle *root = nullptr, *fproot = nullptr;
@@ -212,7 +261,7 @@ bool BaseDevice::buildSkeleton(const char *filename)
         return false;
     }
 
-    fproot = readXMLFile(fp, lp, errmsg);
+    fproot = readXMLFile(fp, d->lp, errmsg);
     fclose(fp);
 
     if (fproot == nullptr)
@@ -224,22 +273,22 @@ bool BaseDevice::buildSkeleton(const char *filename)
     //prXMLEle(stderr, fproot, 0);
 
     for (root = nextXMLEle(fproot, 1); root != nullptr; root = nextXMLEle(fproot, 0))
-        buildProp(root, errmsg);
+        buildProp(root, errmsg, true);
 
     delXMLEle(fproot);
     return true;
     /**************************************************************************/
 }
 
-int BaseDevice::buildProp(XMLEle *root, char *errmsg)
+int BaseDevice::buildProp(XMLEle *root, char *errmsg, bool isDynamic)
 {
+    D_PTR(BaseDevice);
     IPerm perm    = IP_RO;
     IPState state = IPS_IDLE;
     XMLEle *ep    = nullptr;
     char *rtag, *rname, *rdev;
 
-    INDI::Property *indiProp = nullptr;
-    int n = 0;
+    INDI::Property indiProp;
 
     rtag = tagXMLEle(root);
 
@@ -247,11 +296,10 @@ int BaseDevice::buildProp(XMLEle *root, char *errmsg)
     if (crackDN(root, &rdev, &rname, errmsg) < 0)
         return -1;
 
-    if (!deviceID[0])
-        strncpy(deviceID, rdev, MAXINDINAME);
+    if (d->deviceName.empty())
+        d->deviceName = rdev;
 
-    //if (getProperty(rname, type) != nullptr)
-    if (getProperty(rname) != nullptr)
+    if (getProperty(rname).isValid())
         return INDI_PROPERTY_DUPLICATED;
 
     if (strcmp(rtag, "defLightVector") && crackIPerm(findXMLAttValu(root, "perm"), &perm) < 0)
@@ -270,9 +318,7 @@ int BaseDevice::buildProp(XMLEle *root, char *errmsg)
     {
         AutoCNumeric locale;
 
-        INumberVectorProperty *nvp = new INumberVectorProperty;
-
-        INumber *np = nullptr;
+        INDI::PropertyNumber nvp {0};
 
         /* pull out each name/value pair */
         for (ep = nextXMLEle(root, 1); ep != nullptr; ep = nextXMLEle(root, 0))
@@ -280,52 +326,42 @@ int BaseDevice::buildProp(XMLEle *root, char *errmsg)
             if (strcmp(tagXMLEle(ep), "defNumber"))
                 continue;
 
-            np = static_cast<INumber *>(realloc(np, (n + 1) * sizeof(INumber)));
-            INumber *it = &np[n];
-            memset(it, 0, sizeof(*it));
-            it->nvp = nvp;
-
-            strncpy(it->name, findXMLAttValu(ep, "name"), MAXINDINAME);
-            if (*it->name == '\0')
+            INDI::WidgetView<INumber> np;
+            np.setName(findXMLAttValu(ep, "name"));
+            if (np.getName()[0] == '\0')
                 continue;
 
-            if (f_scansexa(pcdataXMLEle(ep), &(it->value)) < 0)
+            double value;
+            if (f_scansexa(pcdataXMLEle(ep), &value) < 0)
             {
                 IDLog("%s: Bad format %s\n", rname, pcdataXMLEle(ep));
                 continue;
             }
 
-            strncpy(it->label,  findXMLAttValu(ep, "label" ), MAXINDILABEL );
-            strncpy(it->format, findXMLAttValu(ep, "format"), MAXINDIFORMAT);
+            np.setValue(value);
+            np.setLabel(findXMLAttValu(ep, "label"));
+            np.setFormat(findXMLAttValu(ep, "format"));
 
-            it->min  = atof(findXMLAttValu(ep, "min"));
-            it->max  = atof(findXMLAttValu(ep, "max"));
-            it->step = atof(findXMLAttValu(ep, "step"));
-            ++n;
+            np.setMin(atof(findXMLAttValu(ep, "min")));
+            np.setMax(atof(findXMLAttValu(ep, "max")));
+            np.setStep(atof(findXMLAttValu(ep, "step")));
+
+            np.setParent(nvp->getNumber());
+
+            nvp.push(std::move(np));
         }
 
-        if (n > 0)
-        {
-            nvp->nnp = n;
-            nvp->np  = np;
-
-            indiProp = new INDI::Property(nvp);
-        }
-        else
-        {
-            IDLog("%s: newNumberVector with no valid members\n", rname);
-            delete (nvp);
-            free (np);
-        }
+        indiProp = nvp;
     }
     else if (!strcmp(rtag, "defSwitchVector"))
     {
-        ISwitchVectorProperty *svp = new ISwitchVectorProperty;
 
-        ISwitch *sp = nullptr;
+        ISRule rule = ISR_1OFMANY;
+        if (crackISRule(findXMLAttValu(root, "rule"), &rule) < 0)
+            rule = ISR_1OFMANY;
 
-        if (crackISRule(findXMLAttValu(root, "rule"), (&svp->r)) < 0)
-            svp->r = ISR_1OFMANY;
+        INDI::PropertySwitch svp {0};
+        svp.setRule(rule);
 
         /* pull out each name/value pair */
         for (ep = nextXMLEle(root, 1); ep != nullptr; ep = nextXMLEle(root, 0))
@@ -333,39 +369,28 @@ int BaseDevice::buildProp(XMLEle *root, char *errmsg)
             if (strcmp(tagXMLEle(ep), "defSwitch"))
                 continue;
 
-            sp = static_cast<ISwitch *>(realloc(sp, (n + 1) * sizeof(ISwitch)));
-            ISwitch *it = &sp[n];
-            memset(it, 0, sizeof(*it));
-            it->svp = svp;
-
-            strncpy(it->name, findXMLAttValu(ep, "name"), MAXINDINAME);
-            if (*it->name == '\0')
+            INDI::WidgetView<ISwitch> sp;
+            sp.setName(findXMLAttValu(ep, "name"));
+            if (sp.getName()[0] == '\0')
                 continue;
 
-            crackISState(pcdataXMLEle(ep), &(it->s));
+            ISState state;
+            crackISState(pcdataXMLEle(ep), &state);
 
-            strncpy(it->label, findXMLAttValu(ep, "label"), MAXINDILABEL);
-            ++n;
+            sp.setState(state);
+            sp.setLabel(findXMLAttValu(ep, "label"));
+
+            sp.setParent(svp->getSwitch());
+
+            svp.push(std::move(sp));
         }
 
-        if (n > 0)
-        {
-            svp->nsp = n;
-            svp->sp  = sp;
-            indiProp = new INDI::Property(svp);
-        }
-        else
-        {
-            IDLog("%s: newSwitchVector with no valid members\n", rname);
-            delete (svp);
-            free (sp);
-        }
+        indiProp = svp;
     }
 
     else if (!strcmp(rtag, "defTextVector"))
     {
-        ITextVectorProperty *tvp = new ITextVectorProperty;
-        IText *tp                = nullptr;
+        INDI::PropertyText tvp {0};
 
         // pull out each name/value pair
         for (ep = nextXMLEle(root, 1); ep != nullptr; ep = nextXMLEle(root, 0))
@@ -373,39 +398,24 @@ int BaseDevice::buildProp(XMLEle *root, char *errmsg)
             if (strcmp(tagXMLEle(ep), "defText"))
                 continue;
 
-            tp = static_cast<IText *>(realloc(tp, (n + 1) * sizeof(IText)));
-            IText *it = &tp[n];
-            memset(it, 0, sizeof(*it));
-            it->tvp = tvp;
-
-            strncpy(it->name, findXMLAttValu(ep, "name"), MAXINDINAME);
-            if (*it->name == '\0')
+            INDI::WidgetView<IText> tp;
+            tp.setName(findXMLAttValu(ep, "name"));
+            if (tp.getName()[0] == '\0')
                 continue;
 
-            it->text = strndup(pcdataXMLEle(ep), pcdatalenXMLEle(ep));
+            tp.setText(pcdataXMLEle(ep), pcdatalenXMLEle(ep));
+            tp.setLabel(findXMLAttValu(ep, "label"));
 
-            strncpy(it->label, findXMLAttValu(ep, "label"), MAXINDILABEL);
-            ++n;
+            tp.setParent(tvp->getText());
+
+            tvp.push(std::move(tp));
         }
 
-        if (n > 0)
-        {
-            tvp->ntp = n;
-            tvp->tp  = tp;
-
-            indiProp = new INDI::Property(tvp);
-        }
-        else
-        {
-            IDLog("%s: newTextVector with no valid members\n", rname);
-            delete (tvp);
-            free (tp);
-        }
+        indiProp = tvp;
     }
     else if (!strcmp(rtag, "defLightVector"))
     {
-        ILightVectorProperty *lvp = new ILightVectorProperty;
-        ILight *lp                = nullptr;
+        INDI::PropertyLight lvp {0};
 
         /* pull out each name/value pair */
         for (ep = nextXMLEle(root, 1); ep != nullptr; ep = nextXMLEle(root, 0))
@@ -413,112 +423,91 @@ int BaseDevice::buildProp(XMLEle *root, char *errmsg)
             if (strcmp(tagXMLEle(ep), "defLight"))
                 continue;
 
-            lp = static_cast<ILight *>(realloc(lp, (n + 1) * sizeof(ILight)));
-            ILight *it = &lp[n];
-            memset(it, 0, sizeof(*it));
-            it->lvp = lvp;
-
-            strncpy(it->name, findXMLAttValu(ep, "name"), MAXINDINAME);
-            if (*it->name == '\0')
+            INDI::WidgetView<ILight> lp;
+            lp.setName(findXMLAttValu(ep, "name"));
+            if (lp.getName()[0] == '\0')
                 continue;
 
-            crackIPState(pcdataXMLEle(ep), &(it->s));
+            IPState state;
+            crackIPState(pcdataXMLEle(ep), &state);
+            lp.setState(state);
+            lp.setLabel(findXMLAttValu(ep, "label"));
 
-            strncpy(it->label, findXMLAttValu(ep, "label"), MAXINDILABEL);
-            ++n;
+            lp.setParent(lvp.getLight());
+
+            lvp.push(std::move(lp));
         }
 
-        if (n > 0)
-        {
-            lvp->nlp = n;
-            lvp->lp  = lp;
-
-            indiProp  = new INDI::Property(lvp);
-        }
-        else
-        {
-            IDLog("%s: newLightVector with no valid members\n", rname);
-            delete (lvp);
-            free (lp);
-        }
+        indiProp = lvp;
     }
     else if (!strcmp(rtag, "defBLOBVector"))
     {
-        IBLOBVectorProperty *bvp = new IBLOBVectorProperty;
-        IBLOB *bp                = nullptr;
+        INDI::PropertyBlob bvp {0};
 
         /* pull out each name/value pair */
-        for (n = 0, ep = nextXMLEle(root, 1); ep != nullptr; ep = nextXMLEle(root, 0))
+        for (ep = nextXMLEle(root, 1); ep != nullptr; ep = nextXMLEle(root, 0))
         {
             if (strcmp(tagXMLEle(ep), "defBLOB"))
                 continue;
 
-            bp = static_cast<IBLOB *>(realloc(bp, (n + 1) * sizeof(IBLOB)));
-            IBLOB *it = &bp[n];
-            memset(it, 0, sizeof(*it));
-            it->bvp = bvp;
-
-            strncpy(it->name, findXMLAttValu(ep, "name"), MAXINDINAME);
-            if (*it->name == '\0')
+            INDI::WidgetView<IBLOB> bp;
+            bp.setName(findXMLAttValu(ep, "name"));
+            if (bp.getName()[0] == '\0')
                 continue;
 
-            strncpy(it->label,  findXMLAttValu(ep, "label" ), MAXINDILABEL );
-            strncpy(it->format, findXMLAttValu(ep, "format"), MAXINDIBLOBFMT);
-            ++n;
+            bp.setLabel(findXMLAttValu(ep, "label"));
+            bp.setFormat(findXMLAttValu(ep, "format"));
+
+            bp.setParent(bvp.getBLOB());
+
+            bvp.push(std::move(bp));
         }
 
-        if (n > 0)
-        {
-            bvp->nbp = n;
-            bvp->bp  = bp;
-
-            indiProp  = new INDI::Property(bvp);
-        }
-        else
-        {
-            IDLog("%s: newBLOBVector with no valid members\n", rname);
-            delete (bvp);
-            free (bp);
-        }
+        indiProp = bvp;
     }
 
-    if (indiProp)
+    if (!indiProp.isValid())
     {
-        indiProp->setBaseDevice(this);
-        indiProp->setDynamic(true);
-        indiProp->setDeviceName(deviceID);
-        indiProp->setName(rname);
-        indiProp->setLabel(findXMLAttValu(root, "label"));
-        indiProp->setGroupName(findXMLAttValu(root, "group"));
-        indiProp->setPermission(perm);
-        indiProp->setState(state);
-        indiProp->setTimeout(atoi(findXMLAttValu(root, "timeout")));
-
-        pAll.push_back(indiProp);
-
-        //IDLog("Adding number property %s to list.\n", indiProp->getName());
-        if (mediator)
-            mediator->newProperty(indiProp);
+        IDLog("%s: invalid name '%s'\n", rname, rtag);
+        return 0;
     }
 
+    if (indiProp.isEmpty())
+    {
+        IDLog("%s: %s with no valid members\n", rname, rtag);
+        return 0;
+    }
+
+    indiProp.setBaseDevice(this);
+    indiProp.setDynamic(isDynamic);
+    indiProp.setDeviceName(getDeviceName());
+    indiProp.setName(rname);
+    indiProp.setLabel(findXMLAttValu(root, "label"));
+    indiProp.setGroupName(findXMLAttValu(root, "group"));
+    indiProp.setPermission(perm);
+    indiProp.setState(state);
+    indiProp.setTimeout(atoi(findXMLAttValu(root, "timeout")));
+
+    std::unique_lock<std::mutex> lock(d->m_Lock);
+    d->pAll.push_back(indiProp);
+    lock.unlock();
+
+    //IDLog("Adding number property %s to list.\n", indiProp->getName());
+    if (d->mediator)
+        d->mediator->newProperty(indiProp);
+
     return (0);
 }
 
-bool BaseDevice::isConnected()
+bool BaseDevice::isConnected() const
 {
-    ISwitchVectorProperty *svp = getSwitch(INDI::SP::CONNECTION);
+    auto svp = getSwitch(INDI::SP::CONNECTION);
     if (!svp)
         return false;
 
-    ISwitch *sp = IUFindSwitch(svp, "CONNECT");
+    auto sp = svp->findWidgetByName("CONNECT");
 
-    if (!sp)
-        return false;
-
-    if (sp->s == ISS_ON && svp->s == IPS_OK)
-        return true;
-    else
-        return false;
+    return sp && sp->getState() == ISS_ON && svp->getState() == IPS_OK;
 }
 
 /*
@@ -526,6 +515,7 @@ bool BaseDevice::isConnected()
  */
 int BaseDevice::setValue(XMLEle *root, char *errmsg)
 {
+    D_PTR(BaseDevice);
     XMLEle *ep = nullptr;
     char *name = nullptr;
     double timeout = 0;
@@ -569,135 +559,135 @@ int BaseDevice::setValue(XMLEle *root, char *errmsg)
 
     if (!strcmp(rtag, "setNumberVector"))
     {
-        INumberVectorProperty *nvp = getNumber(name);
-        if (nvp == nullptr)
+        auto nvp = getNumber(name);
+        if (!nvp)
         {
-            snprintf(errmsg, MAXRBUF, "INDI: Could not find property %s in %s", name, deviceID);
+            snprintf(errmsg, MAXRBUF, "INDI: Could not find property %s in %s", name, getDeviceName());
             return -1;
         }
 
         if (stateSet)
-            nvp->s = state;
+            nvp->setState(state);
 
         if (timeoutSet)
-            nvp->timeout = timeout;
+            nvp->setTimeout(timeout);
 
         AutoCNumeric locale;
 
         for (ep = nextXMLEle(root, 1); ep != nullptr; ep = nextXMLEle(root, 0))
         {
-            INumber *np = IUFindNumber(nvp, findXMLAttValu(ep, "name"));
+            auto np = nvp->findWidgetByName(findXMLAttValu(ep, "name"));
             if (!np)
                 continue;
 
-            np->value = atof(pcdataXMLEle(ep));
+            np->setValue(atof(pcdataXMLEle(ep)));
 
             // Permit changing of min/max
             if (findXMLAtt(ep, "min"))
-                np->min = atof(findXMLAttValu(ep, "min"));
+                np->setMin(atof(findXMLAttValu(ep, "min")));
             if (findXMLAtt(ep, "max"))
-                np->max = atof(findXMLAttValu(ep, "max"));
+                np->setMax(atof(findXMLAttValu(ep, "max")));
         }
 
         locale.Restore();
 
-        if (mediator)
-            mediator->newNumber(nvp);
+        if (d->mediator)
+            d->mediator->newNumber(nvp);
 
         return 0;
     }
     else if (!strcmp(rtag, "setTextVector"))
     {
-        ITextVectorProperty *tvp = getText(name);
-        if (tvp == nullptr)
+        auto tvp = getText(name);
+        if (!tvp)
             return -1;
 
         if (stateSet)
-            tvp->s = state;
+            tvp->setState(state);
 
         if (timeoutSet)
-            tvp->timeout = timeout;
+            tvp->setTimeout(timeout);
 
         for (ep = nextXMLEle(root, 1); ep != nullptr; ep = nextXMLEle(root, 0))
         {
-            IText *tp = IUFindText(tvp, findXMLAttValu(ep, "name"));
+            auto tp = tvp->findWidgetByName(findXMLAttValu(ep, "name"));
             if (!tp)
                 continue;
 
-            IUSaveText(tp, pcdataXMLEle(ep));
+            tp->setText(pcdataXMLEle(ep));
         }
 
-        if (mediator)
-            mediator->newText(tvp);
+        if (d->mediator)
+            d->mediator->newText(tvp);
 
         return 0;
     }
     else if (!strcmp(rtag, "setSwitchVector"))
     {
         ISState swState;
-        ISwitchVectorProperty *svp = getSwitch(name);
-        if (svp == nullptr)
+        auto svp = getSwitch(name);
+        if (!svp)
             return -1;
 
         if (stateSet)
-            svp->s = state;
+            svp->setState(state);
 
         if (timeoutSet)
-            svp->timeout = timeout;
+            svp->setTimeout(timeout);
 
         for (ep = nextXMLEle(root, 1); ep != nullptr; ep = nextXMLEle(root, 0))
         {
-            ISwitch *sp = IUFindSwitch(svp, findXMLAttValu(ep, "name"));
+            auto sp = svp->findWidgetByName(findXMLAttValu(ep, "name"));
             if (!sp)
                 continue;
 
             if (crackISState(pcdataXMLEle(ep), &swState) == 0)
-                sp->s = swState;
+                sp->setState(swState);
         }
 
-        if (mediator)
-            mediator->newSwitch(svp);
+        if (d->mediator)
+            d->mediator->newSwitch(svp);
 
         return 0;
     }
     else if (!strcmp(rtag, "setLightVector"))
     {
         IPState lState;
-        ILightVectorProperty *lvp = getLight(name);
+        auto lvp = getLight(name);
 
-        if (lvp == nullptr)
+        if (!lvp)
             return -1;
 
         if (stateSet)
-            lvp->s = state;
+            lvp->setState(state);
 
         for (ep = nextXMLEle(root, 1); ep != nullptr; ep = nextXMLEle(root, 0))
         {
-            ILight *lp = IUFindLight(lvp, findXMLAttValu(ep, "name"));
+            auto lp = lvp->findWidgetByName(findXMLAttValu(ep, "name"));
             if (!lp)
                 continue;
 
             if (crackIPState(pcdataXMLEle(ep), &lState) == 0)
-                lp->s = lState;
+                lp->setState(lState);
         }
 
-        if (mediator)
-            mediator->newLight(lvp);
+        if (d->mediator)
+            d->mediator->newLight(lvp);
 
         return 0;
     }
     else if (!strcmp(rtag, "setBLOBVector"))
     {
-        IBLOBVectorProperty *bvp = getBLOB(name);
+        auto bvp = getBLOB(name);
 
-        if (bvp == nullptr)
+        if (!bvp)
             return -1;
 
         if (stateSet)
-            bvp->s = state;
+            bvp->setState(state);
 
         if (timeoutSet)
-            bvp->timeout = timeout;
+            bvp->setTimeout(timeout);
 
         return setBLOB(bvp, root, errmsg);
     }
@@ -711,6 +701,7 @@ int BaseDevice::setValue(XMLEle *root, char *errmsg)
 */
 int BaseDevice::setBLOB(IBLOBVectorProperty *bvp, XMLEle *root, char *errmsg)
 {
+    D_PTR(BaseDevice);
     /* pull out each name/BLOB pair, decode */
     for (XMLEle *ep = nextXMLEle(root, 1); ep; ep = nextXMLEle(root, 0))
     {
@@ -729,17 +720,16 @@ int BaseDevice::setBLOB(IBLOBVectorProperty *bvp, XMLEle *root, char *errmsg)
                 /* Blob size = 0 when only state changes */
                 if (blobSize == 0)
                 {
-                    if (mediator)
-                        mediator->newBLOB(blobEL);
+                    if (d->mediator)
+                        d->mediator->newBLOB(blobEL);
                     continue;
                 }
 
                 blobEL->size    = blobSize;
-                int bloblen     = pcdatalenXMLEle(ep);
-                int blobBufferSize = 3 * bloblen / 4;
-                if (blobBufferSize != blobEL->bloblen)
-                    blobEL->blob    = static_cast<unsigned char *>(realloc(blobEL->blob, blobBufferSize));
-                blobEL->bloblen = from64tobits_fast(static_cast<char *>(blobEL->blob), pcdataXMLEle(ep), bloblen);
+                uint32_t base64_encoded_size = pcdatalenXMLEle(ep);
+                uint32_t base64_decoded_size = 3 * base64_encoded_size / 4;
+                blobEL->blob    = static_cast<unsigned char *>(realloc(blobEL->blob, base64_decoded_size));
+                blobEL->bloblen = from64tobits_fast(static_cast<char *>(blobEL->blob), pcdataXMLEle(ep), base64_encoded_size);
 
                 strncpy(blobEL->format, valuXMLAtt(fa), MAXINDIFORMAT);
 
@@ -769,8 +759,8 @@ int BaseDevice::setBLOB(IBLOBVectorProperty *bvp, XMLEle *root, char *errmsg)
                     blobEL->blob = dataBuffer;
                 }
 
-                if (mediator)
-                    mediator->newBLOB(blobEL);
+                if (d->mediator)
+                    d->mediator->newBLOB(blobEL);
             }
             else
             {
@@ -786,12 +776,26 @@ int BaseDevice::setBLOB(IBLOBVectorProperty *bvp, XMLEle *root, char *errmsg)
 
 void BaseDevice::setDeviceName(const char *dev)
 {
-    strncpy(deviceID, dev, MAXINDINAME);
+    D_PTR(BaseDevice);
+    d->deviceName = dev;
 }
 
-const char *BaseDevice::getDeviceName()
+const char *BaseDevice::getDeviceName() const
 {
-    return deviceID;
+    D_PTR(const BaseDevice);
+    return d->deviceName.data();
+}
+
+bool BaseDevice::isDeviceNameMatch(const char *otherName) const
+{
+    D_PTR(const BaseDevice);
+    return d->deviceName == otherName;
+}
+
+bool BaseDevice::isDeviceNameMatch(const std::string &otherName) const
+{
+    D_PTR(const BaseDevice);
+    return d->deviceName == otherName;
 }
 
 /* add message to queue
@@ -835,94 +839,177 @@ void BaseDevice::doMessage(XMLEle *msg)
 
 void BaseDevice::addMessage(const std::string &msg)
 {
-    messageLog.push_back(msg);
+    D_PTR(BaseDevice);
+    std::unique_lock<std::mutex> guard(d->m_Lock);
+    d->messageLog.push_back(msg);
+    guard.unlock();
 
-    if (mediator)
-        mediator->newMessage(this, messageLog.size() - 1);
+    if (d->mediator)
+        d->mediator->newMessage(this, d->messageLog.size() - 1);
 }
 
-std::string BaseDevice::messageQueue(int index) const
+const std::string &BaseDevice::messageQueue(size_t index) const
 {
-    if (index >= static_cast<int>(messageLog.size()))
-        return nullptr;
-
-    return messageLog.at(index);
+    D_PTR(const BaseDevice);
+    std::lock_guard<std::mutex> lock(d->m_Lock);
+    assert(index < d->messageLog.size());
+    return d->messageLog.at(index);
 }
 
-std::string BaseDevice::lastMessage()
+const std::string &BaseDevice::lastMessage() const
 {
-    return messageLog.back();
+    D_PTR(const BaseDevice);
+    std::lock_guard<std::mutex> lock(d->m_Lock);
+    assert(d->messageLog.size() != 0);
+    return d->messageLog.back();
 }
 
 void BaseDevice::registerProperty(void *p, INDI_PROPERTY_TYPE type)
 {
+    D_PTR(BaseDevice);
     if (p == nullptr || type == INDI_UNKNOWN)
         return;
 
     const char *name = INDI::Property(p, type).getName();
 
-    INDI::Property *pContainer = getProperty(name, type);
+    auto pContainer = getProperty(name, type);
 
-    if (pContainer != nullptr)
-        pContainer->setRegistered(true);
+    if (pContainer.isValid())
+        pContainer.setRegistered(true);
     else
-        pAll.push_back(new INDI::Property(p, type));
+    {
+        std::lock_guard<std::mutex> lock(d->m_Lock);
+        d->pAll.push_back(INDI::Property(p, type));
+    }
 }
 
-const char *BaseDevice::getDriverName()
+void BaseDevice::registerProperty(INDI::Property &property)
 {
-    ITextVectorProperty *driverInfo = getText("DRIVER_INFO");
+    D_PTR(BaseDevice);
 
-    if (driverInfo == nullptr)
-        return nullptr;
+    if (property.getType() == INDI_UNKNOWN)
+        return;
 
-    IText *driverName = IUFindText(driverInfo, "DRIVER_NAME");
-    if (driverName)
-        return driverName->text;
+    auto pContainer = getProperty(property.getName(), property.getType());
 
-    return nullptr;
+    if (pContainer.isValid())
+        pContainer.setRegistered(true);
+    else
+    {
+        std::lock_guard<std::mutex> lock(d->m_Lock);
+        d->pAll.push_back(property);
+    }
 }
 
-const char *BaseDevice::getDriverExec()
+const char *BaseDevice::getDriverName() const
 {
-    ITextVectorProperty *driverInfo = getText("DRIVER_INFO");
+    auto driverInfo = getText("DRIVER_INFO");
 
-    if (driverInfo == nullptr)
+    if (!driverInfo)
         return nullptr;
 
-    IText *driverExec = IUFindText(driverInfo, "DRIVER_EXEC");
-    if (driverExec)
-        return driverExec->text;
+    auto driverName = driverInfo->findWidgetByName("DRIVER_NAME");
 
-    return nullptr;
+    return driverName ? driverName->getText() : nullptr;
 }
 
-const char *BaseDevice::getDriverVersion()
-{
-    ITextVectorProperty *driverInfo = getText("DRIVER_INFO");
 
-    if (driverInfo == nullptr)
+void BaseDevice::registerProperty(ITextVectorProperty *property)
+{
+    registerProperty(property, INDI_TEXT);
+}
+
+void BaseDevice::registerProperty(INumberVectorProperty *property)
+{
+    registerProperty(property, INDI_NUMBER);
+}
+
+void BaseDevice::registerProperty(ISwitchVectorProperty *property)
+{
+    registerProperty(property, INDI_SWITCH);
+}
+
+void BaseDevice::registerProperty(ILightVectorProperty *property)
+{
+    registerProperty(property, INDI_LIGHT);
+}
+
+void BaseDevice::registerProperty(IBLOBVectorProperty *property)
+{
+    registerProperty(property, INDI_BLOB);
+}
+
+void BaseDevice::registerProperty(PropertyView<IText> *property)
+{
+    registerProperty(static_cast<ITextVectorProperty*>(property));
+}
+
+void BaseDevice::registerProperty(PropertyView<INumber> *property)
+{
+    registerProperty(static_cast<INumberVectorProperty*>(property));
+}
+
+void BaseDevice::registerProperty(PropertyView<ISwitch> *property)
+{
+    registerProperty(static_cast<ISwitchVectorProperty*>(property));
+}
+
+void BaseDevice::registerProperty(PropertyView<ILight> *property)
+{
+    BaseDevice::registerProperty(static_cast<ILightVectorProperty*>(property));
+}
+
+void BaseDevice::registerProperty(PropertyView<IBLOB> *property)
+{
+    BaseDevice::registerProperty(static_cast<IBLOBVectorProperty*>(property));
+}
+
+const char *BaseDevice::getDriverExec() const
+{
+    auto driverInfo = getText("DRIVER_INFO");
+
+    if (!driverInfo)
         return nullptr;
 
-    IText *driverVersion = IUFindText(driverInfo, "DRIVER_VERSION");
-    if (driverVersion)
-        return driverVersion->text;
+    auto driverExec = driverInfo->findWidgetByName("DRIVER_EXEC");
 
-    return nullptr;
+    return driverExec ? driverExec->getText() : nullptr;
+}
+
+const char *BaseDevice::getDriverVersion() const
+{
+    auto driverInfo = getText("DRIVER_INFO");
+
+    if (!driverInfo)
+        return nullptr;
+
+    auto driverVersion = driverInfo->findWidgetByName("DRIVER_VERSION");
+
+    return driverVersion ? driverVersion->getText() : nullptr;
 }
 
 uint16_t BaseDevice::getDriverInterface()
 {
-    ITextVectorProperty *driverInfo = getText("DRIVER_INFO");
+    auto driverInfo = getText("DRIVER_INFO");
 
-    if (driverInfo == nullptr)
+    if (!driverInfo)
         return 0;
 
-    IText *driverInterface = IUFindText(driverInfo, "DRIVER_INTERFACE");
-    if (driverInterface)
-        return atoi(driverInterface->text);
+    auto driverInterface = driverInfo->findWidgetByName("DRIVER_INTERFACE");
 
-    return 0;
+    return driverInterface ? atoi(driverInterface->getText()) : 0;
+}
+
+void BaseDevice::setMediator(INDI::BaseMediator *mediator)
+{
+    D_PTR(BaseDevice);
+    d->mediator = mediator;
+}
+
+INDI::BaseMediator *BaseDevice::getMediator() const
+{
+    D_PTR(const BaseDevice);
+    return d->mediator;
 }
 
 }
