@@ -58,7 +58,7 @@ void CCDChip::setResolution(uint32_t x, uint32_t y)
 
     ImageFrameN[FRAME_W].min = 1;
     ImageFrameN[FRAME_W].max = x;
-    ImageFrameN[FRAME_H].max = 1;
+    ImageFrameN[FRAME_H].min = 1;
     ImageFrameN[FRAME_H].max = y;
     IUUpdateMinMax(&ImageFrameNP);
 }
@@ -164,7 +164,13 @@ void CCDChip::setExposureLeft(double duration)
 {
     ImageExposureNP.s = IPS_BUSY;
     ImageExposureN[0].value = duration;
+    IDSetNumber(&ImageExposureNP, nullptr);
+}
 
+void CCDChip::setExposureComplete()
+{
+    ImageExposureNP.s = IPS_OK;
+    ImageExposureN[0].value = 0;
     IDSetNumber(&ImageExposureNP, nullptr);
 }
 
@@ -274,6 +280,7 @@ void CCDChip::binFrame()
             uint16_t *bin_buf    = reinterpret_cast<uint16_t *>(BinFrame);
             uint16_t *RawFrame16 = reinterpret_cast<uint16_t *>(RawFrame);
             uint16_t val;
+
             for (uint32_t i = 0; i < SubH; i += BinX)
                 for (uint32_t j = 0; j < SubW; j += BinX)
                 {
@@ -304,4 +311,104 @@ void CCDChip::binFrame()
     BinFrame = rawFramePointer;
 }
 
+
+// Thx8411:
+// Binning Bayer frames
+// Each raw frame pixel is mapped and summed onto the binned frame
+// The right place of each pixel in the 2x2 Bayer matrix is found by :
+// (((i/BinX) & 0xFFFFFFFE) + (i & 0x00000001))
+// and
+// ((j/BinX) & 0xFFFFFFFE) + (j & 0x00000001)
+//
+void CCDChip::binBayerFrame()
+{
+    if (BinX == 1)
+        return;
+
+    // Jasem: Keep full frame shadow in memory to enhance performance and just swap frame pointers after operation is complete
+    if (BinFrame == nullptr)
+        BinFrame = new uint8_t[RawFrameSize];
+
+    memset(BinFrame, 0, RawFrameSize);
+
+    switch (getBPP())
+    {
+        // 8 bpp frame
+        case 8:
+        {
+            uint32_t BinFrameOffset;
+            uint32_t val;
+            uint32_t BinW=SubW/BinX;
+            uint8_t BinFactor=BinX*BinY;
+            uint32_t RawOffset=0;
+            uint32_t BinOffsetH;
+
+            // for each raw frame row
+            for (uint32_t i = 0; i < SubH; i++)
+            {
+                // find the binned frame row
+                BinOffsetH=(((i/BinY) & 0xFFFFFFFE) + (i & 0x00000001)) * BinW;
+                // for each raw column
+                for (uint32_t j = 0; j < SubW; j++)
+                {
+                    // find the proper position in the binned frame
+                    BinFrameOffset=BinOffsetH + ((j/BinX) & 0xFFFFFFFE) + (j & 0x00000001);
+                    // get the existing value in the binned frame
+                    val=BinFrame[BinFrameOffset];
+                    // add the new value, averaged and caped
+                    val+=RawFrame[RawOffset]/BinFactor;
+                    if(val>UINT8_MAX)
+                        val=UINT8_MAX;
+                    // write back into the binned frame
+                    BinFrame[BinFrameOffset]=(uint8_t)val;
+                    // next binned frame pixel
+                    RawOffset++;
+                }
+            }
+        }
+        break;
+
+        // 16 bpp frame
+        case 16:
+        {
+            // works the same as the 8 bits version, without averaging but
+            // mapped onto 16 bits pixel
+            uint16_t *RawFrame16 = reinterpret_cast<uint16_t *>(RawFrame);
+            uint16_t *BinFrame16 = reinterpret_cast<uint16_t *>(BinFrame);
+
+            uint32_t BinFrameOffset;
+            uint32_t val;
+            uint32_t BinW=SubW/BinX;
+            uint32_t RawOffset=0;
+            uint32_t BinOffsetH;
+
+            for (uint32_t i = 0; i < SubH; i++)
+            {
+                BinOffsetH=(((i/BinY) & 0xFFFFFFFE) + (i & 0x00000001)) * BinW;
+                for (uint32_t j = 0; j < SubW; j++)
+                {
+                    BinFrameOffset=BinOffsetH + ((j/BinX) & 0xFFFFFFFE) + (j & 0x00000001);
+                    val=BinFrame16[BinFrameOffset];
+                    val+=RawFrame16[RawOffset];
+                    if(val>UINT16_MAX)
+                        val=UINT16_MAX;
+                    BinFrame16[BinFrameOffset]=(uint16_t)val;
+                    RawOffset++;
+                }
+            }
+
+        }
+        break;
+
+        default:
+            return;
+    }
+
+    // Swap frame pointers
+    uint8_t *rawFramePointer = RawFrame;
+    RawFrame                 = BinFrame;
+    // We just memset it next time we use it
+    BinFrame = rawFramePointer;
+}
+
 }
