@@ -103,33 +103,33 @@ bool FocuserInterface::updateProperties()
     if (m_defaultDevice->isConnected())
     {
         //  Now we add our focusser specific stuff
-        m_defaultDevice->defineSwitch(&FocusMotionSP);
+        m_defaultDevice->defineProperty(&FocusMotionSP);
 
         if (HasVariableSpeed())
         {
-            m_defaultDevice->defineNumber(&FocusSpeedNP);
+            m_defaultDevice->defineProperty(&FocusSpeedNP);
 
             // We only define Focus Timer if we can not absolute move
             if (CanAbsMove() == false)
-                m_defaultDevice->defineNumber(&FocusTimerNP);
+                m_defaultDevice->defineProperty(&FocusTimerNP);
         }
         if (CanRelMove())
-            m_defaultDevice->defineNumber(&FocusRelPosNP);
+            m_defaultDevice->defineProperty(&FocusRelPosNP);
         if (CanAbsMove())
         {
-            m_defaultDevice->defineNumber(&FocusAbsPosNP);
-            m_defaultDevice->defineNumber(&FocusMaxPosNP);
+            m_defaultDevice->defineProperty(&FocusAbsPosNP);
+            m_defaultDevice->defineProperty(&FocusMaxPosNP);
         }
         if (CanAbort())
-            m_defaultDevice->defineSwitch(&FocusAbortSP);
+            m_defaultDevice->defineProperty(&FocusAbortSP);
         if (CanSync())
-            m_defaultDevice->defineNumber(&FocusSyncNP);
+            m_defaultDevice->defineProperty(&FocusSyncNP);
         if (CanReverse())
-            m_defaultDevice->defineSwitch(&FocusReverseSP);
+            m_defaultDevice->defineProperty(&FocusReverseSP);
         if (HasBacklash())
         {
-            m_defaultDevice->defineSwitch(&FocusBacklashSP);
-            m_defaultDevice->defineNumber(&FocusBacklashNP);
+            m_defaultDevice->defineProperty(&FocusBacklashSP);
+            m_defaultDevice->defineProperty(&FocusBacklashNP);
         }
     }
     else
@@ -204,6 +204,7 @@ bool FocuserInterface::processNumber(const char * dev, const char * name, double
         {
             FocusSpeedN[0].value = current_speed;
             FocusSpeedNP.s       = IPS_ALERT;
+            m_defaultDevice->saveConfig(true, FocusSpeedNP.name);
         }
 
         //  Update client display
@@ -219,9 +220,14 @@ bool FocuserInterface::processNumber(const char * dev, const char * name, double
         {
             IUUpdateNumber(&FocusMaxPosNP, values, names, n);
 
-            FocusAbsPosN[0].max = FocusSyncN[0].max = FocusMaxPosN[0].value;
-            FocusAbsPosN[0].step = FocusSyncN[0].step = FocusMaxPosN[0].value / 50.0;
-            FocusAbsPosN[0].min = FocusSyncN[0].min = 0;
+            FocusAbsPosN[0].max = FocusMaxPosN[0].value;
+            FocusSyncN[0].max = FocusMaxPosN[0].value;
+
+            FocusAbsPosN[0].step = FocusMaxPosN[0].value / 50.0;
+            FocusSyncN[0].step = FocusMaxPosN[0].value / 50.0;
+
+            FocusAbsPosN[0].min = 0;
+            FocusSyncN[0].min = 0;
 
             FocusRelPosN[0].max  = FocusMaxPosN[0].value / 2;
             FocusRelPosN[0].step = FocusMaxPosN[0].value / 100.0;
@@ -231,6 +237,8 @@ bool FocuserInterface::processNumber(const char * dev, const char * name, double
             IUUpdateMinMax(&FocusRelPosNP);
             IUUpdateMinMax(&FocusSyncNP);
 
+            m_defaultDevice->saveConfig(true, FocusMaxPosNP.name);
+
             FocusMaxPosNP.s = IPS_OK;
         }
         else
@@ -245,7 +253,8 @@ bool FocuserInterface::processNumber(const char * dev, const char * name, double
     {
         if (SyncFocuser(rint(values[0])))
         {
-            FocusSyncN[0].value = FocusAbsPosN[0].value = rint(values[0]);
+            FocusSyncN[0].value = rint(values[0]);
+            FocusAbsPosN[0].value = rint(values[0]);
             FocusSyncNP.s = IPS_OK;
             IDSetNumber(&FocusSyncNP, nullptr);
             IDSetNumber(&FocusAbsPosNP, nullptr);
@@ -276,6 +285,7 @@ bool FocuserInterface::processNumber(const char * dev, const char * name, double
             {
                 FocusBacklashN[0].value = values[0];
                 FocusBacklashNP.s = IPS_OK;
+                m_defaultDevice->saveConfig(true, FocusBacklashNP.name);
             }
             else
                 FocusBacklashNP.s = IPS_ALERT;
@@ -373,7 +383,8 @@ bool FocuserInterface::processNumber(const char * dev, const char * name, double
 
         if ((ret = MoveRelFocuser((FocusMotionS[0].s == ISS_ON ? FOCUS_INWARD : FOCUS_OUTWARD), newPos)) == IPS_OK)
         {
-            FocusRelPosNP.s = FocusAbsPosNP.s = IPS_OK;
+            FocusRelPosNP.s = IPS_OK;
+            FocusAbsPosNP.s = IPS_OK;
             IUUpdateNumber(&FocusRelPosNP, values, names, n);
             IDSetNumber(&FocusRelPosNP, "Focuser moved %d steps %s", newPos,
                         FocusMotionS[0].s == ISS_ON ? "inward" : "outward");
@@ -383,7 +394,8 @@ bool FocuserInterface::processNumber(const char * dev, const char * name, double
         else if (ret == IPS_BUSY)
         {
             IUUpdateNumber(&FocusRelPosNP, values, names, n);
-            FocusRelPosNP.s = FocusAbsPosNP.s = IPS_BUSY;
+            FocusRelPosNP.s = IPS_BUSY;
+            FocusAbsPosNP.s = IPS_BUSY;
             IDSetNumber(&FocusAbsPosNP, "Focuser is moving %d steps %s...", newPos,
                         FocusMotionS[0].s == ISS_ON ? "inward" : "outward");
             IDSetNumber(&FocusAbsPosNP, nullptr);
@@ -442,6 +454,7 @@ bool FocuserInterface::processSwitch(const char * dev, const char * name, ISStat
         {
             IUUpdateSwitch(&FocusBacklashSP, states, names, n);
             FocusBacklashSP.s = IPS_OK;
+            m_defaultDevice->saveConfig(true, FocusBacklashSP.name);
         }
         else
         {
@@ -489,6 +502,7 @@ bool FocuserInterface::processSwitch(const char * dev, const char * name, ISStat
         if (ReverseFocuser(IUFindOnSwitchIndex(&FocusReverseSP) == DefaultDevice::INDI_ENABLED))
         {
             FocusReverseSP.s = IPS_OK;
+            m_defaultDevice->saveConfig(true, FocusReverseSP.name);
         }
         else
         {
